unit blz_BaseControls;

{$mode objfpc}{$H+}

interface

//uses
//   Classes, SysUtils, LResources,LCLProc, LCLType, LCLIntf, Controls ;
uses
{$IFDEF UNIX}
cthreads, cmem,
{$ENDIF}
Classes, SysUtils, Controls, LResources,LCLProc, LCLType, LCLIntf, SyncObjs ;


// Class pour la sécurisation de la lecture et de l'ecriture de données
//Type
// TBZSecuredData=class
//private
//  FIndex: TList; //FIndex:Cardinal;
//  function GetValue: Pointer;
//  procedure SetValue(const Value: Pointer);
//protected
//  property Value:Pointer read GetValue write SetValue;
//public
//  constructor Create;
//  destructor Destroy;override;
//end;
Type
  TBZSecureDataAccess = class// class(TBZSecuredData)
    private
       // Pour securiser l'acces de nos données externes
      FTotalReadLockCount:Integer;
      FUnLockEvent:PRTLEvent;
      FWriteLockCount:Integer;
      FSection:TCriticalSection;
      FValue:Pointer;
    protected
      // Lock Unlock
      property Value:Pointer read FValue write FValue;

    public
      procedure LockRead;virtual;
      procedure LockWrite;virtual;
      procedure UnlockRead;virtual;
      procedure UnlockWrite;virtual;

      property WriteLockCount:Integer read FWriteLockCount;
  end;

Type
  TOnBZPropertyChange = procedure(ASender: TObject; AData: PtrInt) of object;

  { TBCProperty
    Base BC Property with OnChange event support
  }

TBZComponentProperty = class(TPersistent)
 private
   FOnChange: TOnBZPropertyChange;
 protected
   FControl: TComponent;
   procedure Change(AData: PtrInt = 0); virtual;
 public
   constructor Create(AControl: TComponent); virtual;
 public
   property Control: TComponent read FControl;
   property OnChange: TOnBZPropertyChange read FOnChange write FOnChange;
 end;

TBZControlProperty = class(TPersistent)
 private
   FOnChange: TOnBZPropertyChange;
 protected
   FControl: TCustomControl;
   procedure Change(AData: PtrInt = 0); virtual;
 public
   constructor Create(AControl: TCustomControl); virtual;
 public
   property Control: TCustomControl read FControl;
   property OnChange: TOnBZPropertyChange read FOnChange write FOnChange;
 end;


implementation

//==============================================================================
// TBZThreadSecureData
//==============================================================================
//constructor TBZSecuredData.Create;
//begin
//  inherited;
//  FIndexList:=TList.Create;
//  //FIndex:=TlsAlloc;
//  //if FIndex=TLS_OUT_OF_INDEXES then
//  //  RaiseLastOSError;
//end;
//
//destructor TBZSecuredData.Destroy;
//begin
//  //if not TlsFree(FIndex) then
//  //  RaiseLastOSError;
//  FIndexList.free;
//  inherited;
//end;
//
//function TBZSecuredData.GetValue: Pointer;
//var
//  e:Cardinal;
//begin
//  Result :=FIndexList.Items[FCurrentIndex];// TlsGetValue(FIndex);
//  e:=GetLastError;
//  if e<>ERROR_SUCCESS then
//    raise Exception.Create(SysErrorMessage(GetLastError));
//end;
//
//procedure TBZSecuredData.SetValue(const Value: Pointer);
//begin
//  //if not TlsSetValue(FIndex,Value) then
//  //  RaiseLastOSError;
//  FIndexList.
//end;


{**-----------------------------------------------------------------------------
 * @Name        : LockRead
 * @Description : Bloque l'acces d'une donnée externe au thread pour sa lecture
 -----------------------------------------------------------------------------**}
procedure TBZSecureDataAccess.LockRead;
begin
  FSection.Enter;
  try
    //RTLResetEvent(FUnLockEvent);
   // RtlResetEvent(FUnLockEvent);
    Value:=Pointer(Integer(Value)+1);
    InterlockedIncrement(FTotalReadLockCount);
  finally
    FSection.Leave;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : LockRead
 * @Description : Bloque l'acces d'une donnée externe au thread pour son ecriture
 -----------------------------------------------------------------------------**}
procedure TBZSecureDataAccess.LockWrite;
begin
  if Value<>nil then
    raise Exception.Create('LockWrite : Impossible de proteger l''acces en écriture celui-ci est deja protéger en lecture');
  FSection.Enter;
  repeat
    if FTotalReadLockCount=Integer(Value) then
      Break;
    RTLEventWaitFor(FUnLockEvent,INFINITE);
  until False;
  RTLeventResetEvent(FUnLockEvent);
  Inc(FWriteLockCount);
end;

{**-----------------------------------------------------------------------------
 * @Name        : UnLockRead
 * @Description : Debloque l'acces d'une donnée externe en lecture par le thread
 -----------------------------------------------------------------------------**}
procedure TBZSecureDataAccess.UnLockRead;
begin
  Value:=Pointer(Longint(Value)-1);
  if Longint(Value)<0 then
    raise Exception.Create('Compteur Local de la protection des données NEGATIF');
  if InterlockedDecrement(FTotalReadLockCount)<0 then
    raise Exception.Create('Compteur Global de la protection des données NEGATIF');
  if Value=nil then
    RTLeventSetEvent(FUnLockEvent);
end;

{**-----------------------------------------------------------------------------
 * @Name        : UnLockWrite
 * @Description : Debloque l'acces d'une donnée externe en ecriture par le thread
 -----------------------------------------------------------------------------**}
procedure TBZSecureDataAccess.UnLockWrite;
begin
  Dec(FWriteLockCount);
  FSection.Leave;
end;

{ TBZComponentProperty }
constructor TBZComponentProperty.Create(AControl: TComponent);
begin
  FControl := AControl;

  inherited Create;
end;

procedure TBZComponentProperty.Change(AData: PtrInt);
begin
  if Assigned(FOnChange) then
    FOnChange(Self,AData);
end;


constructor TBZControlProperty.Create(AControl: TCustomControl);
begin
  FControl := AControl;

  inherited Create;
end;

procedure TBZControlProperty.Change(AData: PtrInt);
begin
  if Assigned(FOnChange) then
    FOnChange(Self,AData);
end;



//==============================================================================

initialization



//------------------------------------------------------------------------------
finalization

//==============================================================================
end.

