unit blz_customobject;
//==============================================================================
// @Projet : ORUS
// @Unit : OGE_CustomObject
// @Version : 0.0.1
//------------------------------------------------------------------------------
// @Auteur : J.Delauney
// @Licience : GNU/GPL
//------------------------------------------------------------------------------
// @Description :
//  Objets parents et definitions communes
//
//==============================================================================
// @Historique :
//  - 11/2007 : Creation
//
//==============================================================================
// @Commentaire :
//
//------------------------------------------------------------------------------
// @A Faire
//
//==============================================================================
{$mode objfpc}{$H+}
//{$I Orus.inc}
//==============================================================================
interface
{.$Define USELogger}
{$Define USEFASTOBJECT}
uses
  Classes, SysUtils,
  {$IFDEF FPC}
  LCLType, LCLIntf, LCLProc, LResources,
  {$ELSE}
  windows,
  {$ENDIF}

  BLZ_SysUtils, BLZ_LowClasses
  {$IFDEF USELogger}, OGE_LogClasses {$ENDIF}
  ;
  
//---> A Passer dans OGE_LogClass

const
  BLZMemLogFile = 'BLZ_MemoryMonitor.Log';
  
Type
  {$IFDEF USEFASTOBJECT}
  TBLZNotifyEvent = Procedure (Sender : TBLZFastObject) of object;
  TBLZCustomMonitorObject = Class(TBLZFastObject)
  {$Else}
  TBLZNotifyEvent = Procedure (Sender : TObject) of object;
  TBLZCustomMonitorObject = Class(TObject)
  {$Endif}
  Private
    MonitorTag : Integer;

  Public
    constructor Create;override;
    destructor Destroy; override;
      // All descendants of Assign and AssignTo should call inherited if
      // supplied with an unsupported object type
    procedure Assign(Other: TObject); virtual;
    procedure AssignTo(Other: TObject); virtual;
    {$IFDEF USELogger}
    Procedure StartLogTimer;
    Procedure StopLogTimer;
    Procedure LogTimer(msg:string;params);
    Procedure Log(typemsg;msg:string;params);
    Procedure LogAndRaise(typemsg;msg:string;param);
    Procedure AddError;
    {$Endif}

  End;

  TBLZMonitorObject = Class(TBLZCustomMonitorObject)
  {$IFDEF USEFASTOBJECT}
    FOwner : TBLZFastObject;
    {$Else}
    FOwner : TObject;
    {$Endif}
  Public

    constructor Create(AOwner:TBLZFastObject);virtual;

    Property Owner : TBLZFAstObject Read FOwner Write FOwner;
  end;

  TBLZCustomNotifableObject = Class(TBLZCustomMonitorObject)
  Private
    FUpdateCount: Integer;
    FOnChange: TBLZNotifyEvent;
  protected
    property UpdateCount: Integer read FUpdateCount;
  public
    procedure Changed; virtual;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    property OnChange: TBLZNotifyEvent read FOnChange write FOnChange;
  end;

  TBLZCustomActionThreadObject = class(TBLZCustomNotifableObject)
  private
    FLock: TCriticalSection;
    FLockCount: Integer;
  protected
    property LockCount: Integer read FLockCount;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure Lock;
    procedure Unlock;
  end;
  
  
//---> A Passer dans OGE_SysUtils
procedure GetMem(var P; Size: Integer);
function AllocMem(Size: Cardinal): Pointer;
procedure FreeMem(const P; Size: Integer=-1);
procedure FreeAndNilData(var P);

//---> A Passer dans OGE_LogClass
// Retourne La liste des objets actifs et des bloques memoire
function GetMonitorLog(FreeObjects: Boolean=False): TStrings;

implementation

var
  PointerList, ObjectList : TStringList;
  Log : TStringList;
  Sync : TCriticalSection;
  TagNumber : Integer = 0;

procedure FreeMem(const P; Size: Integer=-1);
var
  I : Integer;
begin
  //Sync.Acquire;
  EnterCriticalSection(Sync);
  try
    I:=PointerList.IndexOfObject(TObject(P));
    if I=-1 then
    begin
      if Size<>-1 then Log.Add(Format('Data at $%p allready freed',[Pointer(P)]))
    end
    else PointerList.Delete(I);
  finally
    //Sync.Release;
    LeaveCriticalSection(Sync);
  end;

  if Size<>-1 then System.FreeMem(Pointer(P),Size)
  else System.FreeMem(Pointer(P));
end;

procedure GetMem(var P; Size: Integer);
var
  Addr: TAddr; // Longint ?? -->64bits
begin
  {$ASMMODE intel}
  asm
    mov eax,[rbp+8]  //EAX=RAX ; [EBP+4]=???
    mov Addr,eax
  end;
  System.GetMem(Pointer(P),Size);
  //Sync.Acquire;
  EnterCriticalSection(Sync);
  try
    PointerList.AddObject(IntToHex(Addr,8)+', tag='+IntToStr(TagNumber)+': '+IntToStr(Size),TObject(P));
    Inc(TagNumber);
  finally
    //Sync.Release;
    LeaveCriticalSection(Sync);
  end;
end;

procedure FreeAndNilData(var P);
var
  Ptr : Pointer;
begin
  if Assigned(Pointer(P)) then
  begin
    Ptr:=Pointer(P);
    Pointer(P):=nil;
    FreeMem(Ptr);
  end;
end;

function AllocMem(Size: Cardinal): Pointer;
begin
  GetMem(Result,Size);
 // ZeroMem(Result^,Size);
  FillByte(result,size,0);
end;

function GetMonitorLog(FreeObjects: Boolean): TStrings;
var
  I, Count : Integer;
  Str : string;
begin
  Result:=TStringList.Create;
  //Sync.Acquire;
  EnterCriticalSection(Sync);
  try
    Result.Assign(Log);
    for I:=ObjectList.Count-1 downto 0 do
    try
      Str:=ObjectList[I];
      if FreeObjects then ObjectList.Objects[I].Free;
      Result.Add('At $'+Str+' not freed');
    except
      Result.Add('At $'+ObjectList[I]+' Inherited destructor not called (or project needs rebuilding)');
    end;

    for I:=0 to PointerList.Count-1 do Result.Add('At $'+PointerList[I]+' bytes not freed');
  finally
    //Sync.Release;
    LeaveCriticalSection(Sync);
  end;

  TStringList(Result).Sort;
  I:=0;
  while I<Result.Count-1 do
  begin
    Count:=1;
    while (I<Result.Count-1) and (Result[I]=Result[I+1]) do
    begin
      Result.Delete(I+1);
      Inc(Count);
    end;
    if Count>1 then Result[I]:=Result[I]+' * '+IntToStr(Count);
    Inc(I);
  end;
end;



constructor TBLZCustomMonitorObject.Create;
var
  Addr: TAddr;

begin
  {$ASMMODE intel}
  asm
    mov eax,[rbp+8]
    mov Addr,eax
  end;
  inherited;
 // Sync.Acquire;
  EnterCriticalSection(Sync);
  try
    MonitorTag:=TagNumber;
    Inc(TagNumber);
    ObjectList.AddObject(IntToHex(Addr,8)+', time='+IntToStr(GetTickCount)+', tag='+IntToStr(MonitorTag)+': '+ClassName,Self);
  finally
   // Sync.Release;
    LeaveCriticalSection(Sync);
  end;
end;



destructor TBLZCustomMonitorObject.Destroy;
var
  I : Integer;
begin
 // Sync.Acquire;
  EnterCriticalSection(Sync);
  try
    I:=ObjectList.IndexOfObject(Self);
    if I>=0 then ObjectList.Delete(I)
    else Log.Add(ClassName+' at $'+IntToHex(Self.GetHashCode,8)+' allready freed or inherited constructor/destructor not called')
  finally
   // Sync.Release;
    LeaveCriticalSection(Sync);
  end;
  inherited;
end;



procedure TBLZCustomMonitorObject.Assign(Other: TObject);
begin
 // if Other is TObject then TObject(Other).AssignTo(Self)
 // else raise Exception.Create('Cannot assign '+Other.ClassName+' to '+ClassName);
  
  // Remplace ici Par TBLZLogger --> TErrorManager.raise
end;

procedure TBLZCustomMonitorObject.AssignTo(Other: TObject);
begin
 // if Other is TObject then TObject(Other).Assign(Self)
//  else raise Exception.Create('Cannot assign '+ClassName+' to '+Other.ClassName);
end;


constructor TBLZMonitorObject.Create(AOwner:TBLZFastObject);
begin
  inherited create;
  FOwner := AOwner;
end;

{ TNotifiablePersistent }

procedure TBLZCustomNotifableObject.BeginUpdate;
begin
  Inc(FUpdateCount);
end;

procedure TBLZCustomNotifableObject.Changed;
begin
  if (FUpdateCount = 0) and Assigned(FOnChange) then FOnChange(Self);
end;

procedure TBLZCustomNotifableObject.EndUpdate;
begin
  Assert(FUpdateCount > 0, 'Unpaired TThreadPersistent.EndUpdate');
  Dec(FUpdateCount);
end;


{ TThreadPersistent }

constructor TBLZCustomActionThreadObject.Create;
begin
  InitializeCriticalSection(FLock);
end;

destructor TBLZCustomActionThreadObject.Destroy;
begin
  DeleteCriticalSection(FLock);
  inherited;
end;

procedure TBLZCustomActionThreadObject.Lock;
begin
  InterlockedIncrement(FLockCount);
  EnterCriticalSection(FLock);
end;

procedure TBLZCustomActionThreadObject.Unlock;
begin
  LeaveCriticalSection(FLock);
  InterlockedDecrement(FLockCount);
end;


initialization

  ObjectList:=TStringList.Create;
  ObjectList.Capacity:=200;

  PointerList:=TStringList.Create;
  PointerList.Capacity:=200;

  Log:=TStringList.Create;

  //Sync:=TCriticalSection.Create;
  InitializeCriticalSection(Sync);

finalization

  with GetMonitorLog(True) do
  begin
    if (Count>0) or (FileExists(BLZMemLogFile)) then
    try
      SaveToFile(BLZMemLogFile);
    except
    end;
		
	if (Count>0) then
      begin
    
      try
        Log.Add('ATTENTION !, Le Monitor a detecte des fuites de memoire. ');
        Log.Add('Terminating Application - Finalization' );
        Log.SaveToFile(BLZMemLogFile);
        // MessageBox(0,PChar(Text),'Le Monitor a detecter des fuites de memoire ! ',MB_OK or MB_ICONWARNING);
      except
      end;
    
    end
    else
    Begin
    
      Log.Add('FELICITATION !, Le Monitor n''a pas detecte de fuites de memoire. ');
      Log.Add('Terminating Application - Finalization' );
      Log.SaveToFile(BLZMemLogFile);
    
    End;	
    //if Count>0 then
      //MessageBox(0,PChar(Text),'Monitor has detected a memory leak',MB_OK or MB_ICONWARNING);
    Free;
  end;
  //Sync.Free;
  DeleteCriticalSection(Sync);
  Log.Free;
  PointerList.Free;
  ObjectList.Free;
end.

