unit bz_FormEx;
//==============================================================================
// @NAME    : BZFormEx
// @VERSION : 0.7.4
// @DATE    : 22/10/2015
// @UPDATED : 22/10/2015
// @PACKAGE : BEANZ
// @DESC    : TBZFormEx est un composant non visuel ajoutant quelques possibilités aux Forms comme :
//            - La magnetisation sur les bord de l'ecran
//            - Le snapping des "form" enfants sur la form principal
//            - Connaitre la position de la souris à l'écran
//            - Prendre une photo du contenu de la fenêtre
//
// @AUTHOR  : J.DELAUNEY Aka BeanzMaster
// @LICENCE : WTFPL
//------------------------------------------------------------------------------
// @NOTE    : Ce composant à uniquement été tester sous environnement WINDOWS 7
//
//------------------------------------------------------------------------------
// @TODO    :
// - Sauvegarde et restauration de la position et dimensions des fenêtres
// - Magnetisation des fenêtre enfant sur les bord de l'écran
// - Gestion du redimensionnement auto des fenêtres
//==============================================================================

{$mode objfpc}{$H+}

interface

//==============================================================================

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs,
  bz_ThreadTimer;

//==============================================================================

//==============================================================================
// Définitions des paramètres des propriétés pour notre composant
type
  TActiveBorders      = set of (abLeft, abTop, abRight, abBottom);
  TSnappingTargets    = set of (stLeft, stTop, stRight, stBottom);
  TSnappingTarget     = (stToNone,stToLeft, stToTop, stToRight, stToBottom);
  //TActiveMovingArea   = (amClient, amTitleBar);

const
  stAll = [stLeft, stTop, stRight, stBottom];
  abAll = [abLeft, abTop, abRight, abBottom];
 // FormPropKeywords : array of string = ['left','top','right','bottom','min-height','min-width','max-height','max-width','visible','snap','snapto'];
const
  MAXWNDS = 19;

Type TFormChildRec = record
  name:String;
  aform : TForm;
  SnapTo: TSnappingTarget;
end;

//==============================================================================
//==== [ TBZFormEx ]============================================================
// @Desc : Composant ajoutont des fonctions aux fenêtres
//==============================================================================
type
  TBZFormEx = class(TComponent)
  private
    { Private declarations }
    FProcessForm     : TForm;
    FChildForms      : Array[0..MAXWNDS] of TFormChildRec;
    //FChildsForm        : TList;
    FChildFormsCount : Byte;

    FTimer           : TBZThreadTimer;
    //FPropsStorage    : TXMstorage;

    FActive          : Boolean;

    FMouseX,
    FMouseY : Integer;

    FScreenMagnet    : Boolean;  // Colle la fenetre aux bords de l'ecran
    FMagneticScreen  : TActiveBorders;
    FMagnetOnScreen  : TSnappingTargets;

    //-------------------------------------------------------
    FSnapping : Boolean;  // Colle les fenetres enfants a la fenetre principale
    FSnappingBorders : TActiveBorders;
    // FChildDockable : Boolean;  // Les fenetres enfants peuvent être attachés un control (panel,scrollbox) de la fenetre principale

    //-------------------------------------------------------
    FMagneticField : cardinal; // default 15px;

    //-------------------------------------------------------

    FOldFormWidth    : Integer;
    FOldFormHeight   : Integer;
    FOldFormPos      : TPoint;

    //-------------------------------------------------------
    FActiveMoving    : Boolean;  // Possiblité a la fenetre de bouger
    FActiveSizing    : Boolean;  // Possiblité de redimensionner la fenetre
    //FSizingBorders   : TActiveBorders;
    // FAutoResizing    : Boolean;  // Redimensionnement de la fenetre et de ces composants automatiquement

    //-------------------------------------------------------
    //FAutoLayout      : Boolean;   // Sauve et restore la position et dimension des fenetres automatiquement

    //-------------------------------------------------------
    //FExtendMove   : Boolean;  // Pour deplacer la fenetre en selectionnant son contenu

    //-------------------------------------------------------
    FFullScreen    : Boolean;
    //-------------------------------------------------------



    procedure SetActive(AValue:Boolean);
    procedure SetFullScreen(AValue:Boolean);
    procedure SetActiveMoving(AValue:Boolean);
    procedure SetActiveSizing(AValue:Boolean);
    procedure SetSnapping(AValue:Boolean);
    //procedure SetChildDockable(AValue:Boolean);

  protected
    { Protected declarations }
    procedure ProcessScreenMagnet();
    procedure ProcessFormSnapping();
    procedure ProcessMovingSizing();

    procedure DoProcess(Sender:TObject);

    procedure Loaded;override;

  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;

    procedure addChildForm(AName:string;AForm:TForm);
    function getChildForm(AName:String;var Index:Integer):TForm;
    function getChildForm(AName:String):TForm;overload;
    function getChildFormIndex(AName:String):Integer;

    procedure MagnetToScreen(ATarget: TSnappingTarget);
    //procedure MagnetChildToScreen(AName:String;ATarget: TSnappingTarget);

    procedure SnapChildToLeft(AName:String);
    procedure SnapChildToTop(AName:String);
    procedure SnapChildToRight(AName:String);
    procedure SnapChildToBottom(AName:String);
    procedure SnapChildTo(AName:String;ATarget: TSnappingTarget);
    procedure UnSnapChild(AName:String);

    procedure EnterFullScreen;
    procedure ExitFullScreen;

 //   procedure DockChildFormTo(AName:String,ATargetName: string);
 //   procedure UnDockChildForm(AName:String);

    procedure ShowForm(AName:String;Fade:Boolean);
    procedure HideForm(AName:String;Fade:Boolean);

    //procedure SaveLayout();
    //procedure RestoreLayout();
    function TakeFormScreenShot:TBitmap;
    function TakeChildFormScreenShot(AName:String):TBitmap;

    procedure Start;
    procedure Stop;

  published
    { Published declarations }
    property Active            : Boolean           read FActive          write setActive;

    property ScreenMagnet      : Boolean           read FScreenMagnet    write FScreenMagnet;
    property MagnetOnScreen    : TSnappingTargets  read FMagnetOnScreen  write FMagnetOnScreen;

    property SnappingBorders   : TActiveBorders    Read FSnappingBorders Write FSnappingBorders;
    property MagneticScreen    : TActiveBorders    Read FMagneticScreen  Write FMagneticScreen;

    property Snapping          : Boolean           read FSnapping        write FSnapping;

    property MagneticField     : Cardinal          read FMagneticField   write FMagneticField;

    property ActiveMoving      : Boolean           read FActiveMoving    write SetActiveMoving;
    property ActiveSizing      : Boolean           read FActiveSizing    write SetActiveSizing;

    property FullScreen        : Boolean           read FFullScreen      Write SetFullScreen;

    //property ActiveSizing      : Boolean           read FActiveSizing      write FActiveSizing    default True;
    //property AutoResizing      : Boolean  read FAutoResizing      write FAutoResizing    default False;
    //property AutoLayout        : Boolean  read FAutoLayout        write FAutoLayout      default False;

  public
    property MousePosX : Integer Read FMouseX;
    property MousePosY : Integer Read FMouseY;
  end;

//==============================================================================
procedure Register;

//==============================================================================
implementation

//==============================================================================

{**-----------------------------------------------------------------------------
 * @Name        :  Register
 * @Description :  Enregistre notre composant dans l'IDE
 -----------------------------------------------------------------------------**}
procedure Register;
begin
  {$I ../../ressources/bz_formex_icon.lrs}
  RegisterComponents('Beanz',[TBZFormEx]);
end;

//==============================================================================

//==============================================================================
//==============================================================================
// 1. TBZFormEx
//==============================================================================

//==== PRIVATE =================================================================

{**------------------------------------------------------------------------------
 * Setters pour les differentes propriétés
------------------------------------------------------------------------------**}
procedure TBZFormEx.setActive(AValue:Boolean);
begin
  if FActive = AValue then exit;
  FActive:=AValue;
  if not (csDesigning in ComponentState)  then
  begin
     Start;
  end;
end;

procedure TBZFormEx.SetFullScreen(AValue:Boolean);
begin
  if FFullScreen = AValue then exit;
  FFullScreen := AValue;
  if FFullScreen then EnterFullScreen else ExitFullScreen;
end;

procedure TBZFormEx.SetActiveMoving(AValue:Boolean);
begin
  if FActiveMoving = AValue then exit;
  FActiveMoving := AValue;
  if not(FActiveMoving) then
  begin
    FOldFormPos:=Point(FProcessForm.Left,FProcessForm.Top);
  end;
end;

procedure TBZFormEx.SetActiveSizing(AValue:Boolean);
begin
  if FActiveSizing = AValue then exit;
  FActiveSizing := AValue;
  if not(FActiveSizing) then
  begin
    FOldFormWidth := FProcessForm.Width;
    FOldFormHeight := FProcessForm.Height;
  end;
end;

procedure TBZFormEx.SetSnapping(AValue:Boolean);
begin
  if FSnapping = AValue then exit;
  FSnapping := AValue;
end;

//==== PROTECTED ===============================================================

{**-----------------------------------------------------------------------------
 * @Name        : Loaded
 * @Description :
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.Loaded;
begin
  if not (csDesigning in ComponentState) and Active  then
  begin
    FOldFormHeight := FProcessForm.Height;
    FOldFormWidth  := FProcessForm.Width;
    FOldFormPos.x  := FProcessForm.Left;
    FOldFormPos.y  := FProcessForm.Top;
    Start;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : ProcessScreenMagnet
 * @Description : Gestion du magnetisme au bords de l'écran
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.ProcessScreenMagnet();
var
    wWidth,wHeight:Integer;
    x,y,nx,ny:integer;
begin
  If ActiveMoving then
  begin
    If (FScreenMagnet) and (FProcessForm.WindowState = wsNormal)  then
    begin
      wWidth := FProcessForm.Width;
      wHeight := FProcessForm.Height;
      nx:= FProcessForm.Left;
      ny:= FProcessForm.Top;
      x:=nx;
      y:=ny;
      if (stTop in MagnetOnScreen) and (abTop in MagneticScreen) then
      begin
        if y <= MagneticField then ny:= 2;
      end;
      if (stLeft in MagnetOnScreen) and (abLeft in MagneticScreen) then
      begin
        if x <= MagneticField then nx := 2;
      end;
      if (stBottom in MagnetOnScreen) and (abBottom in MagneticScreen) then
      begin
        if (y+wHeight) >= (Screen.Height-78-MagneticField) then ny := Screen.Height-wHeight-78;
      end;
      if (stRight in MagnetOnScreen) and (abRight in MagneticScreen) then
      begin
        if (x+wWidth) >= (Screen.Width-MagneticField) then nx := Screen.Width-wWidth-4;
      end;
      FProcessForm.Top:=ny;
      FProcessForm.Left:=nx;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : ProcessFormSnapping
 * @Description : Gestion du "snapping" des fenêtres enfant
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.ProcessFormSnapping();
var I:integer;
    fx,fy,fw,fh : integer;
begin
  { A la place d'utiliser un tableau on peut faire :

   if Assigned(FProcessForm) then begin
    for I := 0 to Application.ComponentCount - 1 do begin
      if Application.Components[I] is TForm then begin
       // with Application.Components[I] as TForm do begin
   }

  if FChildFormsCount>0 then
  begin
    for I:=0 to FChildFormsCount-1 do
    begin
      if FChildForms[i].AForm.Visible then
      begin
        If Snapping and (FProcessForm.WindowState = wsNormal)  then
        begin
          fx := FChildForms[i].AForm.Left;
          fy := FChildForms[i].AForm.Top;
          fw := FChildForms[i].AForm.Width;
          fh := FChildForms[i].AForm.Height;
          FChildForms[i].SnapTo:=stToNone;
          if (abTop in SnappingBorders)  then
          begin
            if  ( ( ((fx+fw) <= (FOldFormPos.X+FOldFormWidth)) and ((fx+fw) >= (FOldFormPos.X)) ) or ( (fx >= FOldFormPos.X) and (fx <= FOldFormPos.X+FOldFormWidth) ) ) then
              if ((fy+fh)>=FOldFormPos.Y-MagneticField) and ((fy+fh)<=FOldFormPos.Y+MagneticField) then
              begin
                FChildForms[i].AForm.Left := fx;
                FChildForms[i].AForm.Top  := FOldFormPos.y-fh-34; // le -34 corresponspond à la taille de la barre de titre (24+2 ou 2 est la marge) + la taille de la bordure des fenêtres (4*2)
                FChildForms[i].SnapTo:=stToTop;
              end;
              //else FChildForms[i].SnapTo:=stToNone;
          end;

          if (abBottom in SnappingBorders) then
          begin
            if  ( ( ((fx+fw) <= (FOldFormPos.X+FOldFormWidth)) and ((fx+fw) >= (FOldFormPos.X)) ) or ( (fx >= FOldFormPos.X) and (fx <= FOldFormPos.X+FOldFormWidth) ) ) then
              if (fy<=(FOldFormPos.Y+FOldFormHeight)+MagneticField) and (fy>=(FOldFormPos.Y+FOldFormHeight)-MagneticField) then
              begin
                FChildForms[i].AForm.Left := fx;
                FChildForms[i].AForm.Top  := FOldFormPos.y+FOldFormHeight+34;
                FChildForms[i].SnapTo:=stToBottom;
              end;
              //else FChildForms[i].SnapTo:=stToNone;
          end;

          if (abLeft in SnappingBorders)  then
          begin
            if ((fy<=(FOldFormPos.Y+FOldFormHeight)) and (fy>=FOldFormPos.Y)) or (((fy+fh)<=(FOldFormPos.Y+FOldFormHeight)) and ((fy+fh)>=FOldFormPos.Y)) then
              if  ( ( (fx+fw) <= (FOldFormPos.X+MagneticField)) and ( (fx+fw) >= (FOldFormPos.X-MagneticField))) then
              begin
                FChildForms[i].AForm.Left := FOldFormPos.x-fw-8;
                FChildForms[i].AForm.Top  := fy;
                FChildForms[i].SnapTo:=stToLeft;
              end;
              //else FChildForms[i].SnapTo:=stToNone;
          end;

          if (abRight in SnappingBorders) then
          begin
            if ((fy<=(FOldFormPos.Y+FOldFormHeight)) and (fy>=FOldFormPos.Y)) or (((fy+fh)<=(FOldFormPos.Y+FOldFormHeight)) and ((fy+fh)>=FOldFormPos.Y)) then
              if ( (fx >= ((FOldFormPos.X+FOldFormWidth)-MagneticField)) and (fx <= ((FOldFormPos.X+FOldFormWidth)+MagneticField))) then
              begin
                FChildForms[i].AForm.Left := FOldFormPos.x+FOldFormWidth+8;
                FChildForms[i].AForm.Top  := fy;
                FChildForms[i].SnapTo:=stToRight;
              end;
              //else FChildForms[i].SnapTo:=stToNone;
          end;
        end;
      end;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : ProcessMoving
 * @Description : Gestion du déplacement de la fenêtre
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.ProcessMovingSizing();
var dx, dy,I :integer;
begin
  if not(ActiveMoving) then
  begin
    FProcessForm.Left := FOldFormPos.X;
    FProcessForm.Top  := FOldFormPos.Y;
  end;

  if not(ActiveSizing) then
  begin
    FProcessForm.Width := FOldFormWidth;
    FProcessForm.Height:= FOldFormHeight;
  end;

  if (ActiveSizing) or (ActiveMoving) then
  begin
    if FChildFormsCount>0 then
    begin
      for I:=0 to FChildFormsCount-1 do
      begin
        If Snapping and (FProcessForm.WindowState = wsNormal) and (FChildForms[i].AForm.Visible) then
           if (FChildForms[i].SnapTo<>stToNone) then
           begin
             dx := FOldFormPos.X - FProcessForm.Left;
             dy := FOldFormPos.Y - FProcessForm.Top;
             FChildForms[i].AForm.Left := FChildForms[i].AForm.Left - dx;
             FChildForms[i].AForm.Top := FChildForms[i].AForm.Top - dy;
           end;
      end;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : DoProcess
 * @Description : Gestion des êvenments dans le timer
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.DoProcess(Sender:TObject);
begin
  FMouseX:=Mouse.CursorPos.x;
  FMouseY:=Mouse.CursorPos.Y;

  ProcessScreenMagnet;
  ProcessFormSnapping;
  ProcessMovingSizing;

  if (ActiveSizing) then
  begin
    FOldFormHeight := FProcessForm.Height;
    FOldFormWidth  := FProcessForm.Width;
  end;
  FOldFormPos.x  := FProcessForm.Left;
  FOldFormPos.y  := FProcessForm.Top;
end;

//==== PUBLIC ==================================================================

{**-----------------------------------------------------------------------------
 * @Name        : Create
 * @Description : Creation de notre bouton personnalisé
 -----------------------------------------------------------------------------**}
constructor TBZFormEx.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FProcessForm   := TForm(AOwner);

  FActive        := False;

  FActiveMoving  := True;
  FActiveSizing  := True;

  FScreenMagnet   := True;
  FMagneticField  := 15;
  FMagnetOnScreen := stAll;
  FMagneticScreen := abAll;

  FSnapping        := True;
  FSnappingBorders := abAll;

  FChildFormsCount := 0;

  FTimer := TBZThreadTimer.Create(self);
  FTimer.Enabled     := False;
  FTimer.KeepAlive   := True;
  FTimer.Synchronize := True;
  FTimer.Accurate    := True;
  FTimer.Interval    := 100;
  FTimer.OnTimer     := @DoProcess;
 // FMovingBorders := mbAllBorders;
 //  FSizingBorders := abAll;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Destroy
 * @Description : Destruction de notre composant
 -----------------------------------------------------------------------------**}
destructor TBZFormEx.Destroy;
begin
  Stop;
  FTimer.Free;
end;

{**-----------------------------------------------------------------------------
 * @Name        : addChildForm
 * @Description : Ajoute une fenêtre enfant à gérer dans la liste
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.addChildForm(AName:string;AForm:TForm);
begin
  inc(FChildFormsCount);
  FChildForms[FChildFormsCount-1].Name            := AName;
  FChildForms[FChildFormsCount-1].AForm           := AForm;
  FChildForms[FChildFormsCount-1].SnapTo          := stToNone;
end;

{**-----------------------------------------------------------------------------
 * @Name        : getChildForm
 * @Description : Retourne la fenêtre enfant et son Index de la liste
 -----------------------------------------------------------------------------**}
function TBZFormEx.getChildForm(AName:String;var Index:Integer):TForm;
var I:integer;
begin
  Result := nil;
  Index:=-1;
  if FChildFormsCount>0 then
  begin
    for I:=0 to FChildFormsCount-1 do
    begin
      if FChildForms[I].Name = AName then
      begin
        Result :=FChildForms[i].AForm;
        Index := I;
        Break;
      end;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : getChildForm
 * @Description : Retourne la fenêtre enfant
 -----------------------------------------------------------------------------**}
function TBZFormEx.getChildForm(AName:String):TForm;
var I:integer;
begin
  Result := nil;
  if FChildFormsCount>0 then
  begin
    for I:=0 to FChildFormsCount-1 do
    begin
      if FChildForms[I].Name = AName then
      begin
        Result :=FChildForms[i].AForm;
        Break;
      end;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : getChildForm
 * @Description : Retourne l'index de la fenêtre enfant
 -----------------------------------------------------------------------------**}
function TBZFormEx.getChildFormIndex(AName:String):Integer;
var I:integer;
begin
  Result := -1;
  if FChildFormsCount>0 then
  begin
    for I:=0 to FChildFormsCount-1 do
    begin
      if FChildForms[I].Name = AName then
      begin
        Result := I;
        Break;
      end;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : MagnetToScreen
 * @Description : Magnetise notre fenêtre sur les bords de l'écran
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.MagnetToScreen(ATarget: TSnappingTarget);
var
    wWidth,wHeight:Integer;
    x,y,nx,ny:integer;
begin
  If (ScreenMagnet) and (FProcessForm.WindowState = wsNormal)  then
  begin
    //CurrentRect:=FProcessForm.BoundsRect;
    wWidth := FProcessForm.Width; //CurrentRect.Right - CurrentRect.Left;
    wHeight := FProcessForm.Height; //CurrentRect.Bottom - CurrentRect.Top;
    nx:= FProcessForm.Left;
    ny:= FProcessForm.Top;
    x:=nx;
    y:=ny;
    Case ATarget of
      stToTop :
      begin
        if (stTop in MagnetOnScreen) and (abTop in MagneticScreen) then
        begin
          if y <= MagneticField then ny:= 2;
        end;
      end;
      stToLeft :
      Begin
        if (stLeft in MagnetOnScreen) and (abLeft in MagneticScreen) then
        begin
          if x <= MagneticField then nx := 2;
        end;
      end;
      stToBottom :
      begin
        if (stBottom in MagnetOnScreen) and (abBottom in MagneticScreen) then
        begin
          if (y+wHeight) >= (Screen.Height-78-MagneticField) then ny := Screen.Height-wHeight-78;
        end;
      end;
      stToRight :
      begin
        if (stRight in MagnetOnScreen) and (abRight in MagneticScreen) then
        begin
          if (x+wWidth) >= (Screen.Width-MagneticField) then nx := Screen.Width-wWidth-4;
        end;
      end;
      stToNone :
      begin

      end;
    end;
    FProcessForm.Top:=ny;
    FProcessForm.Left:=nx;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : SnapChildTo
 * @Description : Attache une fenêtre enfant sur un bord de notre fenêtre principale
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.SnapChildTo(AName:String;ATarget: TSnappingTarget);
var fx,fy,fh,fw,i:integer;
begin
  I:= getChildFormIndex(AName);
  if I>-1 then
  begin
    fx := FChildForms[i].AForm.Left;
    fy := FChildForms[i].AForm.Top;
    fw := FChildForms[i].AForm.Width;
    fh := FChildForms[i].AForm.Height;
    Case ATarget of
      stToTop:
      begin
        FChildForms[i].AForm.Left := FOldFormPos.x;
        FChildForms[i].AForm.Top  := FOldFormPos.y-fh-34; // le -34 corresponspond à la taille de la barre de titre (24+2 ou 2 est la marge) + la taille de la bordure des fenêtres (4*2)
        FChildForms[i].SnapTo:=stToTop;
      end;
      stToBottom :
      begin
        FChildForms[i].AForm.Left := FOldFormPos.x;
        FChildForms[i].AForm.Top  := FOldFormPos.y+FOldFormHeight+34;
        FChildForms[i].SnapTo:=stToBottom;
      end;
      stToLeft :
      Begin
        FChildForms[i].AForm.Left := FOldFormPos.x-fw-8;
        FChildForms[i].AForm.Top  := FOldFormPos.y;
        FChildForms[i].SnapTo:=stToLeft;
      end;
      stToRight :
      begin
        FChildForms[i].AForm.Left := FOldFormPos.x+FOldFormWidth+8;
        FChildForms[i].AForm.Top  := FOldFormPos.y;
        FChildForms[i].SnapTo:=stToRight;
      end;
      stToNone : FChildForms[i].SnapTo:=stToNone;
    end;

  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : SnapChildToLeft
 * @Description : Attache une fenêtre enfant sur le bord Gauche de notre fenêtre principale
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.SnapChildToLeft(AName:String);
begin
  SnapChildTo(AName,stToLeft);
end;

{**-----------------------------------------------------------------------------
 * @Name        : SnapChildToTop
 * @Description : Attache une fenêtre enfant sur le bord Haut de notre fenêtre principale
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.SnapChildToTop(AName:String);
begin
  SnapChildTo(AName,stToTop);
end;

{**-----------------------------------------------------------------------------
 * @Name        : SnapChildToRight
 * @Description : Attache une fenêtre enfant sur le bord Droit de notre fenêtre principale
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.SnapChildToRight(AName:String);
begin
  SnapChildTo(AName,stToRight);
end;

{**-----------------------------------------------------------------------------
 * @Name        : SnapChildToBottom
 * @Description : Attache une fenêtre enfant sur le bord Bas de notre fenêtre principale
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.SnapChildToBottom(AName:String);
begin
  SnapChildTo(AName,stToBottom);
end;

{**-----------------------------------------------------------------------------
 * @Name        : UnSnapChildToBottom
 * @Description : Déttache une fenêtre enfant d'un des bords de notre fenêtre principale
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.UnSnapChild(AName:String);
begin
  SnapChildTo(AName,stToNone);
end;


{**-----------------------------------------------------------------------------
 * @Name        : MagnetChildToScreen
 * @Description : Magnetise une fenêtre enfant sur les bords de l'écran
 -----------------------------------------------------------------------------**}
//procedure TBZFormEx.MagnetChildToScreen(AName:String;ATarget: TSnappingTarget);
//begin
//
//end;

{**-----------------------------------------------------------------------------
 * @Name        : EnterFullScreen
 * @Description : Passe en mode plein écran
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.EnterFullScreen;
begin
 FProcessForm.BorderStyle:=bsNone;
 FProcessForm.FormStyle:=fsSystemStayOnTop;
 FProcessForm.WindowState:=wsMaximized; //wsFullScreen;
 FFullScreen:=True;
end;

{**-----------------------------------------------------------------------------
 * @Name        : ExitFullScreen
 * @Description : Sort du mode plein ecran
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.ExitFullScreen;
begin
  FProcessForm.WindowState:=wsNormal;
  FProcessForm.Position:=poDesigned;
  FProcessForm.FormStyle:=fsNormal;
  FProcessForm.BorderStyle:=bsSizeable;
  FFullScreen:=False;
end;

{**-----------------------------------------------------------------------------
 * @Name        : ShowForm
 * @Description : Affiche une fenêtre enfant avec un effet de fading
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.ShowForm(AName:String;Fade:Boolean);
var
  i: Integer;
  AForm:TForm;
begin
  AForm:=getChildForm(AName);
  if Fade then
  begin
    AForm.AlphaBlend:=True;
    AForm.Show;
    for i := 1 to 51 do begin
      AForm.AlphaBlendValue:=i*5;
      Sleep(10);
    end;
    AForm.AlphaBlend:=False;
  end
  else AForm.Show;
end;

{**-----------------------------------------------------------------------------
 * @Name        : ShowForm
 * @Description : Cache une fenêtre enfant avec un effet de fading
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.HideForm(AName:String;Fade:Boolean);
var
  i: Integer;
  AForm:TForm;
begin
  AForm:=getChildForm(AName);
  if Fade then
  begin
    AForm.AlphaBlend:=True;
    for i := 51 downto 1 do begin
      AForm.AlphaBlendValue:=i*5;
      Sleep(10);
    end;
    AForm.Hide;
    AForm.AlphaBlend:=False;
  end
  else AForm.Hide;
end;

{**-----------------------------------------------------------------------------
 * @Name        : TakeFormScreenShot
 * @Description : Capture de la fenetre vers un Bitmap
 -----------------------------------------------------------------------------**}
function TBZFormEx.TakeFormScreenShot:TBitmap;
begin
  Result := FProcessForm.GetFormImage;
end;

{**-----------------------------------------------------------------------------
 * @Name        : TakeFormScreenShot
 * @Description : Capture d'une fen^tre enfant vers un Bitmap
 -----------------------------------------------------------------------------**}
function TBZFormEx.TakeChildFormScreenShot(AName:String):TBitmap;
Var
  AForm : TForm;
begin
  Result:=nil;
  AForm:=getChildForm(AName);
  if AForm.Visible then Result := AForm.GetFormImage;
end;


{**-----------------------------------------------------------------------------
 * @Name        : Start
 * @Description : Débute la gestion de nos évênements
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.Start;
begin
  FTimer.Enabled:=True;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Stop
 * @Description : Arrête la gestion de nos évênements
 -----------------------------------------------------------------------------**}
procedure TBZFormEx.Stop;
begin
  FTimer.Enabled:=False;
end;
//==============================================================================
//==============================================================================

//initialization

//------------------------------------------------------------------------------
//finalization

//==============================================================================
end.
