unit bz_ThreadTimer;
//==============================================================================
// @NAME    : TBZThreadTimer
// @VERSION : 0.7.1
// @DATE    : 28/09/2015
// @UPDATED : 22/10/2015
// @PACKAGE : BEANZ
// @DESC    : TBZThreadTimer est un Timer qui s'execute dans Thread
//            Ce composant est la conversion du composant TJvThreadTimer de la librairie JVCL
// @AUTHOR  : J.DELAUNEY Aka BeanzMaster
// @LICENCE :
//------------------------------------------------------------------------------
// @NOTE    : Ce composant à uniquement été tester sous environnement WINDOWS 7
//
//------------------------------------------------------------------------------
// @TODO    :
//==============================================================================

{$mode objfpc}{$H+}

interface

//==============================================================================
uses
  {$IFDEF UNIX}
  cthreads, cmem,
  {$ENDIF}
  Classes, SysUtils, LResources,LCLProc, LCLType, LCLIntf,
  Forms, Controls, Graphics, Dialogs;

//==============================================================================
//==============================================================================
//==== [ TBZThreadTimer ]=======================================================
// @Desc : Notre Composant TBZThreadTimer
//==============================================================================
type
 TBZThreadTimer = class(TComponent)
  private
    { Private declarations }
       FEnabled: Boolean;
       FInterval: Cardinal;
       FKeepAlive: Boolean;
       FOnTimer: TNotifyEvent;

       FSynchronize : Boolean;
       FUseCriticalSection:Boolean;
       FAccurate : Boolean;
       //FUseWaitfor:boolean;

       FStreamedEnabled: Boolean;
       FThread: TThread;
       {$IFDEF WINDOWS}
        FPriority: TThreadPriority;
       {$ENDIF WINDOWS}
       procedure SetEnabled(const Value: Boolean);
       procedure SetInterval(const Value: Cardinal);
       procedure SetSynchronize(const Value: Boolean);
       //procedure SetUseCriticalSection(const Value: Boolean);
       procedure SetOnTimer(const Value: TNotifyEvent);
       {$IFDEF WINDOWS}
       procedure SetPriority(const Value: TThreadPriority);
       {$ENDIF WINDOWS}
       procedure SetKeepAlive(const Value: Boolean);
     protected
       procedure DoOnTimer;
       procedure Loaded; override;
       procedure StopTimer;
       procedure UpdateTimer;
     public
       constructor Create(AOwner: TComponent); override;
       destructor Destroy; override;
       property Thread: TThread read FThread;
     published

       property Enabled: Boolean      Read FEnabled     Write SetEnabled default False;
       property Interval: Cardinal    Read FInterval    Write SetInterval default 1000;
       property KeepAlive: Boolean    Read FKeepAlive   Write SetKeepAlive default False;
       property Synchronize: Boolean  Read FSynchronize write SetSynchronize default False;
       property Accurate : Boolean    Read FAccurate    Write FAccurate default false;
       {$IFDEF WINDOWS}
       property Priority: TThreadPriority read FPriority write SetPriority default tpNormal;
       {$ENDIF WINDOWS}

       property OnTimer: TNotifyEvent Read FOnTimer     Write SetOnTimer;


     end;

//==============================================================================
procedure Register;
//==============================================================================

implementation

//==============================================================================
//uses SyncObjs;
//==============================================================================

//==============================================================================
//==== [ TBZTimerThread ]=======================================================
// @Desc : Le Thread ou l'on va executer notre Timer
//==============================================================================
type
  TBZTimerThread = class(TThread)
  private
    FEvent: PRtlEvent; //Delphi = THandle;
    FHasBeenSuspended: Boolean;
    FInterval: Cardinal;
    FTimer:TBZThreadTimer;
    {$IFDEF WINDOWS}
    FPriority: TThreadPriority;
    {$ENDIF WINDOWS}
    FSynchronizing: Boolean;
    FThreadName :string;
  protected
    procedure DoSuspend;
    procedure Execute; override;
  public
    constructor Create(ATimer:TBZThreadTimer);
    destructor Destroy; override;
    procedure Stop;
    property Interval: Cardinal read FInterval;
    property Timer:TBZThreadTimer read FTimer;
    property ThreadName: String read FThreadName write FThreadName;
    property Synchronizing: Boolean read FSynchronizing;
  end;


//==============================================================================

{**-----------------------------------------------------------------------------
 * @Name        :  Register
 * @Description :  Enregistre notre composant dans l'IDE
-----------------------------------------------------------------------------**}
procedure Register;
begin
  {$I ../../ressources/bz_threadtimer_icon.lrs}

  RegisterComponents('Beanz',[TBZThreadTimer]);
end;

//==============================================================================
//==============================================================================
// Fonctions Internes
//==============================================================================
function SubtractMin0(const Big, Small: Cardinal): Cardinal;
begin
  if Big <= Small then
    Result := 0
  else
    Result := Big - Small;
end;

//==============================================================================
//==============================================================================
// 1. TBZTimerThread
//==============================================================================

//==== PRIVATE =================================================================

//==== PROTECTED ===============================================================
{**-----------------------------------------------------------------------------
 * @Name        : DoSuspend
 * @Description : Met en pause notre Thread
 -----------------------------------------------------------------------------**}
procedure TBZTimerThread.DoSuspend;
begin
  //FHasBeenSuspended := True;
  Suspended := True;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Execute
 * @Description : Execution du Thread
 -----------------------------------------------------------------------------**}
procedure TBZTimerThread.Execute;
var
  Offset, TickCount: Cardinal;
begin
//  NameThread(ThreadName);
  {$IFDEF WINDOWS}
  Priority := FPriority;
  {$ENDIF WINDOWS}

  if Not(FTimer.Accurate) then RTLEventWaitFor(FEvent, FInterval);

  while  (not Self.Terminated) and (FTimer.Enabled) do
  begin
    FHasBeenSuspended := False;

    TickCount := GetTickCount64;
    if not Terminated then
    begin
      FSynchronizing := True;
      try

        if FTimer.Synchronize then Synchronize(@FTimer.DoOnTimer)
        else FTimer.DoOnTimer;

      finally
        FSynchronizing := False;

      end;
    end;

    // Determine how much time it took to execute OnTimer event handler. Take a care
    // of wrapping the value returned by GetTickCount API around zero if Windows is
    // run continuously for more than 49.7 days.
    if FHasBeenSuspended then
      Offset := 0
    else
    begin
      Offset := GetTickCount64;

      if Offset >= TickCount then
        Dec(Offset, TickCount)
      else
        Inc(Offset, High(Cardinal) - TickCount);
    end;
    // RtlEventSetEvent(FEvent);
    // Make sure Offset is less than or equal to FInterval.
    // (rb) Ensure it's atomic, because of KeepAlive
    if Not(FTimer.Accurate) then RTLEventWaitFor(FEvent, SubtractMin0(Interval, Offset));
    if Terminated then
      Exit;
  end;
end;
//==== PUBLIC ==================================================================
{**-----------------------------------------------------------------------------
 * @Name        : Create
 * @Description : Creation de notre Thread personnalisé
 -----------------------------------------------------------------------------**}
constructor TBZTimerThread.Create(ATimer:TBZThreadTimer);
begin
  inherited Create(False);

  { Manually reset = false; Initial State = false }
  if Not(ATimer.Accurate) then
  begin
    FEvent :=RTLEventCreate; //CreateEvent(nil, False, False, nil);
    if FEvent = nil then
      RaiseLastOSError;
  end;
  FInterval := ATimer.FInterval;
  FTimer := ATimer;
  {$IFDEF WINDOWS}
  FPriority := ATimer.Priority; // setting the priority is deferred to Execute()
  {$ENDIF WINDOWS}
  FThreadName := Format('%s: %s',[ClassName, ATimer.Name]);
end;

{**-----------------------------------------------------------------------------
 * @Name        : Destroy
 * @Description : Destruction de notre Thread
 -----------------------------------------------------------------------------**}
destructor TBZTimerThread.Destroy;
begin
  Stop;
  if Not(FTimer.Accurate) then
     if FEvent <> nil then RTLeventDestroy(FEvent);
  inherited Destroy;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Strop
 * @Description : Arret complet du Thread
 -----------------------------------------------------------------------------**}
procedure TBZTimerThread.Stop;
begin
  Terminate;
 // SetEvent(FEvent);
  if Suspended then
    Suspended := False;
end;

//==============================================================================
//==============================================================================
// 1. TBZThreadTimer
//==============================================================================

//==== PRIVATE =================================================================

{**------------------------------------------------------------------------------
 * Setters pour les differentes propriétés
------------------------------------------------------------------------------**}
procedure TBZThreadTimer.SetEnabled(const Value: Boolean);
begin
  if csLoading in ComponentState then
    FStreamedEnabled := Value
  else
  if FEnabled <> Value then
  begin
    FEnabled := Value;
    UpdateTimer;
  end;
end;

procedure TBZThreadTimer.SetSynchronize(const Value: Boolean);
begin
  if csLoading in ComponentState then
    FStreamedEnabled := Value
  else
  if FSynchronize <> Value then
  begin
    FSynchronize := Value;
    UpdateTimer;
  end;
end;

procedure TBZThreadTimer.SetInterval(const Value: Cardinal);
begin
  if FInterval <> Value then
  begin
    FInterval := Value;
    UpdateTimer;
  end;
end;

procedure TBZThreadTimer.SetKeepAlive(const Value: Boolean);
begin
  if FKeepAlive <> Value then
  begin
    StopTimer;
    FKeepAlive := Value;
    UpdateTimer;
  end;
end;

procedure TBZThreadTimer.SetOnTimer(const Value: TNotifyEvent);
begin
  if @FOnTimer <> @Value then
  begin
    FOnTimer := Value;
    UpdateTimer;
  end;
end;

{$IFDEF MSWINDOWS}
procedure TBZThreadTimer.SetPriority(const Value: TThreadPriority);
begin
  if FPriority <> Value then
  begin
    FPriority := Value;
    if FThread <> nil then
      FThread.Priority := FPriority;
  end;
end;
{$ENDIF MSWINDOWS}

//==== PROTECTED ===============================================================
{**-----------------------------------------------------------------------------
 * @Name        : DoOnTimer
 * @Description : Execution de notre procedure "OnTimer" dans le TimerThread
 -----------------------------------------------------------------------------**}
procedure TBZThreadTimer.DoOnTimer;
begin
  if csDestroying in ComponentState then
    Exit;
  try
    if Assigned(FOnTimer) then
      FOnTimer(Self);
  except
    if Assigned(ApplicationHandleException) then
      ApplicationHandleException(Self);
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Loaded
 * @Description : Chargement de notre composant
 -----------------------------------------------------------------------------**}
procedure TBZThreadTimer.Loaded;
begin
  inherited Loaded;
  SetEnabled(FStreamedEnabled);
end;

{**-----------------------------------------------------------------------------
 * @Name        : StopTimer
 * @Description : Arret de notre composant TimerThread
 -----------------------------------------------------------------------------**}
procedure TBZThreadTimer.StopTimer;
begin
  if FThread <> nil then
  begin
    TBZTimerThread(FThread).Stop;
    if not TBZTimerThread(FThread).Synchronizing then
      FreeAndNil(FThread)
    else
    begin
      // We can't destroy the thread because it called us through Synchronize()
      // and is waiting for our return. But we need to destroy it after it returned.
      TBZTimerThread(FThread).FreeOnTerminate := True;
      FThread := nil
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : UpdateTimer
 * @Description : Mise à jour de notre TimerThread
 -----------------------------------------------------------------------------**}
procedure TBZThreadTimer.UpdateTimer;
var
  DoEnable: Boolean;
begin
  if ComponentState * [csDesigning, csLoading] <> [] then
    Exit;

  DoEnable := FEnabled and Assigned(FOnTimer) and (FInterval > 0);

  if not KeepAlive then
    StopTimer;

  if DoEnable then
  begin
    if FThread <> nil then
    begin
      TBZTimerThread(FThread).FInterval := FInterval;
      if FThread.Suspended then
        FThread.Suspended := False;
    end
    else
      FThread := TBZTimerThread.Create(Self);
      if Assigned(FThread.FatalException) then
       raise FThread.FatalException;
  end
  else
  if FThread <> nil then
  begin
    if not FThread.Suspended then
      TBZTimerThread(FThread).DoSuspend;

    TBZTimerThread(FThread).FInterval := FInterval;

  end;
end;

//==== PUBLIC ==================================================================
{**-----------------------------------------------------------------------------
 * @Name        : Create
 * @Description : Creation de notre composant
 -----------------------------------------------------------------------------**}
constructor TBZThreadTimer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FInterval := 1000;
  {$IFDEF MSWINDOWS}
  FPriority := tpNormal;
  {$ENDIF MSWINDOWS}
end;

{**-----------------------------------------------------------------------------
 * @Name        : Destroy
 * @Description : Destruction de notre composant
 -----------------------------------------------------------------------------**}
destructor TBZThreadTimer.Destroy;
begin
  StopTimer;
  inherited Destroy;
end;


//==============================================================================
//==============================================================================

//initialization

//------------------------------------------------------------------------------
//finalization

//==============================================================================
End.
