unit bz_Button;
//==============================================================================
// @NAME    : BZButton
// @VERSION : 0.7.4
// @DATE    : 28/09/2015
// @UPDATED : 22/10/2015
// @PACKAGE : BEANZ
// @DESC    : TBZButton est un composant visuel pour dessiner de zolies  boutons
// @AUTHOR  : J.DELAUNEY Aka BeanzMaster
// @LICENCE : WTFPL
//------------------------------------------------------------------------------
// @NOTE    : Ce composant à uniquement été tester sous environnement WINDOWS 7
//
//------------------------------------------------------------------------------
// @TODO    :
//  - A Tester sous environnement Linux (gnome, kde ect..) et Mac
//  - Optimizer les fonctions de calcul des positions et dimensions des éléments
//  - Calcul automatique Marging et Spacing  si égal à -1
//  - Ajouter la possibiliter d'avoir une ombre sous le bouton
//  - Ajouter la possibilité de sauvegarder et d'ouvrir les styles depuis un fichier et ou un stream
//  - Ajouter la possibiliter d'avoir des ombres avec des offsets négatif pour le bouton
//  - Blending des bordures avec le fond
//  - Blending du bouton sur le canvas (utilisation de bitblt ??)
//  - Utiliser des images pour l'aspect (utilisation des régions et des masques)
//  - Redimensionnement des Glyphs si plus grand que les dimensions du bouton
//  - Plus de formes : Triangle, Demi Cercle, Tabs, ...
//  - Ajouter la possibiliter d'animer les couleurs du bouton  entre l'affichage des etats (fading/blending)
//  - Ajouter la possibiliter d'animer les glyphs (avec ou sans utilisation d'une imageList)
//  - Rotation du bouton et du caption
//  - Ajouter d'autres Types de dégradés
//  - Séparer les differents éléments de style (Background, Border, Caption, Glyph) pour une meilleur gestion
//  - Autres ???
//
//==============================================================================

{$mode objfpc}{$H+}

interface

//==============================================================================

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ActnList,
  LCLProc, LCLType, LCLIntf, LResources, LMessages, Menus,
  GraphMath,
  bz_formhotkeymanager;
  //FPImage, GraphType, IntfGraphics;
  //bz_BaseControls;

//==============================================================================

//==============================================================================
//==== [ TBZComponentProperty ]=================================================
// @Desc : Object pour l'edition de propriétés d'un composant avec notification
//         de changement de valeur
//==============================================================================
Type
  TOnBZPropertyChange = procedure(ASender: TObject; AData: PtrInt) of object;

Type
  TBZComponentProperty = class(TPersistent)
   private
     FOnChange: TOnBZPropertyChange;
   protected
     FControl: TObject;
     procedure Change(AData: PtrInt = 0); virtual;
   public
     constructor Create(AControl: TObject); virtual;
   public
     property Control: TObject read FControl;
     property OnChange: TOnBZPropertyChange read FOnChange write FOnChange;
   end;

//==============================================================================
// Définitions des paramètres des propriétés de l'aspect de notre bouton
Type
  TBZButtonGradientKind = (gkHorizontal, gkVertical, gkRadial, gkFromTopLeft, gkFromTopRight, gkDoubleHorz, gkDoubleVert);

  TBZButtonBorders = set of (bbTop, bbLeft, bbBottom, bbRight);
  TBZButtonBorderWidth = 1..10;

  TBZButtonState = (bsNormal, bsHover, bsDown, bsDisabled);

  TBZButtonGlyphLayout = (glLeft, glRight, glTop, glBottom); // @todo : ajouter + de placement cf : TBZButtonCaptionLayout

  TBZButtonShape = (bsRect, bsRoundRect, bsEllipse); // bsTriangleLeft, bsTriangleRight, bsTriangleUp, bsTriangleDown, bsOctogon, bsHexagon, bsCustom
  TBZButtonRoundingType = (rtRounded, rtSquared); // + rtRoundedLeft, rtRoundedRight, rtSquaredLeft, rtSquaredRight

  TBZButtonCaptionStyle = (csDefault, csBevel, csRaised, csEmbossed, csContour);

  TBZButtonPopUpMenuLayout = (ppmlLeft, ppmlRight, ppmlTop, ppmlBottom);

  //TBZButtonCaptionLayout = (clTopLeft, clCenterLeft, clBottomLeft,
  //                          clTopCenter,clCenter,clBottomCenter,
  //                          clTopRight, clCenterRight, clBottomRight);

  TBZButtonArrowDirection = (adToTop,adToLeft,adToBottom, adToRight);
  TBZButtonArrowLayout    = (alLeft,alRight);

  // Pour Dessiner les dégradés : Methode Delphi
  //TRGBArray = array[0..0] of TRGBTriple;
  //PRGBArray = ^TRGBArray;

//  TBZButtonPercentValue = 1..100;

//==============================================================================
//==== [ TBZButtonStateStyle ]=================================================
// @Desc : Object pour définir l'aspect général d'un état du bouton
//==============================================================================
Type
  TBZButtonStateStyle =  class(TBZComponentProperty)
  private
    { Private declarations }

    FGradient           : boolean;
    FGradientKind       : TBZButtonGradientKind;
    FColorFrom,
    FColorTo            : TColor;
    FShape              : TBZButtonShape;

    FRoundingType       : TBZButtonRoundingType;
    FRoundX,
    FRoundY             : Integer;

    //FShadow             : Boolean;
    //FShadowColor        : TColor;
    //FShadowOffsetX      : integer;
    //FShadowOffsetY      : integer;

    FShowArrow        : Boolean;
    FArrowDir         : TBZButtonArrowDirection;
    FArrowLayout      : TBZButtonArrowLayout;
    FArrowWidth       : Integer;
    FArrowHeight      : Integer;
    FArrowColor       : TColor;
    FArrowBorderColor : TColor;
    FArrowBorderWidth : TBZButtonBorderWidth;
    //FArrowStyle      : TBZButtonArrowStyle; //Flat/bevel/raised
    //FArrowShadow   : Boolean;
    //FArrowShadowColor : TColor;
    //FArrowAdjustH,
    //FArrowAdjustV  : Integer;

    FBorderTopColor,
    FBorderLeftColor,
    FBorderBottomColor,
    FBorderRightColor   : TColor;
    FBorders            : TBZButtonBorders;
    FBorderWidth        : TBZButtonBorderWidth;

    FCaptionStyle       : TBZButtonCaptionStyle;
    FCaptionShadow      : Boolean;
    FCaptionShadowOffsetX : integer;
    FCaptionShadowOffsetY : integer;
    FCaptionShadowColor : TColor;
    FFont               : TFont;
    FAlignment          : TAlignment; // TBZButtonCaptionLayout;

    FGlyph              : TPicture;
    FGlyphTransparent   : boolean;
    FGlyphLayout        : TBZButtonGlyphLayout;
    //FUseImageList       : Boolean;
    //FGlyphCount         : Integer;
    //FGlyphIndex         : Integer;
    //FGlyphAdjustH,
    //FGlyphAdjustV  : Integer;

    procedure setShape(AValue : TBZButtonShape);
    procedure setRoundingType(AValue : TBZButtonRoundingType);
    procedure setRoundX(AValue:Integer);
    procedure setRoundY(AValue:Integer);

    //procedure setShadow(Avalue:Boolean);
    //procedure setShadowColor(AValue:TColor);
    //procedure setShadowOffsetX(AValue : Integer);
    //procedure setShadowOffsetY(AValue : Integer);

    procedure setGradient(Avalue:Boolean);
    procedure setGradientKind(AValue:TBZButtonGradientKind);
    procedure setColorFrom(AValue:TColor);
    procedure setColorTo(AValue:TColor);

    procedure setBorderTopColor(AValue:TColor);
    procedure setBorderLeftColor(AValue:TColor);
    procedure setBorderBottomColor(AValue:TColor);
    procedure setBorderRightColor(AValue:TColor);

    procedure setCaptionStyle(AValue:TBZButtonCaptionStyle);
    procedure setCaptionShadow(Avalue:Boolean);
    procedure setCaptionShadowColor(AValue:TColor);
    procedure setCaptionShadowOffsetX(AValue : Integer);
    procedure setCaptionShadowOffsetY(AValue : Integer);

    procedure setAlignment(AValue:TAlignment);
    //procedure setFont(AValue:TFont);

    procedure setGlyph(const AValue: TPicture);
    procedure setGlyphTransparent(const AValue: boolean);
    procedure setGlyphLayout(const AValue: TBZButtonGlyphLayout);

    procedure setShowArrow(const AValue : Boolean);
    procedure setArrowDir(const AValue : TBZButtonArrowDirection);
    procedure setArrowLayout(const AValue : TBZButtonArrowLayout);
    procedure setArrowWidth(const AValue : Integer);
    procedure setArrowHeight(const AValue : Integer);
    procedure setArrowColor(const AValue : TColor);
    procedure setArrowBorderColor(const AValue : TColor);
    procedure setArrowBorderWidth(const AValue : TBZButtonBorderWidth);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AControl: TObject);override;
    destructor Destroy;override;
    procedure Assign(Source: TPersistent);override;

  published
    { Published declarations }
    property Shape                 : TBZButtonShape       Read FShape               Write setShape               default bsRect;

    property Gradient              : Boolean               Read FGradient            Write setGradient            default True;
    property GradientKind          : TBZButtonGradientKind Read FGradientKind        Write setGradientKind       default gkVertical;
    property ColorFrom             : TColor                Read FColorFrom           Write setColorFrom;
    property ColorTo               : TColor                Read FColorTo             Write setColorTo;

    property ShowArrow        : Boolean                 Read FShowArrow        Write setShowArrow;
    property ArrowDir         : TBZButtonArrowDirection Read FArrowDir         Write setArrowDir;
    property ArrowLayout      : TBZButtonArrowLayout    Read FArrowLayout      Write setArrowLayout;
    property ArrowWidth       : Integer                 Read FArrowWidth       Write setArrowWidth;
    property ArrowHeight      : Integer                 Read FArrowHeight      Write setArrowHeight;
    property ArrowColor       : TColor                  Read FArrowColor       Write setArrowColor;
    property ArrowBorderColor : TColor                  Read FArrowBorderColor Write setArrowBorderColor;
    property ArrowBorderWidth : TBZButtonBorderWidth    Read FArrowBorderWidth Write setArrowBorderWidth;

    //property Shadow                : Boolean               Read FShadow              Write setShadow              default False;
    //property ShadowColor           : TColor                Read FShadowColor         Write setShadowColor;
    //property ShadowOffsetX         : Integer               Read FShadowOffsetX       Write setShadowOffsetX;
    //property ShadowOffsetY         : Integer               Read FShadowOffsetY       Write setShadowOffsetY;

    property Borders               : TBZBUttonBorders      Read FBorders             Write FBorders;
    property BorderWidth           : TBZButtonBorderWidth  Read FBorderWidth         Write FBorderWidth;
    property BorderTopColor        : TColor                Read FBorderTopColor      Write setBorderTopColor;
    property BorderLeftColor       : TColor                Read FBorderLeftColor     Write setBorderLeftColor;
    property BorderBottomColor     : TColor                Read FBorderBottomColor   Write setBorderBottomColor;
    property BorderRightColor      : TColor                Read FBorderRightColor    Write setBorderRightColor;

    property Rounding              : TBZButtonRoundingType Read FRoundingType        Write setRoundingType;
    property RoundX                : Integer               Read FRoundX              Write setRoundX;
    property RoundY                : Integer               Read FRoundY              Write setRoundY;

    property Font                  : TFont                 Read FFont                Write FFont;
    property Alignment             : TAlignment            Read FAlignment           Write setAlignment;          // default taCenter;

    property CaptionStyle          : TBZButtonCaptionStyle Read FCaptionStyle        Write setCaptionStyle;
    property CaptionShadow         : Boolean               Read FCaptionShadow       Write setCaptionShadow       default False;
    property CaptionShadowColor    : TColor                Read FCaptionShadowColor  Write setCaptionShadowColor;
    property CaptionShadowOffsetX  : Integer               Read FCaptionShadowOffsetX Write setCaptionShadowOffsetX;
    property CaptionShadowOffsetY  : Integer               Read FCaptionShadowOffsetY Write setCaptionShadowOffsetY;

    property Glyph                 : TPicture              Read FGlyph               Write setGlyph;
    property GlyphTransparent      : Boolean               Read FGlyphTransparent    Write setGlyphTransparent;
    property GlyphLayout           : TBZButtonGLyphLayout  Read FGLyphLayout         Write setGlyphLayout;

    property OnChange;
  end;

//==============================================================================
//==== [ TBZButtonStyles ]=====================================================
// @Desc : Object encapsulant les différent états de notre bouton
//==============================================================================
Type
  TBZButtonStyles =  class(TBZComponentProperty)
  private
    { Private declarations }

    FNormalStyle   : TBZButtonStateStyle;
    FHoverStyle    : TBZButtonStateStyle;
    FDownStyle     : TBZButtonStateStyle;
    FDisabledStyle : TBZButtonStateStyle;

  protected
    { Protected declarations }
    procedure DoChangeStyle(ASender: TObject; AData: PtrInt);

  public
    { Public declarations }
    constructor Create(AControl: TObject);override;
    destructor Destroy;override;
    procedure Assign(Source: TPersistent);override;

  published
    { Published declarations }
    property NormalStyle    : TBZButtonStateStyle    Read FNormalStyle   Write FNormalStyle;
    property HoverStyle     : TBZButtonStateStyle    Read FHoverStyle    Write FHoverStyle;
    property DownStyle      : TBZButtonStateStyle    Read FDownStyle     Write FDownStyle;
    property DisabledStyle  : TBZButtonStateStyle    Read FDisabledStyle Write FDisabledStyle;

    property OnChange;
  end;

//==============================================================================
//==== [ TBZCustomButton ]======================================================
// @Desc : Object de base pour définir notre bouton
//==============================================================================
Type
  TBZCustomButton = class(TCustomControl)
  private
    { Private declarations }
    FBufferState : array[0..3] of TBitmap;
   // {$ifdef FPC}Fmem:TMemoryStream; {$endif}

    FStyles     : TBZButtonStyles;
    FState      : TBZButtonState;

    FAutoSize   : Boolean;
    FFormHotKeyManager : TBZFormHotKeyManager;

    FOnClick : TNotifyEvent;

    //FAllowUp    : Boolean;
    //FDown       : Boolean;
    //FGroupIndex : Boolean;

    // FAnimated : Boolean;
    // FAnimationKind
    // FAnimationTimer : TTimer;

    FPopupMenuLayout : TBZButtonPopUpMenuLayout;

    FShowShortCut   : Boolean;
    FShortCut       : TShortCut;
    FShortCutString : String;

    // FTheme : TBZButtonThemeManager;
    // FImageList : TImageList;

    FSpacing    : integer;
    FMarging    : integer;

    FWordWrap   : Boolean;

    FChecked   : Boolean;
    FCheckable : Boolean;

    FCaption : TCaption;
    FShowCaption : Boolean;

    FEnabled : Boolean;
    FActivated : Boolean;  // Si false le Bouton peut être utiliser comme un TPanel

    FShareGlyph     : Boolean;  // Tous les états du bouton utilise le glyph de l'état "Normal"

    FFocusable      : Boolean;
    FFocused        : Boolean;
    FDrawFocusRect  : Boolean;
    FFocusRectStyle : TPenStyle;
    FFocusRectColor : TColor;
    FFocusRectWidth : TBZButtonBorderWidth;

    FNeededUpdate : Boolean;  // Indicateur pour demander la génération des Etats lorsque une propriété est modifiée

    procedure  DoClick(Sender:TObject);

    procedure UpdateFocus(AFocused: Boolean);

    procedure SetFocusable(AValue: boolean);
    procedure SetSpacing(const AValue: integer);
    procedure SetMarging(const AValue: integer);
    procedure SetState(AValue:TBZButtonState);
    procedure SetCaption(const AValue: TCaption);
    procedure SetShowCaption(const AValue: Boolean);
    procedure SetWordWrap(const AValue: Boolean);
    procedure SetCheckable(const AValue: Boolean);
    procedure SetChecked(const AValue: Boolean);
    procedure SetShowShortCut(const AValue:Boolean);
    procedure SetShortCut(const AValue:TShortCut);
    procedure SetPopUpMenuLayout(const AVAlue : TBZButtonPopUpMenuLayout);
    procedure SetActivated(Const AValue:Boolean);
    procedure SetClick(AEvent:TNotifyEvent);

    //Calcul des dimensions et positions des éléments du bouton
    Function getArrowBoundsRect(AArrowLayout: TBZButtonArrowLayout;AWidth, AHeight:integer;var AButtonRect:TRect;ABorderWidth:Integer) : TRect;
    Function getShortCutBoundsRect(ACanvas:TCanvas;AButtonRect:TRect;ABorderWidth:Integer) : TRect;
    Function getCaptionBoundsRect(AGlyph:TPicture;AGlyphLayout:TBZButtonGlyphLayout;AbuttonRect, AGlyphRect, AShortCutRect:TRect; ABorderWidth : Integer) : TRect;
    Function getGlyphBoundsRect(AGlyph:TPicture;AGlyphLayout:TBZButtonGlyphLayout; AButtonRect,AShortCutRect:TRect;ABorderWidth:Integer) : TRect;

    //Dessin des éléments du bouton
    procedure DrawCaption(ACanvas:TCanvas;ARect:TRect;AStyle:TBZButtonCaptionStyle;ACaption:String;AFlags:Integer);
    procedure DrawGlyph(ACanvas:TCanvas; AGlyph:TPicture;x,y:Integer;ATransp:Boolean);
    procedure DrawBorders(ACanvas:TCanvas;AStyle: TBZButtonStateStyle;APolyArray:PPoint;APolyCount:Byte);
    procedure DrawFocusBorders(ACanvas:TCanvas;AStyle: TBZButtonStateStyle;APolyArray:PPoint;APolyCount:Byte);
    procedure DrawBackground(ABmp:TBitmap;AButtonRect:TRect;AStyle: TBZButtonStateStyle;Var PolyArray:PPoint; Var APolyCount:Byte; Var Region : HRGN);
    procedure DrawArrow(ACanvas: TCanvas; ARect: TRect; ADirection: TBZButtonArrowDirection; AColor, ABorderColor: TColor;AABorderWidth:Integer);
    procedure DrawButton(AState : TBZButtonState;var ABmp:TBitmap);


  protected
    { Protected declarations }
     function GetPopupMenuHeight(APopupMenu:TPopupMenu):integer;
     function GetPopupMenuWidth(APopupMenu:TPopupMenu):integer;

     procedure WMSetFocus(var Message: TLMSetFocus); message LM_SETFOCUS;
     procedure WMKillFocus(var Message: TLMKillFocus); message LM_KILLFOCUS;

     procedure SetEnabled(Value: Boolean); override;
     procedure SetAutoSize(Value:Boolean); override;

     procedure DoChangeStyles(ASender: TObject; AData: PtrInt);

     procedure ChangeBounds(ALeft, ATop, AWidth, AHeight: integer; KeepBase: boolean); override;

     procedure PaintTo(ACanvas: TCanvas; X, Y: Integer);
     procedure Paint; override;

     procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
     procedure MouseLeave; override;
     procedure MouseEnter; override;
     procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
     procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
     procedure KeyUp(var Key: Word; Shift: TShiftState); override;
     procedure KeyDown(var Key: Word; Shift: TShiftState); override;
     //procedure UTF8KeyPress(var UTF8Key: TUTF8Char);override;

     procedure DoEnter; override;
     procedure DoExit; override;

     procedure Loaded;override;

  public
    { Public declarations }
    //function CanFocus: Boolean; override;
    //function CanSetFocus: Boolean; override;
    function Focused: Boolean; override;
    //procedure SetFocus;  override;

    constructor Create(AOwner: TComponent);override;
    destructor Destroy; override;

  published
    { Published declarations }

    property Activated    : Boolean          Read FActivated Write SetActivated;
    property AutoSize     : Boolean          Read FAutoSize  Write SetAutoSize;

    property Enabled      : Boolean          Read FEnabled   Write setEnabled;
    property Focusable    : boolean          read FFocusable write SetFocusable;

    property FocusRectStyle : TPenStyle            read FFocusRectStyle write FFocusRectStyle;
    property FocusRectColor : TColor               read FFocusRectColor write FFocusRectColor;
    property FocusRectWidth : TBZButtonBorderWidth read FFocusRectWidth write FFocusRectWidth;
    property DrawFocusRect  : Boolean              read FDrawFocusRect Write FDrawFocusRect;

    property Caption      : TCaption         Read FCaption   Write setCaption;
    property ShowCaption  : Boolean          Read FShowCaption Write setShowCaption;
    property WordWrap     : Boolean          Read FWordWrap  Write setWordWrap;

    property Checkable    : Boolean          Read FCheckable Write setCheckable;
    property Checked      : Boolean          Read FChecked   Write setChecked;

    property Styles       : TBZButtonStyles Read FStyles    Write FStyles;
    property State        : TBZButtonState  Read FState     Write setState;

    property ShareGlyph   : Boolean          Read FShareGlyph Write FShareGlyph default false;

    property Spacing      : integer          Read FSpacing   Write SetSpacing;
    property Marging      : integer          Read FMarging   Write SetMarging;

    property ShowShortCut : Boolean          Read FShowShortCut Write SetShowShortCut;
    property ShortCut     : TShortCut        Read FShortCut     Write SetShortCut      default 0;
    property FormHotKeyManager : TBZFormHotKeyManager read FFormHotKeyManager write FFormHotKeyManager;
   // property UseFormShortCut   : Boolean read FUseFormShortCut write FUseFormShortCut;

    property popupMenuLayout : TBZButtonPopUpMenuLayout Read FPopupMenuLayout write SetPopupMenuLayout;
    property PopupMenu;

    property OnClick : TNotifyEvent read FOnClick write setClick;

  end;

//==============================================================================
//==== [ TBZButton ]============================================================
// @Desc : Notre bouton publier
//==============================================================================
Type
  TBZButton = class(TBZCustomButton)
    private
    protected
    public
    published

      property Styles;
      property State;
      property Spacing;
      property Marging;
      property WordWrap;
      property Checkable;
      property Checked;
      property ShowShortCut;
      property ShortCut;
      property AutoSize;
      property Activated;
      property Focusable;
      property DrawFocusRect;
      property FocusRectStyle;
      property FocusRectWidth;
      property FocusRectColor;
      property FormHotKeyManager;
      property PopupMenuLayout;


      Property Action;
      Property Align;
      Property Anchors;
      Property Constraints;
      Property Caption;
      Property Cursor;
      Property DragCursor;
      Property DragMode;
      Property DragKind;
      Property Enabled;
      Property HelpContext;
      Property Hint;
      Property ParentFont Default True;
      Property ParentShowHint;
      Property PopupMenu;
      Property ShowHint;
      Property TabOrder;
      Property TabStop Default True;
      Property Tag;
      Property Visible;

      property OnClick;
      property OnContextPopup;
      property OnEnter;
      property OnExit;
      property OnKeyDown;
      property OnKeyPress;
      property OnKeyUp;
      property OnMouseDown;
      property OnMouseMove;
      property OnMouseUp;
      property OnChangeBounds;
      property OnDragDrop;
      property OnDragOver;
      property OnEndDrag;
      property OnMouseEnter;
      property OnMouseLeave;
      property OnResize;
      property OnStartDrag;
      property OnUTF8KeyPress;
  end;

//==============================================================================
// Enregistrement de notre composant dans l'IDE
procedure Register;

//==============================================================================

implementation

uses Math;
//==============================================================================

{**-----------------------------------------------------------------------------
 * @Name        :  Register
 * @Description :  Enregistre notre composant dans l'IDE
 -----------------------------------------------------------------------------**}
procedure Register;
begin
  {$I ../../ressources/bz_button_icon.lrs}
  RegisterComponents('Beanz Controls',[TBZButton]);
end;

//==============================================================================
//==============================================================================
// Fonctions Internes
//==============================================================================

{**-----------------------------------------------------------------------------
 * @Name        : LightenColor
 * @Description : Eclaircir une couleur en fonction du facteur en pourcentage
 -----------------------------------------------------------------------------**}
function LightenColor(AColor:TColor;Factor:Byte):TColor;
var r,g,b,d : Integer;
begin

 r := GetRValue(AColor);
 g := GetGValue(AColor);
 b := GetBValue(AColor);

 d:= (r+g+b) div 3;
 if (d = 0) then d:=round(255*factor / 100) else d:=round(d*factor / 100) ;

 r:=max(r+d,255);
 g:=max(g+d,255);
 b:=max(b+d,255);

 result := RGB(r,g,b);
end;

{**-----------------------------------------------------------------------------
 * @Name        : DarkenColor
 * @Description : Assombrir une couleur en fonction du facteur en pourcentage
 -----------------------------------------------------------------------------**}
function DarkenColor(AColor:TColor;Factor:Byte):TColor;
var r,g,b,d  : Integer;
begin
  r := GetRValue(AColor);
  g := GetGValue(AColor);
  b := GetBValue(AColor);

  d:= (r+g+b) div 3;
  d:=round(d*factor / 100);

  r:=max(r-d,0);
  g:=max(g-d,0);
  b:=max(b-d,0);

  result := RGB(r,g,b);
end;

{**-----------------------------------------------------------------------------
 * @Name        : getCaptionFlags
 * @Description : Renvoi le flag correcte pour la fonction "DrawText"
 -----------------------------------------------------------------------------**}
function getCaptionFlags(Alignment: TAlignment; WordWrap: boolean):integer;
var Flags:Integer;
begin
  Flags :=  DT_VCENTER or DT_END_ELLIPSIS;   //DT_NOPREFIX or

  case Alignment of
    taLeftJustify:
      Flags := DT_LEFT or Flags;
    taCenter:
      Flags := DT_CENTER or Flags;
    taRightJustify:
      Flags := DT_RIGHT or Flags;
  end;

  if WordWrap then
    Flags := Flags or DT_WORDBREAK
  else
    Flags := Flags or DT_SINGLELINE;

  result:=Flags;
end;

{**-----------------------------------------------------------------------------
 * @Name        : DrawGradient
 * @Description : Dessine un dégradé entre Color1 and Color2 dans le bitmap et la region
 -----------------------------------------------------------------------------**}
procedure DrawGradient(Bitmap: TBitmap; CurrRect: TRect; Color1, Color2: TColor; GradientType : TBZButtonGradientKind; Region: HRGN);
var
  y, x, MaxDist, C1, C2,C3 : integer;

  //  rowDest : PRGBArray; --> Methode Delphi avec ScanLine

  // Ma tentative de convertir la methode "ScanLine" de Delphi vers Lazarus avec LazIntImage ne fonctionne pas
  // Du coup je suis passé directement par le Canvas du bitmap pour que ce code soit portable
  //TempIntfImg: TLazIntfImage;
  //ImgHandle,ImgMaskHandle: HBitmap;
  //CurColor: TFPColor;
  //ImgFormatDescription: TRawImageDescription;

  Dep : array[0..2] of integer;
  Dif : array[0..2] of Double;
  Coef : Double;

begin
 // if (Bitmap.Width <= 0) or (Bitmap.Height <= 0) then Exit;

  C3 := colorToRGB(clInfoBK); // "clInfoBK" Couleur Transparente pour Lazarus sinon Canvas.Draw ne prend pas en charge la transparence

  if Color1 = Color2  then begin
    Bitmap.Canvas.Brush.Color := Color1;
    if Region > 0 then
      FillRgn(Bitmap.Canvas.Handle, Region, Bitmap.Canvas.Brush.Reference.Handle)
    else
      Bitmap.Canvas.FillRect(CurrRect);
    Exit;
  end;

  C1 := ColorToRGB(Color1);
  C2 := ColorToRGB(Color2);

  case GradientType of
    gkVertical : MaxDist := CurrRect.Bottom - CurrRect.Top;

    gkHorizontal : MaxDist := CurrRect.Right - CurrRect.Left;

    gkFromTopLeft,
    gkFromTopRight: MaxDist := (Round(Sqrt( Sqr(CurrRect.Right - CurrRect.Left)
                              + Sqr(CurrRect.Bottom - CurrRect.Top)))
                              - CurrRect.Left);
    gkRadial: MaxDist := (Round(Sqrt( Sqr(CurrRect.Right - CurrRect.Left)
                              + Sqr(CurrRect.Bottom - CurrRect.Top)))
                              - CurrRect.Left) div 2;
    gkDoubleHorz: MaxDist :=  (CurrRect.Right - CurrRect.Left) div 2;
    gkDoubleVert: MaxDist :=  (CurrRect.Bottom - CurrRect.Top) div 2;
  end;

  if MaxDist = 0 then MaxDist:=1;

  Dep[0] := GetRValue(C1);
  Dep[1] := GetGValue(C1);
  Dep[2] := GetBValue(C1);

  Dif[0] := (GetRValue(C2) - Dep[0]) / MaxDist;
  Dif[1] := (GetGValue(C2) - Dep[1]) / MaxDist;
  Dif[2] := (GetBValue(C2) - Dep[2]) / MaxDist;

  if Dif[0] > 255 then Dif[0] := 255;
  if Dif[1] > 255 then Dif[1] := 255;
  if Dif[2] > 255 then Dif[2] := 255;

  //TempIntfImg:=TLazIntfImage.Create(Bitmap.Width,Bitmap.Height); --> Provoque un FPImageException avec le message : Failed to create Handles

  //TempIntfImg:=TLazIntfImage.Create(0,0); --> Idem provoque une erreur
  //TempIntfImg.LoadFromBitmap(Bitmap.Handle,Bitmap.MaskHandle);

  //ImgFormatDescription.Init_BPP32_B8G8R8_BIO_TTB(Bitmap.Width,Bitmap.Height);
  //TempIntfImg.DataDescription:=ImgFormatDescription;

  for y:= CurrRect.Top to CurrRect.Bottom -1 do
  begin
    //rowDest := Bitmap.ScanLine[y];

    for x := CurrRect.Left to CurrRect.Right - 1 do
    begin
      case GradientType of
          gkHorizontal : Coef := x - CurrRect.Left;
          gkVertical :   Coef := y - CurrRect.Top;
          gkFromTopLeft :Coef := Sqrt(Sqr(x - CurrRect.Left) + Sqr(y - CurrRect.Top));
          gkFromTopRight:Coef := Sqrt(Sqr(CurrRect.Right - x) + Sqr(y - CurrRect.Top));
          gkRadial: Coef := Sqrt(Sqr(((CurrRect.Right - CurrRect.Left) shr 1) - x) + Sqr(y - ((CurrRect.Bottom - CurrRect.Top) shr 1)));
          gkDoubleHorz: Coef := Abs(x - MaxDist - CurrRect.Left);
          gkDoubleVert: Coef := Abs(y - MaxDist - CurrRect.Top);
      end;

      if (Region > 0) then
      begin
        if PtInRegion(Region, x, y) then
        begin
          // Methode Lazarus
          //CurColor.red:=Byte(Round(Dep[0] + (Dif[0] * Coef)));;
          //CurColor.green:=Byte(Round(Dep[1] + (Dif[1] * Coef)));
          //CurColor.blue:=Byte(Round(Dep[2] + (Dif[2] * Coef)));
          //CurColor.alpha:= 255;
          //TempIntfImg.Colors[x,y]:=CurColor;

           Bitmap.Canvas.Pixels[x,y] :=  RGB(Byte(Round(Dep[0] + (Dif[0] * Coef))),
                                             Byte(Round(Dep[1] + (Dif[1] * Coef))),
                                             Byte(Round(Dep[2] + (Dif[2] * Coef))));
          // Methode Delphi
          //RowDest^[x].rgbtRed := Byte(Round(Dep[0] + (Dif[0] * Coef)));
          //RowDest^[x].rgbtGreen := Byte(Round(Dep[1] + (Dif[1] * Coef)));
          //RowDest^[x].rgbtBlue := Byte(Round(Dep[2] + (Dif[2] * Coef)));
        end
        else
        begin
          //CurColor.red:=GetRValue(C3);
          //CurColor.green:=GetGValue(C3);
          //CurColor.blue:=GetBValue(C3);
          //CurColor.alpha:= 255;
          //TempIntfImg.Colors[x,y]:=CurColor;
          Bitmap.Canvas.Pixels[x,y] :=  C3;
          //RowDest^[x].rgbtRed := GetRValue(C3);
          //RowDest^[x].rgbtGreen := GetGValue(C3);
          //RowDest^[x].rgbtBlue := GetBValue(C3);
        end;
      end
      else
      begin
        //CurColor.red:=Byte(Round(Dep[0] + (Dif[0] * Coef)));;
        //CurColor.green:=Byte(Round(Dep[1] + (Dif[1] * Coef)));
        //CurColor.blue:=Byte(Round(Dep[2] + (Dif[2] * Coef)));
        //CurColor.alpha:= 255;
        //TempIntfImg.Colors[x,y]:=CurColor;
        Bitmap.Canvas.Pixels[x,y] :=  RGB(Byte(Round(Dep[0] + (Dif[0] * Coef))),
                                          Byte(Round(Dep[1] + (Dif[1] * Coef))),
                                          Byte(Round(Dep[2] + (Dif[2] * Coef))));
        //RowDest^[x].rgbtRed := Byte(Round(Dep[0] + (Dif[0] * Coef)));
        //RowDest^[x].rgbtGreen := Byte(Round(Dep[1] + (Dif[1] * Coef)));
        //RowDest^[x].rgbtBlue := Byte(Round(Dep[2] + (Dif[2] * Coef)));
      end;
    end;
  end;

  //Bitmap.LoadFromIntfImage(TempIntfImg);
  //TempIntfImg.CreateBitmaps(ImgHandle,ImgMaskHandle,false);
  //Bitmap.Handle:=ImgHandle;
  //Bitmap.MaskHandle:=ImgMaskHandle;
  //TempIntfImg.Free;
end;


//==============================================================================
//==============================================================================
// 1. TBZComponentProperty
//==============================================================================

//==== PRIVATE =================================================================

//==== PROTECTED ===============================================================

{**-----------------------------------------------------------------------------
 * @Name        : Change
 * @Description : Lance notre événement de notification si assigné
 -----------------------------------------------------------------------------**}
procedure TBZComponentProperty.Change(AData: PtrInt);
begin
  if Assigned(FOnChange) then
    FOnChange(Self,AData);
end;

//==== PUBLIC ==================================================================

{**-----------------------------------------------------------------------------
 * @Name        : Create
 * @Description : Creation de notre conteneur de propriétés interactif
 -----------------------------------------------------------------------------**}
constructor TBZComponentProperty.Create(AControl: TObject);
begin
  //inherited Create;
  FControl := AControl;
end;


//==============================================================================
//==============================================================================
// 2. TBZButtonStateStyle
//==============================================================================

//==== PRIVATE =================================================================

{**------------------------------------------------------------------------------
 * Setters pour les differentes propriétés
------------------------------------------------------------------------------**}
procedure TBZButtonStateStyle.setGradient(Avalue:Boolean);
begin
  if FGradient = AValue then exit;
  FGradient := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setGradientKind(AValue:TBZButtonGradientKind);
begin
  if FGradientKind = AValue then exit;
  FGradientKind := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setShape(AValue : TBZButtonShape);
begin
  if FShape = AValue then exit;
  FShape := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setRoundingType(AValue : TBZButtonRoundingType);
begin
  if FRoundingType = AValue then exit;
  FRoundingType := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setRoundX(AValue:Integer);
begin
  if FRoundX = AValue then exit;
  FRoundX := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setRoundY(AValue:Integer);
begin
  if FRoundY = AValue then exit;
  FRoundY := AValue;
  Change;
end;

//procedure TBZButtonStateStyle.setShadow(Avalue:Boolean);
//begin
//  if FShadow = AValue then exit;
//  FShadow := AValue;
//  Change;
//end;
//
//procedure TBZButtonStateStyle.setShadowColor(AValue:TColor);
//begin
//  if FShadowColor = AValue then exit;
//  FShadowColor := AValue;
//  Change;
//end;
//
//procedure TBZButtonStateStyle.setShadowOffsetX(AValue : Integer);
//begin
//  if FShadowOffsetX = AValue then exit;
//  FShadowOffsetX := AValue;
//  Change;
//end;
//
//procedure TBZButtonStateStyle.setShadowOffsetY(AValue : Integer);
//begin
//  if FShadowOffsetY = AValue then exit;
//  FShadowOffsetY := AValue;
//  Change;
//end;

procedure TBZButtonStateStyle.setColorFrom(AValue:TColor);
begin
  if FColorFrom = AValue then exit;
  FColorFrom := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setColorTo(AValue:TColor);
begin
  if FColorTo = AValue then exit;
  FColorTo := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setBorderTopColor(AValue:TColor);
begin
  if FBorderTopColor = AValue then exit;
  FBorderTopColor := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setBorderLeftColor(AValue:TColor);
begin
  if FBorderLeftColor = AValue then exit;
  FBorderLeftColor := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setBorderBottomColor(AValue:TColor);
begin
  if FBorderBottomColor = AValue then exit;
  FBorderBottomColor := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setBorderRightColor(AValue:TColor);
begin
  if FBorderRightColor = AValue then exit;
  FBorderRightColor := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setCaptionStyle(AValue:TBZButtonCaptionStyle);
begin
  if FCaptionStyle = AValue then exit;
  FCaptionStyle := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setCaptionShadowColor(AValue:TColor);
begin
  if FCaptionShadowColor = AValue then exit;
  FCaptionShadowColor := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setCaptionShadowOffsetX(AValue : Integer);
begin
  if FCaptionShadowOffsetX = AValue then exit;
  FCaptionShadowOffsetX := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setCaptionShadowOffsetY(AValue : Integer);
begin
  if FCaptionShadowOffsetY = AValue then exit;
  FCaptionShadowOffsetY := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setCaptionShadow(Avalue:Boolean);
begin
  if FCaptionShadow = AValue then exit;
  FCaptionShadow := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setAlignment(AValue:TAlignment);
begin
  if FAlignment = AValue then exit;
  FAlignment := AValue;
  Change;
end;

procedure TBZButtonStateStyle.SetGlyph(const AValue: TPicture);
begin
  if FGlyph = AValue then Exit;
  FGlyph := AValue;
  Change;
end;

procedure TBZButtonStateStyle.SetGlyphTransparent(const AValue: boolean);
begin
  if FGlyphTransparent = AValue then Exit;
  FGlyphTransparent := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setGlyphLayout(const AValue: TBZButtonGlyphLayout);
begin
  if FGlyphLayout = AValue then Exit;
  FGlyphLayout := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setShowArrow(const AValue : Boolean);
begin
  if FShowArrow = AValue then Exit;
  FShowArrow := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setArrowDir(const AValue : TBZButtonArrowDirection);
begin
  if FArrowDir = AValue then Exit;
  FArrowDir := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setArrowLayout(const AValue : TBZButtonArrowLayout);
begin
  if FArrowLayout = AValue then Exit;
  FArrowLayout := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setArrowWidth(const AValue : Integer);
begin
  if FArrowWidth = AValue then Exit;
  FArrowWidth := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setArrowHeight(const AValue : Integer);
begin
  if FArrowHeight = AValue then Exit;
  FArrowHeight := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setArrowColor(const AValue : TColor);
begin
  if FArrowColor = AValue then Exit;
  FArrowColor := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setArrowBorderColor(const AValue : TColor);
begin
  if FArrowBorderColor = AValue then Exit;
  FArrowBorderColor := AValue;
  Change;
end;

procedure TBZButtonStateStyle.setArrowBorderWidth(const AValue : TBZButtonBorderWidth);
begin
  if FArrowBorderWidth = AValue then Exit;
  FArrowBorderWidth := AValue;
  Change;
end;

//==== PROTECTED ===============================================================

//==== PUBLIC ==================================================================
{**-----------------------------------------------------------------------------
 * @Name        : Create
 * @Description : Creation de notre conteneur de propriétés d'état
 -----------------------------------------------------------------------------**}
constructor TBZButtonStateStyle.Create(AControl: TObject);
begin
//  inherited.Create(AControl); provoque une erreur SIG

  FShape              := bsRect;
  FGradient           := True;
  FGradientKind       := gkVertical;
  FColorFrom          := clSilver;
  FColorTo            := clWhite;

  //FShadow             := False;
  //FShadowColor        := TColor($66333333);
  //FShadowOffsetX      := 1;
  //FShadowOffsetY      := 1;
  FShowArrow          := False;
  FArrowDir           := adToBottom;
  FArrowLayout        := alRight;
  FArrowWidth         := 16;
  FArrowHeight        := 16;
  FArrowColor         := clBlack;
  FArrowBorderColor   := clWhite;
  FArrowBorderWidth   := 1;

  FRoundingType       := rtRounded;
  FRoundX             := 10;
  FRoundY             := 10;

  FBorders            := [bbTop, bbLeft, bbBottom, bbRight];
  FBorderWidth        := 1;
  FBorderTopColor     := clBlack;
  FBorderLeftColor    := clBlack;
  FBorderBottomColor  := clBlack;
  FBorderRightColor   := clBlack;

  FCaptionShadowColor   := clGray;
  FCaptionShadow        := false;
  FCaptionShadowOffsetX := 1;
  FCaptionShadowOffsetY := 1;
  FCaptionStyle         := csDefault;

  FFont               := TFont.Create;
  FFont.Quality       := fqCleartypeNatural;
  FFont.Size          := 10;
  FFont.Color         := clBlack;

  FAlignment          := taCenter;

  FGlyph              := TPicture.Create;
  FGlyphLayout        := glLeft;
  FGlyphTransparent   := true;

end;

{**-----------------------------------------------------------------------------
 * @Name        : Destroy
 * @Description : Destruction de notre composant
 -----------------------------------------------------------------------------**}
destructor TBZButtonStateStyle.Destroy;
begin
  if Assigned(FGlyph) then FGlyph.Free;
  FFont.free;
  inherited Destroy;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Assign
 * @Description : Definir les propriétés depuis un autre objet de même type
 -----------------------------------------------------------------------------**}
procedure TBZButtonStateStyle.Assign(Source: TPersistent);
begin
  if Source is TBZButtonStateStyle then
  begin
    FShape              := TBZButtonStateStyle(Source).FShape;
    FGradient           := TBZButtonStateStyle(Source).FGradient;
    FGradientKind       := TBZButtonStateStyle(Source).FGradientKind;
    FColorFrom          := TBZButtonStateStyle(Source).FColorFrom;
    FColorTo            := TBZButtonStateStyle(Source).FColorTo;

    //FShadow             := TBZButtonStateStyle(Source).FShadow;
    //FShadowColor        := TBZButtonStateStyle(Source).FShadowColor;
    //FShadowOffsetX      := TBZButtonStateStyle(Source).FShadowOffsetX;
    //FShadowOffsetY      := TBZButtonStateStyle(Source).FShadowOffsetY;

    FShowArrow          := TBZButtonStateStyle(Source).FShowArrow;
    FArrowDir           := TBZButtonStateStyle(Source).FArrowDir;
    FArrowLayout        := TBZButtonStateStyle(Source).FArrowLayout;
    FArrowWidth         := TBZButtonStateStyle(Source).FArrowWidth;
    FArrowHeight        := TBZButtonStateStyle(Source).FArrowHeight;
    FArrowColor         := TBZButtonStateStyle(Source).FArrowColor;
    FArrowBorderColor   := TBZButtonStateStyle(Source).FArrowBorderColor;
    FArrowBorderWidth   := TBZButtonStateStyle(Source).FArrowBorderWidth;

    FRoundingType       := TBZButtonStateStyle(Source).FRoundingType;
    FRoundX             := TBZButtonStateStyle(Source).FRoundX;
    FRoundY             := TBZButtonStateStyle(Source).FRoundY;

    FBorders            := TBZButtonStateStyle(Source).FBorders;
    FBorderWidth        := TBZButtonStateStyle(Source).FBorderWidth;
    FBorderTopColor     := TBZButtonStateStyle(Source).FBorderTopColor;
    FBorderLeftColor    := TBZButtonStateStyle(Source).FBorderLeftColor;
    FBorderBottomColor  := TBZButtonStateStyle(Source).FBorderBottomColor;
    FBorderRightColor   := TBZButtonStateStyle(Source).FBorderRightColor;

    FCaptionStyle         := TBZButtonStateStyle(Source).FCaptionStyle;
    FCaptionShadow        := TBZButtonStateStyle(Source).FCaptionShadow;
    FCaptionShadowColor   := TBZButtonStateStyle(Source).FCaptionShadowColor;
    FCaptionShadowOffsetX := TBZButtonStateStyle(Source).FCaptionShadowOffsetX;
    FCaptionShadowOffsetY := TBZButtonStateStyle(Source).FCaptionShadowOffsetY;
    FAlignment            := TBZButtonStateStyle(Source).FAlignment;
    FFont                 := TBZButtonStateStyle(Source).FFont;

    FGlyph              := TBZButtonStateStyle(Source).FGlyph;
    FGlyphLayout        := TBZButtonStateStyle(Source).FGlyphLayout;
    FGlyphTransparent   := TBZButtonStateStyle(Source).FGlyphTransparent;
  end;
  //else
  //  inherited Assign(Source);
end;


//==============================================================================
//==============================================================================
// 3. TBZButtonStyles
//==============================================================================

//==== PRIVATE =================================================================

//==== PROTECTED ===============================================================

{**-----------------------------------------------------------------------------
 * @Name        : DoChangeStyle
 * @Description : Notification des changements des propriétés
 -----------------------------------------------------------------------------**}
procedure TBZButtonStyles.DoChangeStyle(ASender: TObject; AData: PtrInt);
begin
  Change;
end;

//==== PUBLIC ==================================================================
{**-----------------------------------------------------------------------------
 * @Name        : Create
 * @Description : Creation de notre conteneur d'états
 -----------------------------------------------------------------------------**}
constructor TBZButtonStyles.Create(AControl: TObject);
begin
  inherited Create(AControl);

  FNormalStyle   := TBZButtonStateStyle.Create(TObject(self));
  FHoverStyle    := TBZButtonStateStyle.Create(TObject(self));
  FDownStyle     := TBZButtonStateStyle.Create(TObject(self));
  FDisabledStyle := TBZButtonStateStyle.Create(TObject(self));

  FNormalStyle.OnChange   := @DoChangeStyle;
  FHoverStyle.OnChange    := @DoChangeStyle;
  FDownStyle.OnChange     := @DoChangeStyle;
  FDisabledStyle.OnChange := @DoChangeStyle;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Destroy
 * @Description : Destruction de notre composant
 -----------------------------------------------------------------------------**}
destructor TBZButtonStyles.Destroy;
begin
  FDisabledStyle.Free;
  FDownStyle.Free;
  FHoverStyle.Free;
  FNormalStyle.Free;
  inherited Destroy;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Assign
 * @Description : Definir les propriétés depuis un autre objet de même type
 -----------------------------------------------------------------------------**}
procedure TBZButtonStyles.Assign(Source: TPersistent);
begin
  if Source is TBZButtonStyles then
  begin
    FDisabledStyle.Assign(TBZButtonStyles(Source).FDisabledStyle);
    FDownStyle.Assign(TBZButtonStyles(Source).FDownStyle);
    FHoverStyle.Assign(TBZButtonStyles(Source).FHoverStyle);
    FNormalStyle.Assign(TBZButtonStyles(Source).FNormalStyle);
  end;
end;


//==============================================================================
//==============================================================================
// 4. TBZCustomButton
//==============================================================================

//==== PRIVATE =================================================================

{**------------------------------------------------------------------------------
 * Setters pour les differentes propriétés
------------------------------------------------------------------------------**}
procedure TBZCustomButton.SetSpacing(const AValue: integer);
begin
   if FSpacing = AValue then Exit;
   FSpacing := AValue;
   FNeededUpdate := True;
   Paint;
end;

procedure TBZCustomButton.SetMarging(const AValue: integer);
begin
   if FMarging = AValue then Exit;
   FMarging := AValue;
   FNeededUpdate := True;
   Paint;
end;

procedure TBZCustomButton.setState(AValue:TBZButtonState);
begin
   if FState = AValue then Exit;
   FState := AValue;
   Paint;
end;

procedure TBZCustomButton.SetCaption(const AValue: TCaption);
begin
   if FCaption = AValue then Exit;
   FCaption := AValue;
   FNeededUpdate := True;
   Paint;
end;

procedure TBZCustomButton.SetShowCaption(const AValue: Boolean);
begin
   if FShowCaption = AValue then Exit;
   FShowCaption := AValue;
   FNeededUpdate := True;
   Paint;
end;

procedure TBZCustomButton.SetActivated(Const AValue:Boolean);
begin
   if FActivated = AValue then Exit;
   FActivated := AValue;
   if not(FActivated) then
   begin
     FFocusable := False;
     TabStop :=False;
   end;
end;

procedure TBZCustomButton.SetFocusable(AValue: boolean);
begin
  if FFocusable= AValue then Exit;
  FFocusable:= AValue;
  TabStop:= AValue;
end;

procedure TBZCustomButton.SetWordWrap(const AValue: Boolean);
begin
 if FWordWrap = AValue then exit;
 FWordWrap := AValue;
 FNeededUpdate := True;
 Paint;
end;

procedure TBZCustomButton.SetCheckable(const AValue: Boolean);
begin
  if FCheckable = AValue then exit;
  FCheckable := AValue;
end;

procedure TBZCustomButton.SetChecked(const AValue: Boolean);
begin
  if FCheckable then
  begin
    if FChecked = AValue then exit;
    FChecked := AValue;
    FState := bsDown;
    Paint;
  end;
end;

procedure TBZCustomButton.SetShowShortCut(const AValue: Boolean);
begin
  if FShowShortCut = AValue then exit;
  FShowShortCut := AValue;
  FNeededUpdate := True;
  Paint;
end;

procedure TBZCustomButton.SetShortCut(const AValue: TShortCut);
Var
  I : integer;
begin
  if FShortCut = AValue then exit;
  FShortCut := AValue;
  if not (csDesigning in ComponentState) then
  begin
    if Assigned(FFormHotKeyManager) then
    begin
      I := FFormHotKeyManager.HotKeyExist(FShortCut);
      if (I>-1) then
      begin
         FFormHotKeyManager.ChangeHotKey(I,Cardinal(FShortCut));
        //Exception.Create('Un Raccourci identique existe déja');
        //exit;
      end;
      I := FFormHotKeyManager.AddHotKey(Cardinal(FShortCut));
      if Assigned(FOnClick) then
      begin
        FFormHotKeyManager.HotKeys[I].OnExecute:=@DoClick;
        if Not(FFormHotKeyManager.Active) then FFormHotKeyManager.Active := true;
      end;
    end;
  end;
  FShortCutString := ShortCutToText(FShortCut);
  if FShowShortCut then
  begin
    FNeededUpdate := True;
    Paint;
  end;
end;

procedure TBZCustomButton.SetPopUpMenuLayout(const AVAlue : TBZButtonPopUpMenuLayout);
begin
  if FPopupMenuLayout = AValue then exit;
  FPopupMenuLayout := AValue;
end;

procedure TBZCustomButton.SetClick(AEvent:TNotifyEvent);
var
  I : Integer;
begin
  if FOnClick = AEvent then Exit;
  FOnClick := AEvent;
  if not (csDesigning in ComponentState) then
  begin
    if (FShortCut<>scNone) and assigned(FFormHotKeyManager) then
    begin
      I := FFormHotKeyManager.HotKeyExist(FShortCut);
      if (I>-1) then
      begin
        FFormHotKeyManager.HotKeys[I].OnExecute:=@DoClick;
        if Not(FFormHotKeyManager.Active) then FFormHotKeyManager.Active := true;
      end
      else
      begin
        I := FFormHotKeyManager.AddHotKey(Cardinal(FShortCut));
        FFormHotKeyManager.HotKeys[I].OnExecute:=@DoClick;
        if Not(FFormHotKeyManager.Active) then FFormHotKeyManager.Active := true;
      end;
    end;
 end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : DoClick
 * @Description : Lance l'action de notre Bouton
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.DoClick(Sender:TObject);
begin
  if Assigned(FOnclick) then FOnClick(Sender);
end;

{**-----------------------------------------------------------------------------
 * @Name        : GetPopupMenuWidth
 * @Description : Renvoi la largeur d'un PopupMenu
 -----------------------------------------------------------------------------**}
function TBZCustomButton.GetPopupMenuWidth(APopupMenu:TPopupMenu):integer;
var
  i,j: Integer;
  maxWidth, wWidth : Integer;
begin
 maxWidth := 0;
 j:=APopupMenu.items.count-1;
 for i:=0 to J do
 begin
   wWidth := abs(self.Canvas.TextWidth(APopupMenu.Items[i].Caption));
   if (wWidth > maxWidth) then maxWidth := wWidth;
 end;
 Result:=maxWidth + 70;
end;

{**-----------------------------------------------------------------------------
 * @Name        : GetPopupMenuHeight
 * @Description : Renvoi la hauteur d'un PopupMenu
 -----------------------------------------------------------------------------**}
function TBZCustomButton.GetPopupMenuHeight(APopupMenu:TPopupMenu):integer;
var
  i,j: Integer;
  maxHeight : Integer;
begin
 maxHeight := 0;
 j:=APopupMenu.items.count-1;
 for i:=0 to J do
 begin
   if (APopupMenu.Items[i].Caption = '-') then
      maxHeight := maxHeight + (abs(self.Canvas.TextHeight('H')))
   else
      maxHeight := maxHeight + (abs(self.Canvas.TextHeight(APopupMenu.Items[i].Caption)));
 end;
 Result:=maxHeight + (APopupMenu.items.count*2)-2;
end;

{**-----------------------------------------------------------------------------
 * @Name        : getArrowBoundsRect
 * @Description : Calcul la position et dimension de la flêche
 -----------------------------------------------------------------------------**}
Function TBZCustomButton.getArrowBoundsRect(AArrowLayout: TBZButtonArrowLayout;AWidth, AHeight :integer;var AButtonRect:TRect;ABorderWidth:Integer) : TRect;
var
  rRect : TRect;
begin
  rRect.Top    :=((AButtonRect.Bottom - AButtonRect.Top) div 2)-AHeight div 2;
  rRect.Bottom := rRect.Top + AHeight;
  case AArrowLayout of
    alLeft  :
    begin
      rRect.Left  := AButtonRect.Left + FMarging + ABorderWidth;
      rRect.Right := rRect.Left+AWidth;
     // AButtonRect.Left := rRect.Right+FSpacing;
    end;
    alRight :
    begin
      rRect.Left := AButtonRect.Right  - AWidth - ABorderWidth - FMarging;
    //  AButtonRect.Right := rRect.Left-FSpacing;
      rRect.Right := rRect.Left+AWidth;
    end;
  end;
  Result := rRect;
end;

{**-----------------------------------------------------------------------------
 * @Name        : getShortCutBoundsRect
 * @Description : Calcul la position et dimension du Raccourcis clavier
 -----------------------------------------------------------------------------**}
Function TBZCustomButton.getShortCutBoundsRect(ACanvas:TCanvas;AButtonRect:TRect;ABorderWidth:Integer) : TRect;
var
  rRect : TRect;
  tw : Integer;

begin
  rRect.Top    :=AButtonRect.Top;
  rRect.Left   :=AButtonRect.Left + ABorderWidth + FMarging;
  rRect.Bottom :=AButtonRect.Bottom;
  rRect.Right  :=AButtonRect.Right - ABorderWidth - FMarging ;
  if FShowShortCut and (FShortCut<>scNone) then
  begin
    ACanvas.Font.Size := 7;
    tw := ACanvas.TextWidth(FShortCutString);

    rRect.Top    := AButtonRect.Top;
    rRect.Left   := rRect.Right - tw - FSpacing;
    rRect.Bottom := AButtonRect.Bottom;
  end;
  result:= rRect;
end;

{**-----------------------------------------------------------------------------
 * @Name        : getGlyphBoundsRect
 * @Description : Calcul la position et dimension de l'icone
 -----------------------------------------------------------------------------**}
Function TBZCustomButton.getGlyphBoundsRect(AGlyph:TPicture;AGlyphLayout:TBZButtonGlyphLayout; AButtonRect,AShortCutRect:TRect;ABorderWidth:Integer) : TRect;
var
  rRect : TRect;
  wPicLeft, wPicTop : Integer;

begin
  wPicLeft := 0;
  wPicTop  := 0;
  if assigned(AGlyph.Graphic) then
  begin
    case AGlyphLayout of
      glLeft  : wPicLeft := AButtonRect.Left + FMarging + ABorderWidth;
      glRight :
      begin
        if FShowShortCut and (FShortCut<>scNone) then
          wPicLeft := AShortCutRect.Left -  FSpacing - AGlyph.Graphic.Width
        else
          wPicLeft := AButtonRect.Right  - AGlyph.Graphic.Width - ABorderWidth - FMarging;
      end;
    end;
    wPicTop := (self.Height  div 2) - (AGlyph.Height div 2);
    rRect.Top    := wPicTop;
    rRect.Left   := wPicLeft;
    rRect.Bottom := wPicTop + AGlyph.Height;
    rRect.Right  := wPicLeft + AGlyph.Width;
  end
  else
  begin
    rRect := AButtonRect;
  end;
  result:= rRect;
end;

{**-----------------------------------------------------------------------------
 * @Name        : getCaptionBoundsRect
 * @Description : Calcul la position et dimension du Caption
 -----------------------------------------------------------------------------**}
Function TBZCustomButton.getCaptionBoundsRect(AGlyph:TPicture;AGlyphLayout:TBZButtonGlyphLayout;AbuttonRect, AGlyphRect, AShortCutRect:TRect; ABorderWidth : Integer) : TRect;
var
  rRect : TRect;
  cleft, cRight : Integer;
begin
  if Assigned(AGlyph.Graphic) then
  begin
    Case AGlyphLayout of
      glLeft :
      begin
         cleft := AGlyphRect.Right + FSpacing;
         if FShowShortCut and (FShortCut<>scNone) then
           cRight := AShortCutRect.Left - FSpacing
         else
           cRight := AShortCutRect.Right;
      end;
      glRight :
      begin
        cLeft := AButtonRect.Left + FMarging + ABorderWidth;
        cRight := AGlyphRect.Left - FSpacing;
      end;
    end;
  end
  else
  begin
    cLeft := AButtonRect.Left + FMarging + ABorderWidth;
    if FShowShortCut and (FShortCut<>scNone) then
     cRight := AShortCutRect.Left - FSpacing
    else
     cRight := AShortCutRect.Right;
  end;
  rRect.Top    := AButtonRect.Top;
  rRect.Left   := cLeft;
  rRect.Bottom := AButtonRect.Bottom;
  rRect.Right  := cRight;
  result:= rRect;
end;

{**-----------------------------------------------------------------------------
 * @Name        : DrawBackground
 * @Description : Dessine le fond de notre bouton
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.DrawBackground(ABmp:TBitmap;AButtonRect:TRect;AStyle: TBZButtonStateStyle;Var PolyArray:PPoint; Var APolyCount:Byte; Var Region : HRGN);
var
  isGrad : Boolean;
  gradKind : TBZButtonGradientKind;
  colorF,colorT : TColor;
  bShape : TBZButtonShape;
  bRounding : TBZButtonRoundingType;
  bRoundX, bRoundY : integer;
  x1,y1,x2,y2 : Integer;
  nbPoints : byte;

begin
  bShape := AStyle.Shape;
  isGrad := AStyle.Gradient;
  ColorF := AStyle.ColorFrom;
  ColorT := AStyle.ColorTo;
  gradKind :=AStyle.GradientKind;
  bRounding :=AStyle.Rounding;
  bRoundX := AStyle.RoundX;
  bRoundY := AStyle.RoundX;

  Case bShape of
    //bsRect :
    //begin
    //
    //end;
    bsRoundRect :
    begin
      if bRounding = rtRounded then
      begin
        if bRoundX = -1 then bRoundX := ABmp.Height div 2;
        if bRoundY = -1 then bRoundY := ABmp.Height div 2;
        if bRoundX > (ABmp.Width div 2) then bRoundX := ABmp.Width div 2;
        if bRoundY > (ABmp.Height div 2) then bRoundY := ABmp.Height div 2;

        Region := CreateRoundRectRgn(0,0,ABmp.Width,ABmp.Height,bRoundX,bRoundY);
      end
      else if bRounding = rtSquared then
      begin
        if bRoundX = -1 then bRoundX := ABmp.Height div 4;
        if bRoundY = -1 then bRoundY := ABmp.Height div 4;
        if bRoundX > (ABmp.Width div 2) then bRoundX := ABmp.Width div 2;
        if bRoundY > (ABmp.Height div 2) then bRoundY := ABmp.Height div 2;

        x1 := bRoundX;
        x2 := ABmp.width - bRoundX;
        y1 := bRoundY;
        y2 := ABmp.Height-bRoundY;
        if (x1 = x2) and (y1 = y2) then
        begin
          nbPoints := 4;
          getMem(PolyArray,nbPoints*SizeOf(TPoint));

          PolyArray[0].x:= x1;
          PolyArray[0].y:= 0;

          PolyArray[1].x:= ABmp.width-1;
          PolyArray[1].y:= y1;

          PolyArray[2].x:= x1;
          PolyArray[2].y:= ABmp.Height-1;

          PolyArray[3].x:= 0;
          PolyArray[3].y:= y1;
          APolyCount:=nbPoints;
        end
        else if (x1=x2) and (y1<>y2) then
        begin
          nbPoints := 6;
          getMem(PolyArray,nbPoints*SizeOf(TPoint));

          PolyArray[0].x:= x1;
          PolyArray[0].y:= 0;

          PolyArray[1].x:= ABmp.width-1;
          PolyArray[1].y:= y1;

          PolyArray[2].x:= ABmp.width-1;
          PolyArray[2].y:= y2;

          PolyArray[3].x:= x1;
          PolyArray[3].y:= ABmp.Height-1;

          PolyArray[4].x:= 0;
          PolyArray[4].y:= y2;

          PolyArray[5].x:= 0;
          PolyArray[5].y:= y1;
          APolyCount:=nbPoints;
        end
        else if (x1<>x2) and (y1=y2) then
        begin
          nbPoints := 6;
          getMem(PolyArray,nbPoints*SizeOf(TPoint));

          PolyArray[0].x:= x1;
          PolyArray[0].y:= 0;

          PolyArray[1].x:= x2;
          PolyArray[1].y:= 0;

          PolyArray[2].x:= ABmp.width-1;
          PolyArray[2].y:= y1;

          PolyArray[3].x:= x2;
          PolyArray[3].y:= ABmp.Height-1;

          PolyArray[4].x:= x1;
          PolyArray[4].y:= ABmp.Height-1;

          PolyArray[5].x:= 0;
          PolyArray[5].y:= y1;
          APolyCount:=nbPoints;
        end
        else
        begin
          nbPoints := 8;
          getMem(PolyArray,nbPoints*SizeOf(TPoint));

          PolyArray[0].x:= x1;
          PolyArray[0].y:= 0;

          PolyArray[1].x:= x2;
          PolyArray[1].y:= 0;

          PolyArray[2].x:= ABmp.width-1;
          PolyArray[2].y:= y1;

          PolyArray[3].x:= ABmp.width-1;
          PolyArray[3].y:= y2;

          PolyArray[4].x:= x2;
          PolyArray[4].y:= ABmp.Height-1;

          PolyArray[5].x:= x1;
          PolyArray[5].y:= ABmp.Height-1;

          PolyArray[6].x:= 0;
          PolyArray[6].y:= y2;

          PolyArray[7].x:= 0;
          PolyArray[7].y:= y1;
          APolyCount:=nbPoints;
        end;
        Region := CreatePolygonRgn(PolyArray,nbPoints,WINDING);
        //FreeMem(PolyArray);
      end;
    end;
    bsEllipse :
    begin
      Region := CreateEllipticRgn(0,0,ABmp.Width,ABmp.Height);
    end;
  end;


  if bShape = bsRect then
  begin
    if isGrad then
    begin
      DrawGradient(ABmp,AButtonRect,ColorF,ColorT,gradKind,0);
    end
    else
    begin
      ABmp.Canvas.Brush.Style := bsSolid;
      ABmp.Canvas.Brush.Color := ColorF;
      ABmp.Canvas.FillRect(AButtonRect);
    end;
  end
  else
  begin
    if isGrad then
    begin
      DrawGradient(ABmp,AButtonRect,ColorF,ColorT,gradKind,region);
    end
    else
    begin
      DrawGradient(ABmp,AButtonRect,ColorF,ColorF,gradKind,region);
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : DrawBorders
 * @Description : Dessine les bordures
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.DrawBorders(ACanvas:TCanvas;AStyle: TBZButtonStateStyle;APolyArray:PPoint;APolyCount:Byte);
var
  bShape : TBZButtonShape;
  bRounding : TBZButtonRoundingType;
  bRoundX, bRoundY : integer;
  bdrTop, bdrLeft, bdrBottom, bdrRight : Boolean;
  bdrTopColor, bdrLeftColor : TColor;
  bdrBottomColor, bdrRightColor : TColor;
  bdrWidth : Integer;

begin
  bShape         := AStyle.Shape;
  bRounding      := AStyle.Rounding;
  bRoundX        := AStyle.RoundX;
  bRoundY        := AStyle.RoundX;

  bdrTop         := bbTop in AStyle.Borders;
  bdrLeft        := bbLeft in AStyle.Borders;
  bdrBottom      := bbBottom in AStyle.Borders;
  bdrRight       := bbRight in AStyle.Borders;
  bdrWidth       := AStyle.BorderWidth;
  bdrTopColor    := AStyle.BorderTopColor;
  bdrLeftColor   := AStyle.BorderTopColor;
  bdrBottomColor := AStyle.BorderTopColor;
  bdrRightColor  := AStyle.BorderTopColor;

  ACanvas.Pen.Width := bdrWidth;
  // Top
  if bdrTop then
  begin
    ACanvas.Pen.Color := bdrTopColor;
    case bShape of
      bsRect :
      begin
        ACanvas.MoveTo(0,0);
        ACanvas.LineTo(ACanvas.Width , 0);
      end;
      bsRoundRect :
      begin
        if bRounding = rtRounded then
        begin
          ACanvas.MoveTo((bRoundX div 2),0);
          ACanvas.LineTo(ACanvas.Width-(bRoundX div 2),0);
        end
        else
        begin
          ACanvas.MoveTo(APolyArray[0].x,APolyArray[0].y);
          ACanvas.LineTo(APolyArray[1].x,APolyArray[1].y);
        end;
      end;
      bsEllipse :
      begin
         ACanvas.MoveTo(0,0);
         ACanvas.Arc(0,0,ACanvas.width-1,ACanvas.Height-1,0,ACanvas.Height div 2,0,ACanvas.Height div 2);
      end;
    end;
  end;
  // Left
  if bdrLeft then
  begin
    ACanvas.Pen.Color := bdrLeftColor;
    case bShape of
      bsRect :
      begin
        ACanvas.MoveTo(0,0);
        ACanvas.LineTo(0,ACanvas.Height);
      end;
      bsRoundRect :
      begin
        if bRounding = rtRounded then
        begin
          ACanvas.Arc(bRoundX+2,bRoundY+2,0,0,(bRoundX div 2),0,0,(bRoundY div 2)); //LeftTop
          ACanvas.MoveTo(0,(bRoundY div 2));
          ACanvas.LineTo(0,ACanvas.height-(bRoundY div 2));
          ACanvas.Arc(0,ACanvas.Height,bRoundX+1,ACanvas.Height-(bRoundY+1),0,ACanvas.Height-(bRoundY div 2)-1,(bRoundX div 2)+1,ACanvas.Height-1); //LeftBottom
        end
        else
        begin
          Case APolyCount of
            4 :
            begin
              ACanvas.MoveTo(APolyArray[0].x,APolyArray[0].y);
              ACanvas.LineTo(APolyArray[3].x,APolyArray[3].y);
            end;
            6 :
            begin
              ACanvas.MoveTo(APolyArray[0].x,APolyArray[0].y);
              ACanvas.LineTo(APolyArray[5].x,APolyArray[5].y);
              ACanvas.MoveTo(APolyArray[5].x,APolyArray[5].y);
              ACanvas.LineTo(APolyArray[4].x,APolyArray[4].y);
            end;
            8 :
            begin
              ACanvas.MoveTo(APolyArray[0].x,APolyArray[0].y);
              ACanvas.LineTo(APolyArray[7].x,APolyArray[7].y);
              ACanvas.MoveTo(APolyArray[7].x,APolyArray[7].y);
              ACanvas.LineTo(APolyArray[6].x,APolyArray[6].y);
              ACanvas.MoveTo(APolyArray[6].x,APolyArray[6].y);
              ACanvas.LineTo(APolyArray[5].x,APolyArray[5].y);
            end;
          end;
        end;
      end;
      bsEllipse :
      begin
           // Nothing todo
      end;
    end;
  end;
  // Bottom
  if bdrBottom then
  begin
    ACanvas.Pen.Color := bdrBottomColor;
    case bShape of
      bsRect :
      begin
        ACanvas.MoveTo(0,ACanvas.Height-1);
        ACanvas.LineTo(ACanvas.Width , ACanvas.Height-1);
      end;
      bsRoundRect :
      begin
        if bRounding = rtRounded then
        begin
          ACanvas.MoveTo((bRoundX div 2),ACanvas.Height-1);
          ACanvas.LineTo(ACanvas.Width-(bRoundX div 2),ACanvas.Height-1);
        end
        else
        begin
          Case APolyCount of
            4 :
            begin
              ACanvas.MoveTo(APolyArray[2].x,APolyArray[2].y);
              ACanvas.LineTo(APolyArray[3].x,APolyArray[3].y);
            end;
            6 :
            begin
              ACanvas.MoveTo(APolyArray[3].x,APolyArray[3].y);
              ACanvas.LineTo(APolyArray[4].x,APolyArray[4].y);
            end;
            8 :
            begin
              ACanvas.MoveTo(APolyArray[4].x,APolyArray[4].y);
              ACanvas.LineTo(APolyArray[5].x,APolyArray[5].y);
            end;
          end;
        end;
      end;
      bsEllipse :
      begin
         ACanvas.MoveTo(0,0);
         ACanvas.Arc(0,0,ACanvas.width-1,ACanvas.Height-1,0,ACanvas.Height div 2,ACanvas.Width-1,ACanvas.Height div 2);
      end;
    end;

  end;
  // Right
  if bdrRight then
  begin
    ACanvas.Pen.Color := bdrRightColor;
    case bShape of
      bsRect :
      begin
        ACanvas.MoveTo(ACanvas.Width-1,0);
        ACanvas.LineTo(ACanvas.Width-1 , ACanvas.Height-1);
      end;
      bsRoundRect :
      begin
        if bRounding = rtRounded then
        begin
          ACanvas.Arc(ACanvas.Width-1,1,ACanvas.Width-(bRoundX+1),bRoundY+1,ACanvas.Width-1,bRoundY div 2,ACanvas.Width-(bRoundX div 2),1);   //RightTop
          ACanvas.MoveTo(ACanvas.Width-1,(bRoundY div 2));
          ACanvas.LineTo(ACanvas.Width-1,ACanvas.Height-(bRoundY div 2));
          ACanvas.Arc(ACanvas.Width-1,ACanvas.Height-1,ACanvas.Width-(bRoundX+1),ACanvas.Height-(bRoundY+1),ACanvas.Width-(bRoundX div 2),ACanvas.Height-1,ACanvas.Width-1,ACanvas.Height-(bRoundY div 2)); //RightBottom
        end
        else
        begin
          Case APolyCount of
            4 :
            begin
              ACanvas.MoveTo(APolyArray[1].x,APolyArray[1].y);
              ACanvas.LineTo(APolyArray[2].x,APolyArray[2].y);
            end;
            6 :
            begin
              ACanvas.MoveTo(APolyArray[1].x,APolyArray[1].y);
              ACanvas.LineTo(APolyArray[2].x,APolyArray[2].y);
              ACanvas.MoveTo(APolyArray[2].x,APolyArray[2].y);
              ACanvas.LineTo(APolyArray[3].x,APolyArray[3].y);
            end;
            8 :
            begin
              ACanvas.MoveTo(APolyArray[1].x,APolyArray[1].y);
              ACanvas.LineTo(APolyArray[2].x,APolyArray[2].y);
              ACanvas.MoveTo(APolyArray[2].x,APolyArray[2].y);
              ACanvas.LineTo(APolyArray[3].x,APolyArray[3].y);
              ACanvas.MoveTo(APolyArray[3].x,APolyArray[3].y);
              ACanvas.LineTo(APolyArray[4].x,APolyArray[4].y);
            end;
          end;
        end;
      end;
      bsEllipse :
      begin
       // Nothing Todo
      end;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : DrawBorders
 * @Description : Dessine les bordures du Focus
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.DrawFocusBorders(ACanvas:TCanvas;AStyle: TBZButtonStateStyle;APolyArray:PPoint;APolyCount:Byte);
var
  bShape : TBZButtonShape;
  bRounding : TBZButtonRoundingType;
  bRoundX, bRoundY : integer;
  bdrTop, bdrLeft, bdrBottom, bdrRight : Boolean;
  bdrTopColor, bdrLeftColor : TColor;
  bdrBottomColor, bdrRightColor : TColor;
  bdrWidth,Delta : Integer;

begin
  bShape         := AStyle.Shape;
  bRounding      := AStyle.Rounding;
  bRoundX        := AStyle.RoundX;
  bRoundY        := AStyle.RoundX;

  bdrTop         := True; //bbTop in AStyle.Borders;
  bdrLeft        := True; //bbLeft in AStyle.Borders;
  bdrBottom      := True; //bbBottom in AStyle.Borders;
  bdrRight       := True; //bbRight in AStyle.Borders;
  bdrWidth       := FocusRectWidth;
  bdrTopColor    := FocusRectColor;//AStyle.BorderTopColor;
  bdrLeftColor   := FocusRectColor;//AStyle.BorderTopColor;
  bdrBottomColor := FocusRectColor;//AStyle.BorderTopColor;
  bdrRightColor  := FocusRectColor;//AStyle.BorderTopColor;

  Delta := AStyle.BorderWidth + 2;
  ACanvas.Pen.Width := bdrWidth;
  ACanvas.Pen.Style := FocusRectStyle;
  // Top
  if bdrTop then
  begin
    ACanvas.Pen.Color := bdrTopColor;
    case bShape of
      bsRect :
      begin
        ACanvas.MoveTo(delta,delta);
        ACanvas.LineTo(ACanvas.Width-delta , delta);
      end;
      bsRoundRect :
      begin
        if bRounding = rtRounded then
        begin
          ACanvas.MoveTo((bRoundX div 2),delta);
          ACanvas.LineTo(ACanvas.Width-delta-(bRoundX div 2),delta);
        end
        else
        begin
          // ERREUR SIGSEGV ??????
          ACanvas.MoveTo(APolyArray[0].x+delta,APolyArray[0].y+delta);
          ACanvas.LineTo(APolyArray[1].x-delta,APolyArray[1].y+delta);
        end;
      end;
      bsEllipse :
      begin
         ACanvas.MoveTo(delta,delta);
         ACanvas.Arc(delta,delta,ACanvas.width-1-delta,ACanvas.Height-1-delta,0,(ACanvas.Height-(delta*2)) div 2,0,(ACanvas.Height-(delta*2)) div 2);
      end;
    end;
  end;
  // Left
  if bdrLeft then
  begin
    ACanvas.Pen.Color := bdrLeftColor;
    case bShape of
      bsRect :
      begin
        ACanvas.MoveTo(delta,delta);
        ACanvas.LineTo(delta,ACanvas.Height-delta);
      end;
      bsRoundRect :
      begin
        if bRounding = rtRounded then
        begin
          ACanvas.Arc(bRoundX+2,bRoundY+2,delta,delta,(bRoundX div 2),delta,delta,(bRoundY div 2)); //LeftTop
          ACanvas.MoveTo(delta,(bRoundY div 2));
          ACanvas.LineTo(delta,ACanvas.height-delta-(bRoundY div 2));
          ACanvas.Arc(delta,ACanvas.Height-delta,bRoundX+1,ACanvas.Height-delta-(bRoundY+1),delta,ACanvas.Height-delta-(bRoundY div 2)-1,(bRoundX div 2)+1,ACanvas.Height-1-delta); //LeftBottom
        end
        else
        begin
          Case APolyCount of
            4 :
            begin
              ACanvas.MoveTo(APolyArray[0].x+delta,APolyArray[0].y+delta);
              ACanvas.LineTo(APolyArray[3].x+delta,APolyArray[3].y-delta);
            end;
            6 :
            begin
              ACanvas.MoveTo(APolyArray[0].x+delta,APolyArray[0].y+delta);
              ACanvas.LineTo(APolyArray[5].x+delta,APolyArray[5].y);

              ACanvas.MoveTo(APolyArray[5].x+delta,APolyArray[5].y);
              ACanvas.LineTo(APolyArray[4].x+delta,APolyArray[4].y-delta);
            end;
            8 :
            begin
              ACanvas.MoveTo(APolyArray[0].x+delta,APolyArray[0].y+delta);
              ACanvas.LineTo(APolyArray[7].x+delta,APolyArray[7].y);

              ACanvas.MoveTo(APolyArray[7].x+delta,APolyArray[7].y);
              ACanvas.LineTo(APolyArray[6].x+delta,APolyArray[6].y);

              ACanvas.MoveTo(APolyArray[6].x+delta,APolyArray[6].y);
              ACanvas.LineTo(APolyArray[5].x+delta,APolyArray[5].y-delta);
            end;
          end;
        end;
      end;
      bsEllipse :
      begin
           // Nothing todo
      end;
    end;
  end;
  // Bottom
  if bdrBottom then
  begin
    ACanvas.Pen.Color := bdrBottomColor;
    case bShape of
      bsRect :
      begin
        ACanvas.MoveTo(delta,ACanvas.Height-1-delta);
        ACanvas.LineTo(ACanvas.Width-delta , ACanvas.Height-1-delta);
      end;
      bsRoundRect :
      begin
        if bRounding = rtRounded then
        begin
          ACanvas.MoveTo((bRoundX div 2),ACanvas.Height-1-delta);
          ACanvas.LineTo(ACanvas.Width-delta-(bRoundX div 2),ACanvas.Height-1-delta);
        end
        else
        begin
          Case APolyCount of
            4 :
            begin
              ACanvas.MoveTo(APolyArray[2].x+delta,APolyArray[2].y-delta);
              ACanvas.LineTo(APolyArray[3].x-delta,APolyArray[3].y-delta);
            end;
            6 :
            begin
              ACanvas.MoveTo(APolyArray[3].x+delta,APolyArray[3].y-delta);
              ACanvas.LineTo(APolyArray[4].x-delta,APolyArray[4].y-delta);
            end;
            8 :
            begin
              ACanvas.MoveTo(APolyArray[4].x+delta,APolyArray[4].y-delta);
              ACanvas.LineTo(APolyArray[5].x-delta,APolyArray[5].y-delta);
            end;
          end;
        end;
      end;
      bsEllipse :
      begin
         ACanvas.MoveTo(delta,delta);
         ACanvas.Arc(delta,delta,ACanvas.width-1-delta,ACanvas.Height-1-delta,delta,(ACanvas.Height-(delta*2)) div 2,ACanvas.Width-1-delta,(ACanvas.Height-(delta*2)) div 2);
      end;
    end;

  end;
  // Right
  if bdrRight then
  begin
    ACanvas.Pen.Color := bdrRightColor;
    case bShape of
      bsRect :
      begin
        ACanvas.MoveTo(ACanvas.Width-1-delta,delta);
        ACanvas.LineTo(ACanvas.Width-1-delta , ACanvas.Height-1-delta);
      end;
      bsRoundRect :
      begin
        if bRounding = rtRounded then
        begin
          ACanvas.Arc(ACanvas.Width-1-delta,1,ACanvas.Width-delta-(bRoundX+1),bRoundY+1,ACanvas.Width-1-delta,bRoundY div 2,ACanvas.Width-delta-(bRoundX div 2),delta);   //RightTop
          ACanvas.MoveTo(ACanvas.Width-1-delta,delta+(bRoundY div 2));
          ACanvas.LineTo(ACanvas.Width-1-delta,ACanvas.Height-delta-(bRoundY div 2));
          ACanvas.Arc(ACanvas.Width-1-delta,ACanvas.Height-1-delta,ACanvas.Width-delta-(bRoundX+1),ACanvas.Height-delta-(bRoundY+1),ACanvas.Width-delta-(bRoundX div 2),ACanvas.Height-1-delta,ACanvas.Width-1-delta,ACanvas.Height-delta-(bRoundY div 2)); //RightBottom
        end
        else
        begin
          Case APolyCount of
            4 :
            begin
              ACanvas.MoveTo(APolyArray[1].x-delta,APolyArray[1].y+delta);
              ACanvas.LineTo(APolyArray[2].x-delta,APolyArray[2].y-delta);
            end;
            6 :
            begin
              ACanvas.MoveTo(APolyArray[1].x-delta,APolyArray[1].y+delta);
              ACanvas.LineTo(APolyArray[2].x-delta,APolyArray[2].y);

              ACanvas.MoveTo(APolyArray[2].x-delta,APolyArray[2].y);
              ACanvas.LineTo(APolyArray[3].x-delta,APolyArray[3].y-delta);
            end;
            8 :
            begin
              ACanvas.MoveTo(APolyArray[1].x-delta,APolyArray[1].y+delta);
              ACanvas.LineTo(APolyArray[2].x-delta,APolyArray[2].y);

              ACanvas.MoveTo(APolyArray[2].x-delta,APolyArray[2].y);
              ACanvas.LineTo(APolyArray[3].x-delta,APolyArray[3].y);

              ACanvas.MoveTo(APolyArray[3].x-delta,APolyArray[3].y);
              ACanvas.LineTo(APolyArray[4].x-delta,APolyArray[4].y-delta);
            end;
          end;
        end;
      end;
      bsEllipse :
      begin
       // Nothing Todo
      end;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : DrawArrow
 * @Description : Dessine la flêche
 -----------------------------------------------------------------------------**}
 procedure TBZCustomButton.DrawArrow(ACanvas: TCanvas; ARect: TRect; ADirection: TBZButtonArrowDirection; AColor, ABorderColor: TColor;AABorderWidth:Integer);
 var
   Points : array[0..3] of TPoint;
   cx,cy,i: Integer;
 begin
   //getMem(Points,3*SizeOf(TPoint));


   cx:= Arect.Right  - ARect.Left ;
   cy:= Arect.Bottom  - ARect.Top;

   case ADirection of
     adToTop:
     begin
       Points[0] := Point(0, cy);
       Points[1] := Point(cx, cy);
       Points[2] := Point(cx div 2, 0);
     end;
     adToBottom:
     begin
       Points[0] := Point(0, 0);
       Points[1] := Point(cx, 0);
       Points[2] := Point(cx div 2, cy);
     end;
     adToLeft:
     begin
       Points[0] := Point(cx, 0);
       Points[1] := Point(cx, cy);
       Points[2] := Point(0, cy div 2);
     end;
     adToRight:
     begin
       Points[0] := Point(0, 0);
       Points[1] := Point(0, cy);
       Points[2] := Point(cx, cy div 2);

     end;
   end;
   Points[3] := Points[0];  // Obligatoire sous Lazarus/FPC sinon le polygone n'est pas correctement dessiner
   for i := 0 to 3 do
   with Points[i] do
   begin
     Inc(X, ARect.Left);
     Inc(Y, ARect.Top);
   end;


   ACanvas.Brush.Style:=bsSolid;
   ACanvas.Brush.Color:=AColor;
   ACanvas.Pen.Width:=AABorderWidth;

   ACanvas.Pen.Color:=ABorderColor;

   ACanvas.Polygon(Points);

   ACanvas.Brush.Style:=bsClear;


   //Freemem(Points);
 end;


{**-----------------------------------------------------------------------------
 * @Name        : DrawGlyph
 * @Description : Dessine l'icone si assigné
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.DrawGlyph(ACanvas:TCanvas; AGlyph:TPicture;x,y:Integer;ATransp:Boolean);
begin
  if assigned(AGlyph.Graphic) then
  begin
    AGlyph.Bitmap.Transparent := ATransp;
    ACanvas.Draw(x, y, AGlyph.Graphic);
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : DrawCaption
 * @Description : Dessine un texte
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.DrawCaption(ACanvas:TCanvas;ARect:TRect;AStyle:TBZButtonCaptionStyle;ACaption:String;AFlags:Integer);
var
  ARect2 : TRect;
  AColor : TColor;

begin
  AColor:=ACanvas.Font.Color;
  Case AStyle of
      csDefault :
      begin
        //ACanvas.Font.Color:=cFont.Color;
        DrawText(ACanvas.Handle, PChar(ACaption), Length(ACaption), ARect, AFlags);
      end;
      csBevel :
      begin
        ACanvas.Font.Color:=DarkenColor(AColor,80);
        ARect2.Top    := ARect.Top + 1;
        ARect2.left   := ARect.Left + 1;
        ARect2.Bottom := ARect.Bottom + 1;
        ARect2.Right  := ARect.Right + 1;
        DrawText(ACanvas.Handle,Pchar(ACaption),Length(ACaption),ARect2,AFlags);
        ACanvas.Font.Color:=LightenColor(AColor,80);
        ARect2.Top    := ARect.Top - 1;
        ARect2.left   := ARect.Left - 1;
        ARect2.Bottom := ARect.Bottom - 1;
        ARect2.Right  := ARect.Right - 1;
        DrawText(ACanvas.Handle,Pchar(ACaption),Length(ACaption),ARect2,AFlags);
        ACanvas.Font.Color:=AColor;
        DrawText(ACanvas.Handle, PChar(ACaption), Length(ACaption), ARect, AFlags);
      end;
      csRaised :
      begin
        ACanvas.Font.Color:=LightenColor(AColor,80);
        ARect2.Top    := ARect.Top + 1;
        ARect2.left   := ARect.Left + 1;
        ARect2.Bottom := ARect.Bottom + 1;
        ARect2.Right  := ARect.Right + 1;
        DrawText(ACanvas.Handle,Pchar(ACaption),Length(ACaption),ARect2,AFlags);
        ACanvas.Font.Color:=DarkenColor(AColor,80);
        ARect2.Top    := ARect.Top - 1;
        ARect2.left   := ARect.Left - 1;
        ARect2.Bottom := ARect.Bottom - 1;
        ARect2.Right  := ARect.Right - 1;
        DrawText(ACanvas.Handle,Pchar(ACaption),Length(ACaption),ARect2,AFlags);
        ACanvas.Font.Color:=AColor;
        DrawText(ACanvas.Handle, PChar(ACaption), Length(ACaption), ARect, AFlags);
      end;
      csEmbossed :
      begin
        ACanvas.Font.Color:=LightenColor(AColor,80);
        ARect2.Top    := ARect.Top + 1;
        ARect2.left   := ARect.Left + 1;
        ARect2.Bottom := ARect.Bottom + 1;
        ARect2.Right  := ARect.Right + 1;
        DrawText(ACanvas.Handle,Pchar(ACaption),Length(ACaption),ARect2,AFlags);
        ACanvas.Font.Color:=AColor;
        DrawText(ACanvas.Handle, PChar(ACaption), Length(ACaption), ARect, AFlags);
      end;
      csContour :
      begin
        ARect2.Top    := ARect.Top-1;
        ARect2.left   := ARect.Left;
        ARect2.Bottom := ARect.Bottom- 1;
        ARect2.Right  := ARect.Right;
        DrawText(ACanvas.Handle,Pchar(ACaption),Length(ACaption),ARect2,AFlags);

        ARect2.Top    := ARect.Top+1;
        ARect2.left   := ARect.Left;
        ARect2.Bottom := ARect.Bottom+1;
        ARect2.Right  := ARect.Right;
        DrawText(ACanvas.Handle,Pchar(ACaption),Length(ACaption),ARect2,AFlags);

        ARect2.Top    := ARect.Top;
        ARect2.left   := ARect.Left-1;
        ARect2.Bottom := ARect.Bottom;
        ARect2.Right  := ARect.Right-1;
        DrawText(ACanvas.Handle,Pchar(ACaption),Length(ACaption),ARect2,AFlags);

        ARect2.Top    := ARect.Top;
        ARect2.left   := ARect.Left+1;
        ARect2.Bottom := ARect.Bottom;
        ARect2.Right  := ARect.Right+1;
        DrawText(ACanvas.Handle,Pchar(ACaption),Length(ACaption),ARect2,AFlags);

        ACanvas.Font.Color:=clInfoBK; //Transparent
        DrawText(ACanvas.Handle, PChar(ACaption), Length(ACaption), ARect, AFlags);
      end;
    end;
    ACanvas.Font.Color:=AColor;
end;

{**-----------------------------------------------------------------------------
 * @Name        : DrawButton
 * @Description : Dessine un etat de notre bouton dans un bitmap
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.DrawButton(AState:TBZButtonState; var ABmp:TBitmap);
var
  TempStyle  : TBZButtonStateStyle;
  BackBuffer : TBitmap;
  wCanvas    : TCanvas;
  //BoundsRect,
  ButtonRect,
  GlyphRect,
  ShortCutRect,
  CaptionRect,
  ArrowRect : TRect;
  TempRect : TRect;
  tw,nw,sw,gw : integer;
  ARegion : HRGN;
  APolyArray:PPoint;
  nbPoints : byte;
  cFlags : Integer;

begin
  ARegion := 0;
  nbPoints := 0;
  APolyArray := nil;

  TempStyle := TBZButtonStateStyle.Create(TObject(Self));
  Case AState of
    bsNormal   : TempStyle.Assign(FStyles.NormalStyle);
    bsHover    : TempStyle.Assign(FStyles.HoverStyle);
    bsDown     : TempStyle.Assign(FStyles.DownStyle);
    bsDisabled : TempStyle.Assign(FStyles.DisabledStyle);
  end;

  BackBuffer := TBitmap.Create;
  BackBuffer.SetSize(Self.Width,Self.Height);
  Backbuffer.Canvas.AntialiasingMode:=amOn;

  wCanvas := BackBuffer.Canvas;

  ButtonRect.Top    := 0;
  ButtonRect.Left   := 0;
  ButtonRect.Bottom := Self.Height;
  ButtonRect.Right  := Self.Width;

  //BoundsRect:=ButtonRect;

  if FShareGlyph and (AState<>bsNormal) then TempStyle.Glyph := FStyles.NormalStyle.Glyph;
  // Calcul des positions et dimensions de nos différents éléments
  if TempStyle.ShowArrow then
    ArrowRect    := getArrowBoundsRect(TempStyle.ArrowLayout,TempStyle.ArrowWidth, TempStyle.ArrowHeight,ButtonRect,TempStyle.BorderWidth);

  ShortCutRect := getShortCutBoundsRect(wCanvas,ButtonRect,TempStyle.BorderWidth);
  GlyphRect    := getGlyphBoundsRect(TempStyle.Glyph,TempStyle.GlyphLayout,ButtonRect, ShortCutRect,TempStyle.BorderWidth);
  CaptionRect  := getCaptionBoundsRect(TempStyle.Glyph,TempStyle.GlyphLayout,ButtonRect, GlyphRect, ShortCutRect,TempStyle.BorderWidth);

  wCanvas.Font := TempStyle.Font;

  // Redimensionnement automatique ?
  if FAutoSize then
  begin
    FWordWrap := False;
    tw := wCanvas.TextWidth(FCaption);
    if TempStyle.CaptionShadow then tw:=tw+TempStyle.CaptionShadowOffsetX;
    nw := ButtonRect.Right;
    if TempStyle.ShowArrow then nw := nw + (ArrowRect.Right - ArrowRect.Left);
    if Assigned(TempStyle.Glyph.Graphic) then gw := GlyphRect.Right - GlyphRect.Left else gw:=0;
    if (FShortCut<>scNone) and FShowShortCut then sw := ShortCutRect.Right - ShortCutRect.Left else sw := 0;
    if (nw < (gw+sw+tw)) then
    begin
      nw:= gw+sw+tw + (TempStyle.BorderWidth*2) + (FMarging*2) + (FSpacing*2);
      Width := nw;
      BackBuffer.SetSize(Self.Width,Self.Height);
      ButtonRect.Right  := Width;
      ShortCutRect := getShortCutBoundsRect(wCanvas,ButtonRect,TempStyle.BorderWidth);
      GlyphRect   := getGlyphBoundsRect(TempStyle.Glyph,TempStyle.GlyphLayout,ButtonRect,ShortCutRect,TempStyle.BorderWidth);
      CaptionRect := getCaptionBoundsRect(TempStyle.Glyph,TempStyle.GlyphLayout,ButtonRect, GlyphRect, ShortCutRect,TempStyle.BorderWidth);
    end;
  end;

  // Dessine le fond
  DrawBackground(BackBuffer,ButtonRect,TempStyle,APolyArray,nbPoints,ARegion);
  // Dessine les bordures
  DrawBorders(wCanvas,TempStyle,APolyArray,nbPoints);
  // Dessine la Flêche
  if TempStyle.ShowArrow then
  begin
    //wCanvas.Rectangle(ArrowRect);
//    showMessage('Top : '+inttostr(Arrowrect.Top)+' Left : '+inttostr(Arrowrect.Left)+' Bottom : '+inttostr(Arrowrect.Bottom)+' Right : '+inttostr(Arrowrect.Right));
    DrawArrow(wCanvas,ArrowRect,TempStyle.ArrowDir,TempStyle.ArrowColor,TempStyle.ArrowBorderColor,TempStyle.ArrowBorderWidth);
  end;
  // Dessine l'icone
  DrawGlyph(wCanvas,TempStyle.Glyph, GlyphRect.Left, GlyphRect.Top, TempStyle.GlyphTransparent);

  wCanvas.Brush.Style := bsClear;
  // Dessine le raccourci
  if FShowShortCut and (FShortCut<>scNone) then
  begin
    cFlags:= getCaptionFlags(taRightJustify,False);
    wCanvas.Font.Size:=7;
    wCanvas.Font.Color:=clGray;
    DrawCaption(wCanvas,ShortCutRect,csRaised,FShortCutString,cFlags);
  end;

  // Dessine le Caption
  cFlags:= getCaptionFlags(TempStyle.Alignment,FWordWrap);
  wCanvas.Font := TempStyle.Font;
  if TempStyle.CaptionShadow then
  begin
    TempRect.Top    := CaptionRect.Top + TempStyle.CaptionShadowOffsetY;
    TempRect.Left   := CaptionRect.Left + TempStyle.CaptionShadowOffsetX;
    TempRect.Bottom := CaptionRect.Bottom + TempStyle.CaptionShadowOffsetY;
    TempRect.Right  := CaptionRect.Right + TempStyle.CaptionShadowOffsetX;
    wCanvas.Font.Color := TempStyle.CaptionShadowColor;
    DrawCaption(wCanvas,TempRect,csDefault,FCaption,cFlags);
  end;

  wCanvas.Font.Color := TempStyle.Font.Color;

  DrawCaption(wCanvas,CaptionRect,TempStyle.CaptionStyle,FCaption,cFlags);

  // Le bouton à le focus
  if  Focused and FDrawFocusRect then
  begin
    DrawFocusBorders(wCanvas,FStyles.HoverStyle,APolyArray,nbPoints);
  end;

  //  ABmp.Canvas.Draw(0,0, BackBuffer);
  ABmp.Assign(BackBuffer);
  if APolyArray<>nil then Freemem(APolyArray);
  if ARegion > 0 then deleteObject(ARegion);
  BackBuffer.Free;
  //TempStyle.Free;// --> Bizarre ca Provoque une erreur External SIGSEGV à la fermeture de l'application  ???
end;

//==== PROTECTED ===============================================================

{**-----------------------------------------------------------------------------
 * @Name        : UpdateFocus
 * @Description : Met à jour le focus
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.UpdateFocus(AFocused: Boolean);
var
  lForm: TCustomForm;
begin
  lForm := GetParentForm(Self);
  if lForm = nil then exit;

  if AFocused then
    ActiveDefaultControlChanged(lForm.ActiveControl)
  else
    ActiveDefaultControlChanged(nil);
end;

{**-----------------------------------------------------------------------------
 * @Name        : WMSetFocus
 * @Description : Notre bouton gagne le Focus
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.WMSetFocus(var Message: TLMSetFocus);
begin
  inherited;
  UpdateFocus(True);
  FFocused:=true;
  FNeededUpdate:=True;
  FState := bsHover;
  Invalidate;
end;

{**-----------------------------------------------------------------------------
 * @Name        : WMKillFocus
 * @Description : Notre bouton perd le Focus
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.WMKillFocus(var Message: TLMKillFocus);
begin
  inherited;
  // if no change then exit
  if Message.FocusedWnd <> Handle then
  begin
    UpdateFocus(False);
    FFocused:=False;
    FNeededUpdate:=True;
    FState := bsNormal;
    Invalidate;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Loaded
 * @Description : Initialisation au chargement du bouton
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.Loaded;
Var
  I : Integer;
begin
  inherited;
  if (FShortCut<>scNone) and assigned(FFormHotKeyManager) then
  begin
    I := FFormHotKeyManager.HotKeyExist(FShortCut);
    if (I>-1) then
    begin
      FFormHotKeyManager.HotKeys[I].OnExecute:=@DoClick;
      FFormHotKeyManager.Active:=True;
    end
    else
    begin
      I := FFormHotKeyManager.AddHotKey(Cardinal(FShortCut));
      FFormHotKeyManager.HotKeys[I].OnExecute:=@DoClick;
      FFormHotKeyManager.Active:=True;
    end;
  end;
end;

{**------------------------------------------------------------------------------
 * Setters Herités de TCustomControl
------------------------------------------------------------------------------**}
procedure TBZCustomButton.SetEnabled(Value: Boolean);
begin
   if FEnabled = Value then Exit;
   FEnabled := Value;
   if FEnabled = false then FState := bsDisabled;
   FNeededUpdate := True;
   inherited SetEnabled(Value);
end;

procedure TBZCustomButton.SetAutoSize(Value:Boolean);
begin
  if FAutoSize = Value then exit;
  FAutoSize := Value;
  FNeededUpdate := True;
  inherited setAutoSize(Value);
end;

{**-----------------------------------------------------------------------------
 * @Name        : ChangeBounds
 * @Description : Met à jour les dimmensions notre bouton
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.ChangeBounds(ALeft, ATop, AWidth,AHeight: integer; KeepBase: boolean);
var i:byte;
begin
  for i:=0 to 3 do
  begin
    FBufferState[i].SetSize(AWidth, AHeight);
  end;
  FNeededUpdate := True;
  inherited ChangeBounds(ALeft, ATop, AWidth, AHeight, KeepBase);
end;

{**-----------------------------------------------------------------------------
 * @Name        : Paint
 * @Description : Affiche notre bouton
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.Paint;
begin
  if not HasParent then Exit;
  if FNeededUpdate then
  begin
    DrawButton(bsNormal,FBufferState[0]);
    DrawButton(bsHover,FBufferState[1]);
    DrawButton(bsDown,FBufferState[2]);
    DrawButton(bsDisabled,FBufferState[3]);
    FNeededUpdate:=false;
  end;
  Case FState of
    bsNormal   :
    begin
     //{$ifdef FPC}

      // Pour je ne sais quelle raison FPC n'applique pas la transparence
       // si la couleur Transparente est differente de clInfoBK
       // Sauf si charger depuis un stream ou un fichier
       // De plus le canvas de notre object n'est pas effacé

       //Fmem.Clear;
       //FBufferState[0].SaveToStream(Fmem);
       //Fmem.Position:=0;
       //FBufferState[0].Clear;
       //FBufferState[0].FreeImage;
       //FBufferState[0].LoadFromStream(Fmem);
     //{$endif}
      Canvas.Draw(0,0, FBufferState[0]);
    end;
    bsHover    : Canvas.Draw(0,0, FBufferState[1]);
    bsDown     : Canvas.Draw(0,0, FBufferState[2]);
    bsDisabled : Canvas.Draw(0,0, FBufferState[3]);
  end;

  inherited Paint;
end;

{**-----------------------------------------------------------------------------
 * @Name        : PaintTo
 * @Description : Affiche notre bouton sur un autre Canvas
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.PaintTo(ACanvas: TCanvas; X, Y: Integer);
begin
    Case FState of
    bsNormal   : ACanvas.Draw(x,y, FBufferState[0]);
    bsHover    : ACanvas.Draw(x,y, FBufferState[1]);
    bsDown     : ACanvas.Draw(x,y, FBufferState[2]);
    bsDisabled : ACanvas.Draw(x,y, FBufferState[3]);
  end;
end;

{**------------------------------------------------------------------------------
 * Interactions de notre bouton avec la souris et le clavier
------------------------------------------------------------------------------**}
procedure TBZCustomButton.KeyUp(var Key: Word; Shift: TShiftState);
begin
  if FEnabled and FActivated then
  begin
    if (Key in [VK_RETURN, VK_SPACE]) or IsAccel(Key, Caption)  then
    begin
      if FCheckable then FChecked := not(FChecked);
      Key := 0;
      if not(Assigned(PopupMenu)) then
      begin
         DoClick(self);
      end
      else
      begin
        Case FPopupMenuLayout of
          ppmlLeft   : PopupMenu.PopUp(self.GetControlOrigin.x-GetPopupMenuWidth(PopupMenu),self.GetControlOrigin.y);
          ppmlTop  :
          begin
            //pt.X := self.GetControlOrigin.x;
            //pt.Y := self.GetControlOrigin.y;
            //TrackPopupMenuEx(PopupMenu.Handle, TPM_LEFTALIGN and TPM_BOTTOMALIGN, pt.X, pt.Y, handle, nil);
            // ph :=
             //PopupMenu.items.count * (abs(self.Canvas.TextHeight('H')) + (PopupMenu.items.count * 2))+2;
             PopupMenu.PopUp(self.GetControlOrigin.x,self.GetControlOrigin.y-GetPopupMenuHeight(PopupMenu)-Self.Height);

          end;
          ppmlBottom : PopupMenu.PopUp(self.GetControlOrigin.x,self.GetControlOrigin.Y+Self.Height);
          ppmlRight  : PopupMenu.PopUp(self.GetControlOrigin.x+self.Width,self.GetControlOrigin.y);
        end;
      end;
    end;
  end;
  inherited;
end;

procedure TBZCustomButton.KeyDown(var Key: Word; Shift: TShiftState);
var
  wShortCut: TShortCut;
begin
  if FEnabled and FActivated then
  begin
    if FShortCut<>scNone then
    begin
      wShortCut := KeyToShortCut(Key,Shift);
      if wShortCut <> scNone then
      begin
        if wShortCut = FShortCut then
        begin
          if FCheckable then FChecked := not(FChecked);
          Key := 0;
          DoClick(self);
        end;
      end;
    end;
  end;
 inherited;
end;

procedure TBZCustomButton.DoEnter;
begin
  if FEnabled and FActivated then
  begin
    FState:=bsHover;
    Paint;
    // Invalidate;
    inherited;
  end;
end;

procedure TBZCustomButton.DoExit;
begin
  if FEnabled and FActivated then
  begin
    FState:=bsNormal;
    Paint;
    inherited;
  end;
end;

procedure TBZCustomButton.MouseEnter;
begin
  if FEnabled and FActivated then
  begin
    Cursor:= crHandPoint;
    FState := bsHover;
    Paint;
    inherited;
  end;
end;

procedure TBZCustomButton.MouseMove(Shift: TShiftState;X, Y: Integer);
var
  TempPoint: TPoint;

  function ShiftToState: TBZButtonState;
  begin
    if ssLeft in Shift then
      Result := bsDown
    else
      Result := bsHover;
  end;

begin
  if FEnabled and FActivated then
  begin
    TempPoint := Point(X, Y);
    if PtInRect(Rect(0,0,Width, Height), TempPoint) then
    begin
      FState:= ShiftToState;
      Paint;
    end;
    inherited MouseMove(Shift, X, Y);
  end;
end;

procedure TBZCustomButton.MouseLeave;
begin
  if FEnabled and FActivated then
  begin
    if (FCheckable and not(FChecked)) or not FCheckable and Not(Focused) then
    begin
      FState:= bsNormal;
    end
    else if Focused then
    begin
      FState:= bsHover;
    end
    else
    begin
      FState:= bsDown ;
    end;
    Cursor:=crDefault;
    Paint;
    inherited;
  end;
end;

procedure TBZCustomButton.MouseDown(Button: TMouseButton;Shift: TShiftState; X, Y: Integer);
var
  TempPoint : TPoint;

begin
  if FEnabled and FActivated then
  begin
    TempPoint:= Point(X,Y);

    if PtInRect(Rect(0,0,Width,Height),TempPoint) then
    begin
       FState:=bsDown;
       Paint;
    end
    else
    begin
      FState:=bsNormal;
      if Assigned(PopupMenu) then PopupMenu.Close;
      Paint;
    end;
    inherited;
  end;
end;

procedure TBZCustomButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  TempPoint : TPoint;
  //pt: TPoint;
 // ph : integer;
begin
  if FEnabled and FActivated then
  begin
    TempPoint:= Point(X,Y);

    if PtInRect(Rect(0,0,Width,Height),TempPoint) then
    begin
      if Button = mbLeft then
      begin
        if FCheckable then FChecked:=not(FChecked);
        if (FCheckable and FChecked) or Assigned(PopupMenu) then FState:=bsDown
        else FState:=bsNormal;

        if not(Assigned(PopupMenu)) then
        begin
           // inherited Click;  //plus rapide que de surcharger la procedure "Click" Uniquement pour Delphi

           // Pas besoin d'appeler la procedure Click sous Lazarus, ici celle-ci est gérée en amont par TCustomControl
           // sinon la procedure Click est appelée 2x
           // Sauf si on surcharge celle-ci, la il faut l'appeler
           DoClick(self);
        end
        else
        begin
          Case FPopupMenuLayout of
            ppmlLeft   : PopupMenu.PopUp(self.GetControlOrigin.x-GetPopupMenuWidth(PopupMenu),self.GetControlOrigin.y);
            ppmlTop  :
            begin
              //pt.X := self.GetControlOrigin.x;
              //pt.Y := self.GetControlOrigin.y;
              //TrackPopupMenuEx(PopupMenu.Handle, TPM_LEFTALIGN and TPM_BOTTOMALIGN, pt.X, pt.Y, handle, nil);
              // ph :=
               //PopupMenu.items.count * (abs(self.Canvas.TextHeight('H')) + (PopupMenu.items.count * 2))+2;
               PopupMenu.PopUp(self.GetControlOrigin.x,self.GetControlOrigin.y-GetPopupMenuHeight(PopupMenu)-Self.Height);

            end;
            ppmlBottom : PopupMenu.PopUp(self.GetControlOrigin.x,self.GetControlOrigin.Y+Self.Height);
            ppmlRight  : PopupMenu.PopUp(self.GetControlOrigin.x+self.Width,self.GetControlOrigin.y);
          end;
        end;
      end
      else
      begin
        if FCheckable and FChecked then FState:= bsDown
        else FState := bsNormal;
        if Assigned(PopupMenu) then  PopupMenu.Close;
      end;
      Paint;
    end;
  end;
end;

//==== PUBLIC ==================================================================

{**-----------------------------------------------------------------------------
 * @Name        : Create
 * @Description : Creation de notre bouton personnalisé
 -----------------------------------------------------------------------------**}
constructor TBZCustomButton.Create(AOwner: TComponent);
var i:byte;
begin
  inherited Create(AOwner);
  Self.Parent := TWinControl(AOwner);

  //{$ifdef FPC}  Fmem:=TMemoryStream.Create;    {$endif}

  for i:=0 to 3 do
  begin
    FBufferState[i]:=TBitmap.Create;
    FBufferState[i].SetSize(Self.Width,Self.Height);
    FBufferState[i].PixelFormat := pf32Bit;
    FBufferState[i].Canvas.AntialiasingMode:=amOn;
    FBufferState[i].Transparent:=true;
    FBufferState[i].TransparentColor:=clInfoBK;
  end;

  // Si la Propriété Activated est à False alors le bouton peut s'utiliser comme un TPanel
  ControlStyle := ControlStyle -[csNoStdEvents, csNoFocus,csDoubleClicks, csTripleClicks, csQuadClicks] + [csAcceptsControls];

  FStyles := TBZButtonStyles.Create(self);
  FStyles.OnChange := @DoChangeStyles;

  FState        := bsNormal;
  FShowShortCut := False;
  FPopUpMenuLayout := ppmlTop;


  FMarging      := 2;
  FSpacing      := 4;
  Width         := 80;
  Height        := 24;

  FFocusable      := True;
  TabStop         := True;
  FDrawFocusRect  := True;
  FFocusRectStyle := psDot;
  FFocusRectWidth := 1;
  FFocusRectColor := clGray;

  FActivated    := True;
  FEnabled      := True;
  FAutoSize     := False;
  FWordWrap     := False;
  FNeededUpdate := True;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Destroy
 * @Description : Destruction de notre composant
 -----------------------------------------------------------------------------**}
destructor TBZCustomButton.Destroy;
var i:byte;
begin
  FStyles.Free;
  for i:=3 downto 0 do
  begin
    FBufferState[i].Free;
  end;
  //{$ifdef FPC}Fmem.Free;  {$endif}
  inherited Destroy;
end;

{**-----------------------------------------------------------------------------
 * @Name        : DoChangeStyles
 * @Description : Met à jour les styles notre bouton
 -----------------------------------------------------------------------------**}
procedure TBZCustomButton.DoChangeStyles(ASender: TObject; AData: PtrInt);
begin
  FNeededUpdate:=True;
  Paint;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Focused  (méthode surchargée hérité de TWinControl)
 * @Description : Renvoi TRUE si notre bouton a le focus
 -----------------------------------------------------------------------------**}
function TBZCustomButton.Focused:Boolean;
begin
  result:= FFocused;
end;


//==============================================================================
//==============================================================================

//initialization

//------------------------------------------------------------------------------
//finalization

//==============================================================================
end.
