unit bz_apphotkeymanager;
//==============================================================================
// @NAME    : BZApplicationHotKey
// @VERSION : 0.5.1
// @DATE    : 08/09/2015
// @UPDATED : 09/10/2015
// @PACKAGE : BEANZ
// @DESC    : TBZHotKeyManager est un composant multi-plateforme
//            (WINDOWS/LINUX/MAC) pour gérer et intercepter les raccourcis
//            claviers globaux du systeme d'exploitation.
//            Edition et gestion de la liste des raccourcis dans L'IDE
// @AUTHOR  : J.DELAUNEY Aka BeanzMaster
// @LICENCE : WTFPL
//------------------------------------------------------------------------------
// @NOTE    : Ce composant à uniquement été tester sous environnement WINDOWS 7
//            Il devrait être compatible sous Linux indépendemment de l'interface
//            utilisée Gnome, KDE, XFCE ect....
//            Il devrait être compatible sous Mac seulement et seulement si vous
//            utilisez GTK+ comme interface et la librairie X11 dans vos réalisations.
//            ATTIENTION pour être tester sous Mac en l'etat actuel, vous devrez
//            faire quelques modifications au niveau des directives de compilation
//------------------------------------------------------------------------------
// @TODO    :
//  - A Tester sous environnement Linux (gnome, kde ect..) et Mac
//  - Gérer les raccourcis sous Mac avec l'interface COCOA. CARBONE étant
//    maintenant devenue obsolète.
//
//==============================================================================

{$mode objfpc}{$H+}

interface

//==============================================================================

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs,
  {$IFDEF WINDOWS}
  windows,
  {$ENDIF}
  LCLProc, LCLType, LCLIntf, LResources, LMessages;

//==============================================================================


//==============================================================================
Type
  TOnHotKeyEvent = procedure(Index: Integer) of object;

//==============================================================================
//==== [ TBZHotKeyItem ]========================================================
//==============================================================================
Type
  TBZAppHotKeyItem = class(TCollectionItem)
   private
     FHotKey : TShortCut; //Cardinal;
     FOnExecute : TNotifyEvent;
   public
     constructor Create(ACollection: TCollection); override;
   published
     property HotKey: TShortCut read FHotKey write FHotKey;
     property OnExecute:TNotifyEvent Read FOnExecute Write FOnExecute;
   end;

//==============================================================================
//==== [ TBZHotKeyList ]========================================================
//==============================================================================
   TBZAppHotKeyList = class(TCollection)
   private
     function GetItems(Index: integer): TBZAppHotKeyItem;
     procedure SetItems(Index: integer; AValue: TBZAppHotKeyItem);
   public
     constructor Create;
   public
     function Add: TBZAppHotKeyItem;
     property Items[Index: integer]: TBZAppHotKeyItem read GetItems write SetItems; default;
   end;


//==============================================================================
//==== [ TBZHotKeyManager ]=====================================================
//==============================================================================
type
  TBZAppHotKeyManager = class(TComponent)
  private
    { Private declarations }
    FActive : boolean;
    FInitialized:Boolean;
    FParentForm : TForm;
    FHotKeyList : TBZAppHotKeyList;
    //FOnHotKey : TOnHotKeyEvent; //TNotifyEvent;
    FSaveProc : TShortCutEvent;
    // Lancement de notre evenement à l'interception d'un raccourci
    procedure DoOnHotKey(Index: Integer);
    procedure DoOnApplicationShortCut(var Msg: TLMKey; var Handled: Boolean);
    procedure SetActive(Value: Boolean);



  protected
    { Protected declarations }

     procedure Loaded;override;
    //--------------------------------------------------------------------------
  public
    { Public declarations }

    constructor Create(AOwner: TComponent);override;
    destructor Destroy; override;

    function CreateHotKey(Modifiers, Key: Word): Cardinal;

    function AddHotKey(HotKey: Cardinal): integer;
    function ChangeHotKey(Index: integer; NewHotKey: Cardinal): Integer;
    function RemoveHotKey(HotKey: Cardinal): Boolean;
    function RemoveHotKeyByIndex(Index: Integer): Boolean;
    procedure ClearHotKeys;
    function HotKeyExist(AShortCut:TShortCut): Integer;
  published
    { Published declarations }
    property Active:Boolean read FActive write setActive default false;
    property HotKeys:TBZAppHotKeyList read FHotKeyList write FHotKeyList;

  //  property OnHotKey : TOnHotKeyEvent read FOnHotKey write FOnHotKey;
  end;

//==============================================================================

procedure Register;

//==============================================================================

implementation

//==============================================================================


//==============================================================================

{**-----------------------------------------------------------------------------
 * @Name        :  Register
 * @Description :  Enregistre notre composant dans l'IDE
 -----------------------------------------------------------------------------**}
procedure Register;
begin
  {$I ../../ressources/bz_apphotkeymanager_icon.lrs}
  RegisterComponents('Beanz',[TBZAppHotKeyManager]);
end;

//==============================================================================
//==============================================================================
// Fonctions Internes
//==============================================================================


//==============================================================================
//==============================================================================
// 1. TBZHotKeyItem
//==============================================================================
constructor TBZAppHotKeyItem.Create(ACollection: TCollection);
begin
  if Assigned(ACollection) and (ACollection is TBZAppHotKeyList) then
    inherited Create(ACollection);
end;

//==============================================================================
//==============================================================================
// 2. TBZHotKeyList
//==============================================================================

//==== PRIVATE =================================================================

function TBZAppHotKeyList.GetItems(Index: integer): TBZAppHotKeyItem;
begin
  Result := TBZAppHotKeyItem(inherited Items[Index]);
end;

procedure TBZAppHotKeyList.SetItems(Index: integer; AValue: TBZAppHotKeyItem);
begin
  Items[Index].Assign(AValue);
end;

//==== PROTECTED ===============================================================

//==== PUBLIC ==================================================================
constructor TBZAppHotKeyList.Create;
begin
  inherited Create(TBZAppHotKeyItem);
end;

function TBZAppHotKeyList.Add: TBZAppHotKeyItem;
begin
  Result := inherited Add as TBZAppHotKeyItem;
end;

//==============================================================================
//==============================================================================
// 2. TBZAppHotKeyManager
//==============================================================================

//==== PRIVATE =================================================================


{**-----------------------------------------------------------------------------
 * @Name        : DoOnHotKey
 * @Description : Lance une action lorsque le HotKey est intercepté
 -----------------------------------------------------------------------------**}
procedure TBZAppHotKeyManager.DoOnHotKey(Index: Integer);
begin
  if Assigned(FHotKeyList[Index].OnExecute) then FHotKeyList[Index].OnExecute(self);
end;

{**-----------------------------------------------------------------------------
 * @Name        : DoOnFormShortCut
 * @Description : Handler OnShortCut de la form
 -----------------------------------------------------------------------------**}
procedure TBZAppHotKeyManager.DoOnApplicationShortCut(var Msg: TLMKey; var Handled: Boolean);
var
  wShortCut : TShortCut;
  ShiftState: TShiftState;
  i:integer;
begin
  Handled := False;
  ShiftState := MsgKeyDataToShiftState(Msg.KeyData);
  wShortCut := KeyToShortCut(Msg.CharCode, ShiftState);
  i:=HotKeyExist(wShortCut);
  if i>-1 then
  begin
    DoOnHotKey(i);
    Handled:=true;
  end;
  if assigned(FSaveProc) then
    FSaveProc(MSg,Handled);
end;

{**-----------------------------------------------------------------------------
 * @Name        : SetActive
 * @Description : Active/Désactive l'interception des HotKeys
 *                Est utilisé par la version Linux et n'a aucun effet sous Windows
 -----------------------------------------------------------------------------**}
procedure TBZAppHotKeyManager.SetActive(Value: Boolean);
begin
  if FActive=Value then exit;
  FActive := Value;
  if not (csDesigning in ComponentState) and FInitialized then
  begin
    if FActive  then
    begin
      Application.OnShortcut:=@DoOnApplicationShortCut;
    end
    else
    begin
      Application.OnShortcut:=FSaveProc;
    end
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Loaded
 * @Description : Effectue une action lorsque le composant est chargé
 -----------------------------------------------------------------------------**}
procedure TBZAppHotKeyManager.Loaded;
begin
  inherited;
  if Assigned(FParentForm.OnShortcut) then
    FSaveProc :=Application.OnShortcut
  else FSaveProc:=nil;
  if Active then
  begin
    Application.OnShortcut:=@DoOnApplicationShortCut;
    FInitialized:=true;
  end;
end;

//==== PROTECTED ===============================================================


//==== PUBLIC ==================================================================

{**-----------------------------------------------------------------------------
* @Name        : Create
* @Description : Creation de notre composant
-----------------------------------------------------------------------------**}
constructor TBZAppHotKeyManager.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FInitialized:=False;
  Self.SetSubComponent(True); // add this line if you want to put this class inside other and also be streamed
  FHotKeyList := TBZAppHotKeyList.Create;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Destroy
 * @Description : Destruction de notre composant
 -----------------------------------------------------------------------------**}
destructor TBZAppHotKeyManager.Destroy;
begin
  ClearHotKeys;
  FHotKeyList.free;
  inherited Destroy;
end;

{**-----------------------------------------------------------------------------
 * @Name        :  CreateHotKey
 * @Description :  Creation du raccourcis clavier HotKey
 * @Exemple     :  CreateHotKey(MOD_CONTROL+MOD_ALT,VK_G); // CTRL+ALT+G
 -----------------------------------------------------------------------------**}
function TBZAppHotKeyManager.CreateHotKey(Modifiers, Key: Word): Cardinal;
const
  VK2_SHIFT   =  32;
  VK2_CONTROL =  64;
  VK2_ALT     = 128;
  VK2_WIN     = 256;
var
  hk: Cardinal;
begin
  hk := 0;
  if (Modifiers and MOD_ALT) <> 0 then
    Inc(hk, VK2_ALT);
  if (Modifiers and MOD_CONTROL) <> 0 then
    Inc(hk, VK2_CONTROL);
  if (Modifiers and MOD_SHIFT) <> 0 then
    Inc(hk, VK2_SHIFT);
  if (Modifiers and MOD_WIN) <> 0 then
    Inc(hk, VK2_WIN);
  hk := hk shl 8;
  Inc(hk, Key);
  Result := hk;
end;

{**-----------------------------------------------------------------------------
 * @Name        : AddHotKey
 * @Description : Ajoute un raccourcis clavier à intercepter dans la lsite
 -----------------------------------------------------------------------------**}
function TBZAppHotKeyManager.AddHotKey(HotKey: Cardinal): Integer;
var
  hk: TBZAppHotKeyItem;
begin
  hk := FHotKeyList.Add;
  hk.HotKey := TShortCut(HotKey);
  Result :=FHotKeyList.Count-1;
end;

{**-----------------------------------------------------------------------------
 * @Name        :  ChangeHotKey
 * @Description :  Modifie un raccourcis dans la liste
 -----------------------------------------------------------------------------**}
function TBZAppHotKeyManager.ChangeHotKey(Index: Integer; NewHotKey: Cardinal): Integer;
begin
 result := -1;
 if (Index>=0) and (Index<FHotKeyList.Count) then
 begin
   FHotKeyList[Index].HotKey := TShortCut(NewHotKey);
   result := Index;
 end;

end;

{**-----------------------------------------------------------------------------
 * @Name        : RemoveHotKey
 * @Description : Supprime un raccourci de la liste
 -----------------------------------------------------------------------------**}
function TBZAppHotKeyManager.RemoveHotKey(HotKey: Cardinal): Boolean;
var
  I,J: Integer;
  hk: TBZAppHotKeyItem;
begin
  Result := False;
  J:= FHotKeyList.Count -1;
  for I := 0 to J do
  begin
    hk := FHotKeyList[I];
    if hk.HotKey = TShortCut(HotKey) then
    begin
      FHotKeyList.Delete(I);
      Exit;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : RemoveHotKeyByIndex
 * @Description : Supprime un raccourci de la liste suivant Index
 -----------------------------------------------------------------------------**}
function TBZAppHotKeyManager.RemoveHotKeyByIndex(Index: Integer): Boolean;
begin
  result:=false;
  if (Index>=0) and (Index<FHotKeyList.Count) then
  begin
    FHotKeyList.Delete(Index);
    result:=true;
  end;
end;


{**-----------------------------------------------------------------------------
 * @Name        :  HotKeyExist
 * @Description :  Vérifie si un raccourci est deja inscrit dans le systeme
 *                 Renvoi Vrai si le HotKey peu être utilisé
 -----------------------------------------------------------------------------**}
function TBZAppHotKeyManager.HotKeyExist(AShortCut:TShortCut): Integer;
 var
   I,J: Integer;
 begin
   Result := -1;
   J:=FHotKeyList.Count -1;
   for I := 0 to J do
     if FHotKeyList[I].HotKey = AShortCut then
     begin
       Result :=I;
       Break;
     end;
 end;

{**-----------------------------------------------------------------------------
 * @Name        : ClearHotKeys
 * @Description : Efface la liste des raccourcis
 -----------------------------------------------------------------------------**}
procedure TBZAppHotKeyManager.ClearHotKeys;
var
  I,J: Integer;
begin
  J:=FHotKeyList.Count -1;
  for I := J downto 0 do
  begin
    FHotKeyList.Delete(I);
  end;
  FHotKeyList.Clear;
end;


//==============================================================================
//==============================================================================

//initialization

//------------------------------------------------------------------------------
//finalization

//==============================================================================
end.
