unit bz_HotKeyManager;
//==============================================================================
// @NAME    : BZApplicationHotKey
// @VERSION : 0.5.1
// @DATE    : 08/09/2015
// @UPDATED : 09/10/2015
// @PACKAGE : BEANZ
// @DESC    : TBZHotKeyManager est un composant multi-plateforme
//            (WINDOWS/LINUX/MAC) pour gérer et intercepter les raccourcis
//            claviers globaux du systeme d'exploitation.
//            Edition et gestion de la liste des raccourcis dans L'IDE
// @AUTHOR  : J.DELAUNEY Aka BeanzMaster
// @LICENCE : WTFPL
//------------------------------------------------------------------------------
// @NOTE    : Ce composant à uniquement été tester sous environnement WINDOWS 7
//            Il devrait être compatible sous Linux indépendemment de l'interface
//            utilisée Gnome, KDE, XFCE ect....
//            Il devrait être compatible sous Mac seulement et seulement si vous
//            utilisez GTK+ comme interface et la librairie X11 dans vos réalisations.
//            ATTIENTION pour être tester sous Mac en l'etat actuel, vous devrez
//            faire quelques modifications au niveau des directives de compilation
//------------------------------------------------------------------------------
// @TODO    :
//  - A Tester sous environnement Linux (gnome, kde ect..) et Mac
//  - Gérer les raccourcis sous Mac avec l'interface COCOA. CARBONE étant
//    maintenant devenue obsolète.
//
//==============================================================================

{$mode objfpc}{$H+}

interface

//==============================================================================

uses
  Classes, SysUtils, LResources,
  {$IFDEF WINDOWS}
  windows,
  {$ENDIF}
  {$IFDEF LINUX}
  x,  xlib, gdk2, gdk2x, cthread, cmem,
  {$ENDIF}
  {$IFDEF DARWIN}
    // Il semblrerait qu'en utilisant "NSEvent" il est possible d'intercepter les evenements clavier
    // à la fois sous Carbon et Cocoa. Mais aucune idées comment implémenter ça.
    {$IFDEF CARBON}
    //FPCMacOSAll,
    {$ELSE}
      // Only Carbon Interface is supported
    {$ENDIF}
  {$ENDIF}
  Forms, Controls, Graphics, Dialogs ;

//==============================================================================

{$IFDEF WINDOWS}
const
  WinClassName: string = 'HOTKEYBZAPPLICATIONHOTKEY';
  HotKeyAtomPrefix : string = 'BeanzHotKeyAtom';
{$ENDIF}

//==============================================================================
Type
  TOnHotKeyEvent = procedure(Index: Integer) of object;

//==============================================================================
//==== [ TBZHotKeyItem ]========================================================
//==============================================================================
Type
  TBZHotKeyItem = class(TCollectionItem)
   private
     FHotKey : TShortCut; //Cardinal;
     FKeyIndex : Integer;
     FOnExecute : TNotifyEvent;
   public
     constructor Create(ACollection: TCollection); override;
   published
     property HotKey: TShortCut read FHotKey write FHotKey;
     property Index: Integer read FKeyIndex Write FKeyIndex;
     property OnExecute:TNotifyEvent Read FOnExecute Write FOnExecute;
   end;

//==============================================================================
//==== [ TBZHotKeyList ]========================================================
//==============================================================================
   TBZHotKeyList = class(TCollection)
   private
     function GetItems(Index: integer): TBZHotKeyItem;
     procedure SetItems(Index: integer; AValue: TBZHotKeyItem);
   public
     constructor Create;
   public
     function Add: TBZHotKeyItem;
     property Items[Index: integer]: TBZHotKeyItem read GetItems write SetItems; default;
   end;

{$IFDEF LINUX}
Type TBZHotKeyManager = class;
//==============================================================================
//==== [ TBZXEventListener ]====================================================
//==============================================================================
Type
  TBZXEventListener = class(TThread)
  private
    FApplicationHotKeys : TBZHotKeyManager;
  protected
    procedure Execute; override;
  public
    constructor Create(AAppHotKey:TBZHotKeyManager);
  end;
{$ENDIF}

//==============================================================================
//==== [ TBZHotKeyManager ]=====================================================
//==============================================================================
type
  TBZHotKeyManager = class(TComponent)
  private
    { Private declarations }
    FActive : boolean;
    FHotKeyList : TBZHotKeyList;
    //FOnHotKey : TOnHotKeyEvent; //TNotifyEvent;

    //----------------------------------------------------------------------------
    // Variables pour intercepter les "HotKeys" sous Linux
    {$IFDEF LINUX}
       FXEventListener : TThread;  // Un thread pour intercepter les messages du systeme
    {$ENDIF}

    //--------------------------------------------------------------------------
    // Fonctions pour intercepter les "HotKeys" sous Windows
    {$IFDEF WINDOWS}
      function  CreateAppWindow: boolean;
    {$ENDIF}

    //--------------------------------------------------------------------------
    // Fonctions internes du composant pour construire et detruire les raccourcis
    function DoRegisterHotKey(hk:Cardinal):integer;
    function DoUnregisterHotKey(KeyIndex:integer):boolean;

    // Lancement de notre evenement à l'interception d'un raccourci
    procedure DoOnHotKey(Index: Integer);

    procedure SetActive(Value: Boolean);



  protected
    { Protected declarations }

     procedure Loaded;override;
    //--------------------------------------------------------------------------

    // Fonctions pour intercepter les "HotKeys" sous Linux
    {$IFDEF LINUX}
      procedure WaitForXevent;
    {$ENDIF}

    // Fonctions pour intercepter les "HotKeys" sous Mac
    {$IFDEF DARWIN}
    {$ENDIF}
  public
    { Public declarations }
    {$IFDEF LINUX}
    Display: PDisplay;
    {$ENDIF}

    constructor Create(AOwner: TComponent);override;
    destructor Destroy; override;

    function CreateHotKey(Modifiers, Key: Word): Cardinal;

    function AddHotKey(HotKey: Cardinal): integer;
    function ChangeHotKey(Index: integer; NewHotKey: Cardinal): Integer;
    function RemoveHotKey(HotKey: Cardinal): Boolean;
    function RemoveHotKeyByIndex(Index: Integer): Boolean;
    procedure ClearHotKeys;
    function HotKeyValid(HotKey: Cardinal): Boolean;
    function HotKeyExist(Index: Integer): Integer;
    function FreeHotKey(Index:Integer): Boolean;
  published
    { Published declarations }
    property Active:Boolean read FActive write setActive default false;
    property HotKeys:TBZHotKeyList read FHotKeyList write FHotKeyList;

  //  property OnHotKey : TOnHotKeyEvent read FOnHotKey write FOnHotKey;
  end;

//==============================================================================

procedure SeparateHotKey(HotKey: Cardinal; var Modifiers, Key: Word);

//==============================================================================

procedure Register;

//==============================================================================

implementation

//==============================================================================
var
  //----------------------------------------------------------------------------
  // Variables pour intercepter les "HotKeys" sous Windows
  {$IFDEF WINDOWS}
    HWindow: HWND;
    WindowClassAtom: ATOM; // Renvoyer par RegisterWindowClass en cas de succes.
    WindowClassInfo: WNDCLASSEX; // Structure des infos d'une fenetre
  {$ENDIF}

//==============================================================================

{**-----------------------------------------------------------------------------
 * @Name        :  Register
 * @Description :  Enregistre notre composant dans l'IDE
 -----------------------------------------------------------------------------**}
procedure Register;
begin
  {$I ../../ressources/bz_hotkeymanager_icon.lrs}
  RegisterComponents('Beanz',[TBZHotKeyManager]);
end;

//==============================================================================
//==============================================================================
// Fonctions Internes
//==============================================================================

{$IFDEF WINDOWS}
{**-----------------------------------------------------------------------------
 * @Name        :  WinProc
 * @Description :  Handler pour Intercepter les message WM_HOTKEY
 *                 Nb: Vous pouvez intercepter d'autre messages comme ceux de la
 *                     souris, suffit d'ajouter des conditions au CASE ce qui
 *                     permettrai avec un peu d'ingéniosité de faire des HotKeys
 *                     genre : CRTL + ALT + G + SOURIS_BOUTON_GAUCHE
 -----------------------------------------------------------------------------**}
function WinProc(hw: HWND; uMsg: UINT; wp: WPARAM; lp: LPARAM): LRESULT;stdcall; export;
var
 obj : TBZHotKeyManager;
 idx:integer;
begin
  Result := 0;
  case uMsg of
    WM_HOTKEY :
    begin
        obj := TBZHotKeyManager(GetWindowLongPtr(HWindow,GWL_USERDATA));
        idx:=obj.HotKeyExist(Longint(wp));
        if (idx>-1) and (obj.Active) then obj.DoOnHotKey(idx);
    end
    else
      Result := DefWindowProc(hw, uMsg, wp, lp);
    end;
end;
{$ENDIF}

//==============================================================================
//==============================================================================
// 1. TBZHotKeyItem
//==============================================================================
constructor TBZHotKeyItem.Create(ACollection: TCollection);
begin
  if Assigned(ACollection) and (ACollection is TBZHotKeyList) then
    inherited Create(ACollection);
end;

//==============================================================================
//==============================================================================
// 2. TBZHotKeyList
//==============================================================================

//==== PRIVATE =================================================================

function TBZHotKeyList.GetItems(Index: integer): TBZHotKeyItem;
begin
  Result := TBZHotKeyItem(inherited Items[Index]);
end;

procedure TBZHotKeyList.SetItems(Index: integer; AValue: TBZHotKeyItem);
begin
  Items[Index].Assign(AValue);
end;

//==== PROTECTED ===============================================================

//==== PUBLIC ==================================================================
constructor TBZHotKeyList.Create;
begin
  inherited Create(TBZHotKeyItem);
end;

function TBZHotKeyList.Add: TBZHotKeyItem;
begin
  Result := inherited Add as TBZHotKeyItem;
end;


//==============================================================================
//==============================================================================
// 3. TBZXEventListener
//==============================================================================

//==== PRIVATE =================================================================

//==== PROTECTED ===============================================================
{$IFDEF LINUX}
{**-----------------------------------------------------------------------------
 * @Name        : Execute
 * @Description : Ecoute les evenements
 -----------------------------------------------------------------------------**}
procedure TBZXEventListener.Execute;
begin
  while  Terminated do
  begin
    if not Terminated then
    begin
      Synchronize(@FApplicationHotKeys.WaitForXevent)
    end;
    if Terminated then
      Exit;
  end;
end;

//==== PUBLIC ==================================================================

{**-----------------------------------------------------------------------------
 * @Name        : Create
 * @Description : Creation de notre thread "Ecouteur d'evenements"
 -----------------------------------------------------------------------------**}
constructor TBZXEventListener.Create(AAppHotKey:TBZHotKeyManager);
begin
  inherited Create(True);
  FApplicationHotKeys := AAppHotKey;
end;
{$ENDIF}

//==============================================================================
//==============================================================================
// 2. TBZHotKeyManager
//==============================================================================

//==== PRIVATE =================================================================

{$IFDEF WINDOWS}
{**-----------------------------------------------------------------------------
 * @Name        : CreateAppWindows
 * @Description : Creation d'une fenetre virtuelle avec notre Hook
 -----------------------------------------------------------------------------**}
function  TBZHotKeyManager.CreateAppWindow: boolean;

  function RegisterWindowClass: boolean;
  begin
    WindowClassInfo.cbSize := sizeof(WindowClassInfo);
    WindowClassInfo.Style := 0;
    WindowClassInfo.lpfnWndProc := @WinProc;
    WindowClassInfo.cbClsExtra := 0;
    WindowClassInfo.cbWndExtra := 0;
    WindowClassInfo.hInstance := HInstance;
    WindowClassInfo.hIcon := 0;
    WindowClassInfo.hCursor := 0;
    WindowClassInfo.hbrBackground := 0;
    WindowClassInfo.lpszMenuName := nil;
    WindowClassInfo.lpszClassName := PChar(WinClassName);
    WindowClassInfo.hIconSm := 0;
    WindowClassAtom := RegisterClassEx(WindowClassInfo);
    Result := WindowClassAtom <> 0;
  end;

begin
  Result := False;
  if not RegisterWindowClass then
  begin
     exit;
  end;
  HWindow := CreateWindowEx(WS_EX_NOACTIVATE or WS_EX_TRANSPARENT,
    PChar(WinClassName), PChar(WinClassName), Ws_popup or WS_CLIPSIBLINGS, 0,
    0, 0, 0, 0, 0, HInstance, nil);
  if HWindow <> 0 then
  begin
    ShowWindow(HWindow, SW_HIDE);
    SetWindowLongPtr(HWindow,GWL_USERDATA,PtrInt(Self));
    UpdateWindow(HWindow);
    Result := True;
    Exit;
  end;
end;
{$ENDIF}

{**-----------------------------------------------------------------------------
 * @Name        : DoRegisterHotKey
 * @Description : Ajoute un HotKey Global dans le systeme d'exploitation
 -----------------------------------------------------------------------------**}
function TBZHotKeyManager.DoRegisterHotKey(hk:cardinal):integer;
var
  Modifiers, Key: Word;
  id :integer;
  {$IFDEF LINUX}
  root :PGdkWindow;
  {$ENDIF}
begin
  result:=0;
  Modifiers:=0;
  Key:=0;
  SeparateHotKey(hk, Modifiers, Key);

  {$IFDEF WINDOWS}
  id := GlobalAddAtom(PChar(HotKeyAtomPrefix + IntToStr(hk)));
  RegisterHotKey(HWindow, Longint(id), Modifiers, Key);
  result:=id;
  {$ENDIF}

  {$IFDEF LINUX}
  root := gdk_get_default_root_window ();
  Display := GDK_WINDOW_XDISPLAY (root);
  gdk_error_trap_push;
  XGrabKey (Display, Key,Modifiers,gdk_x11_drawable_get_xid(root), 1,GrabModeAsync, GrabModeAsync);
  gdk_flush;
  Result := (gdk_error_trap_pop) = 0;
  {$ENDIF}

  {$IFDEF DARWIN}
  {$ENDIF}
end;


{**-----------------------------------------------------------------------------
 * @Name        : DoUnRegisterHotKey
 * @Description : Supprime un HotKey Global dans le systeme d'exploitation
 -----------------------------------------------------------------------------**}
function TBZHotKeyManager.DoUnRegisterHotKey(KeyIndex:integer):boolean;
{$IFDEF LINUX}
var
  root :PGdkWindow;
  Modifiers,Key:word;
  hk:Cardinal;
{$ENDIF}
begin
  {$IFDEF WINDOWS}
  result:=UnRegisterHotkey( HWindow, Longint(KeyIndex));
  GlobalDeleteAtom(KeyIndex);
  {$ENDIF}

  {$IFDEF LINUX}
  root := gdk_get_default_root_window ();
  Display := GDK_WINDOW_XDISPLAY (root);
  hk := Cardinal(FHotKeyList[HotKeyExist(KeyIndex)].HotKey);
  SeparateHotKey(hk, Modifiers, Key);
  XUngrabKey (Display, Key, Modifiers, gdk_x11_drawable_get_xid(root));
  {$ENDIF}

  {$IFDEF DARWIN}
  {$ENDIF}
end;

{**-----------------------------------------------------------------------------
 * @Name        : DoOnHotKey
 * @Description : Lance une action lorsque le HotKey est intercepté
 -----------------------------------------------------------------------------**}
procedure TBZHotKeyManager.DoOnHotKey(Index: Integer);
begin
  if Assigned(FHotKeyList[Index].OnExecute) then FHotKeyList[Index].OnExecute(self);
end;

{**-----------------------------------------------------------------------------
 * @Name        : SetActive
 * @Description : Active/Désactive l'interception des HotKeys
 *                Est utilisé par la version Linux et n'a aucun effet sous Windows
 -----------------------------------------------------------------------------**}
procedure TBZHotKeyManager.SetActive(Value: Boolean);
begin
  if FActive <> Value then
  begin
      FActive := Value;
      if not (csDesigning in ComponentState) then
      begin
        if FActive then
        begin
          {$IFDEF LINUX}
          FXEventListener.Suspended:=false;
          {$ENDIF}
        end
        else
        begin
          {$IFDEF LINUX}
          FXEventListener.Suspended:=true;
          {$ENDIF}
        end
      end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Loaded
 * @Description : Effectue une action lorsque le composant est chargé
 -----------------------------------------------------------------------------**}
procedure TBZHotKeyManager.Loaded;
var
  i,j:integer;
 // hk: TBZHotKeyItem;
  id:integer;
begin
  inherited;
  j:= FHotKeyList.Count;
  if J>0 then
  begin
    for i:= 0 to j-1 do
    begin
      //hk:=;
      id:=DoRegisterHotKey(Cardinal(FHotKeyList.Items[i].HotKey));
      if id>0 then
      begin
        FHotKeyList.Items[i].Index := id;
      end;
    end;
  end;
end;

//==== PROTECTED ===============================================================

{$IFDEF LINUX}
{**-----------------------------------------------------------------------------
 * @Name        : WaitForXevent
 * @Description : Attente d'interception des raccourcis sous Linux
 -----------------------------------------------------------------------------**}
procedure TBZHotKeyManager.WaitForXevent;
var
  event : XEvent;
  I,J,state:Integer;
  Modifiers,Key:word;
  xkey : XKeyEvent;
  hk  :TBZHotKeyItem;
begin
  //event := XEvent.Create();
  while (XPending(Display) > 0) do
  begin
   XNextEvent(Display, @event);
   if (event._type = KeyPress) then
   begin
     xkey := XKeyEvent(event.readField('xkey'));
     state := xkey.state and (ShiftMask or ControlMask or Mod1Mask or Mod4Mask);
     J:=FHotKeyList.Count-1;
     for I:=0 to J do
     begin
       hk:=FHotKeyList[I]//;^.HotKey;
       separateHotKey(Cardinal(hk.HotKey),Modifiers,Key);
       if (xkey.keycode = key ) and (state = Modifiers  ) then
       begin
         DoOnHotKey(I);
       end;
     end;
   end;
end;
{$ENDIF}

//==== PUBLIC ==================================================================

{**-----------------------------------------------------------------------------
 * @Name        : Create
 * @Description : Construction de notre composant
 -----------------------------------------------------------------------------**}

{**-----------------------------------------------------------------------------
* @Name        : Create
* @Description : Creation de notre composant
-----------------------------------------------------------------------------**}
constructor TBZHotKeyManager.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Self.SetSubComponent(True); // add this line if you want to put this class inside other and also be streamed
  FHotKeyList := TBZHotKeyList.Create;

  if not (csDesigning in ComponentState) then
    begin
      {$IFDEF WINDOWS}
      CreateAppWindow;
      {$ENDIF}

      {$IFDEF LINUX}
      FXEventListener := TBZXEventListener.Create(self);
      Display := XOpenDisplay(gdk_get_display);
      {$ENDIF}

      {$IFDEF DARWIN}
      {$ENDIF}
    end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : Destroy
 * @Description : Destruction de notre composant
 -----------------------------------------------------------------------------**}
destructor TBZHotKeyManager.Destroy;
begin
  ClearHotKeys;

  {$IFDEF WINDOWS}
  DestroyWindow(HWindow);
  {$ENDIF}

  {$IFDEF LINUX}
  FXEventListener.Terminate;
  FXEventListener.WaitFor;
  FXEventListener.Free;
  XCloseDisplay(Display);
  {$ENDIF}

  {$IFDEF DARWIN}
  {$ENDIF}

  FHotKeyList.free;
  inherited Destroy;
end;

{**-----------------------------------------------------------------------------
 * @Name        :  CreateHotKey
 * @Description :  Creation du raccourcis clavier HotKey
 * @Exemple     :  CreateHotKey(MOD_CONTROL+MOD_ALT,VK_G); // CTRL+ALT+G
 -----------------------------------------------------------------------------**}
function TBZHotKeyManager.CreateHotKey(Modifiers, Key: Word): Cardinal;
const
  VK2_SHIFT   =  32;
  VK2_CONTROL =  64;
  VK2_ALT     = 128;
  VK2_WIN     = 256;
var
  hk: Cardinal;
begin
  hk := 0;
  if (Modifiers and MOD_ALT) <> 0 then
    Inc(hk, VK2_ALT);
  if (Modifiers and MOD_CONTROL) <> 0 then
    Inc(hk, VK2_CONTROL);
  if (Modifiers and MOD_SHIFT) <> 0 then
    Inc(hk, VK2_SHIFT);
  if (Modifiers and MOD_WIN) <> 0 then
    Inc(hk, VK2_WIN);
  hk := hk shl 8;
  Inc(hk, Key);
  Result := hk;
end;

{**-----------------------------------------------------------------------------
 * @Name        : AddHotKey
 * @Description : Ajoute un raccourcis clavier à intercepter dans la lsite
 -----------------------------------------------------------------------------**}
function TBZHotKeyManager.AddHotKey(HotKey: Cardinal): Integer;
var
  hk: TBZHotKeyItem;//PHotKey;
  id:integer;
begin
  // Creation d'un ID Unique
  id:=DoRegisterHotKey(HotKey);
  if id>0 then
  begin
    hk := FHotKeyList.Add;
    hk.HotKey := TShortCut(HotKey);
    hk.Index := id;

    //hk := New(PHotKey);
    //hk^.HotKey := HotKey;
    //hk^.KeyIndex := id;
    //FHotKeyList.Add(hk);

    Result :=FHotKeyList.Count-1;
  end
  else
  begin
    Result := -1;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        :  ChangeHotKey
 * @Description :  Modifie un raccourcis dans la liste
 -----------------------------------------------------------------------------**}
function TBZHotKeyManager.ChangeHotKey(Index: Integer; NewHotKey: Cardinal): Integer;
var
  I,J: Integer;
  hk: TBZHotKeyItem;//PHotKey;
begin
  Result := 0;
  J:=FHotKeyList.Count -1;
  for I := 0 to J do
  begin
    hk := FHotKeyList[I];
    if hk.Index = Index then
    begin
      RemoveHotKeyByIndex(Index);
      Result := AddHotKey(NewHotKey);
      Exit;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : RemoveHotKey
 * @Description : Supprime un raccourci de la liste
 -----------------------------------------------------------------------------**}
function TBZHotKeyManager.RemoveHotKey(HotKey: Cardinal): Boolean;
var
  I,J: Integer;
  hk: TBZHotKeyItem;//PHotKey;
begin
  Result := False;
  J:= FHotKeyList.Count -1;
  for I := 0 to J do
  begin
    hk := FHotKeyList[I];
    if hk.HotKey = TShortCut(HotKey) then
    begin
      Result := FreeHotKey(I);
      FHotKeyList.Delete(I);
      Exit;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        : RemoveHotKeyByIndex
 * @Description : Supprime un raccourci de la liste suivant Index
 -----------------------------------------------------------------------------**}
function TBZHotKeyManager.RemoveHotKeyByIndex(Index: Integer): Boolean;
var
  I,J: Integer;
  hk: TBZHotKeyItem;
begin
  Result := False;
  J:= FHotKeyList.Count -1;
  for I := 0 to J do
  begin
    hk := FHotKeyList[I];
    if hk.Index = Index then
    begin
      Result := FreeHotKey(I);
      FHotKeyList.Delete(I);
      Exit;
    end;
  end;
end;

{**-----------------------------------------------------------------------------
 * @Name        :  HotKeyValid
 * @Description :  Vérifie si un raccourci est deja inscrit dans le systeme
 *                 Renvoi Vrai si le HotKey peu être utilisé
 -----------------------------------------------------------------------------**}
function TBZHotKeyManager.HotKeyValid(HotKey: Cardinal): Boolean;
var
  M, K: Word;
  WasRegistered: Boolean;
  {$IFDEF WINDOWS}
  Atom: Word;
  {$ENDIF}
begin
  {$IFDEF WINDOWS}
  K:=0;
  M:=0;
  Atom := GlobalAddAtom(PChar(HotKeyAtomPrefix + IntToStr(HotKey)));
  SeparateHotKey(HotKey, M, K);
  WasRegistered := UnregisterHotKey(HWindow, Atom);
  if WasRegistered then
  begin
    RegisterHotKey(HWindow, Atom, M, K);
    Result := True;
  end
  else
  begin
    Result := RegisterHotKey(HWindow, Atom, M, K);
    if Result then
      UnregisterHotKey(HWindow, Atom);
  end;
  GlobalDeleteAtom(Atom);
  {$ENDIF}

  {$IFDEF LINUX}
  //@TODO : A Compléter
  result:=true;
  {$ENDIF}

  {$IFDEF DARWIN}
  {$ENDIF}
end;

{**-----------------------------------------------------------------------------
 * @Name        :  HotKeyExist
 * @Description :  Vérifie si un raccourci est deja inscrit dans le systeme
 *                 Renvoi Vrai si le HotKey peu être utilisé
 -----------------------------------------------------------------------------**}
function TBZHotKeyManager.HotKeyExist(Index: Integer): Integer;
 var
   I,J: Integer;
 begin
   Result := -1;
   J:=FHotKeyList.Count -1;
   for I := 0 to J do
     if FHotKeyList[I].Index = Index then
     begin
       Result :=I;
       Break;
     end;
 end;

{**-----------------------------------------------------------------------------
 * @Name        : ClearHotKeys
 * @Description : Efface la liste des raccourcis
 -----------------------------------------------------------------------------**}
procedure TBZHotKeyManager.ClearHotKeys;
var
  I,J: Integer;
//  hk: PHotKey;
begin
  J:=FHotKeyList.Count -1;
  for I := J downto 0 do
  begin
   // hk := PHotKey(FHotKeyList[I]);
    FreeHotKey(I);
    FHotKeyList.Delete(I);
  end;
//  FHotKeyList.Clear;
end;

{**-----------------------------------------------------------------------------
 * @Name        : FreeHotKey
 * @Description : Désinscrit et libère un raccourci
 -----------------------------------------------------------------------------**}
function TBZHotKeyManager.FreeHotKey(Index:Integer): Boolean;
begin
  Result := DoUnregisterHotKey(FHotKeyList[Index].Index);

  //if Result then
  //  Dispose(hk);
end;

//==============================================================================
//==============================================================================

//==============================================================================
// Fonctions Internes
//==============================================================================

{**-----------------------------------------------------------------------------
 * @Name        : SeparateHotKey
 * @Description : Separe les touches normal et les touches de controle du Hotkeys
 *                pour les utiliser avec RegisterHotKey
 -----------------------------------------------------------------------------**}
procedure SeparateHotKey(HotKey: Cardinal; var Modifiers, Key: Word);
{$IFDEF WINDOWS}
const
  VK2_SHIFT   =  32;
  VK2_CONTROL =  64;
  VK2_ALT     = 128;
  VK2_WIN     = 256;
{$ENDIF}
var
  Virtuals: Integer;
  V: Word;
  x: Word;
{$IFDEF LINUX}
  I:Integer;

  function correctModifiers(modifiers:word;flags:integer) :integer;
  var ret:word;
  begin
    ret:=modifiers;
    if ((flags and 1) <> 0) then ret := ret OR LockMask;
    if ((flags and 2) <> 0) then ret := ret OR Mod2Mask;
    if ((flags and 4) <> 0) then ret := ret OR Mod3Mask;
    if ((flags and 8) <> 0) then ret := ret OR Mod5Mask;
    result:=ret;
  end;
{$ENDIF}
begin
  Key := Byte(HotKey);
  x := HotKey shr 8;
  Virtuals := x;

  {$IFDEF WINDOWS}
  V := 0;
  if (Virtuals and VK2_WIN) <> 0 then Inc(V, MOD_WIN);
  if (Virtuals and VK2_ALT) <> 0 then Inc(V, MOD_ALT);
  if (Virtuals and VK2_CONTROL) <> 0 then Inc(V, MOD_CONTROL);
  if (Virtuals and VK2_SHIFT) <> 0 then Inc(V, MOD_SHIFT);
  Modifiers := V;
  {$ENDIF}

  {$IFDEF LINUX}
  V:=Virtuals;
  for i:=0 to 15 do
  begin
    V := correctModifiers(V, i);
  end;
  Modifiers := V;
  {$ENDIF}

  {$IFDEF DARWIN}
  {$ENDIF}
end;

//==============================================================================
//==============================================================================

//initialization

//------------------------------------------------------------------------------
//finalization

//==============================================================================
end.
